/****************************************************************************
** Meta object code from reading C++ file 'UIDebugSCSP.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/qt/ui/UIDebugSCSP.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UIDebugSCSP.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_UIDebugSCSP_t {
    QByteArrayData data[12];
    char stringdata0[180];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UIDebugSCSP_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UIDebugSCSP_t qt_meta_stringdata_UIDebugSCSP = {
    {
QT_MOC_LITERAL(0, 0, 11), // "UIDebugSCSP"
QT_MOC_LITERAL(1, 12, 28), // "on_sbSlotNumber_valueChanged"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 1), // "i"
QT_MOC_LITERAL(4, 44, 22), // "on_pbSaveAsWav_clicked"
QT_MOC_LITERAL(5, 67, 30), // "on_pbSaveSlotRegisters_clicked"
QT_MOC_LITERAL(6, 98, 21), // "on_pbPlaySlot_clicked"
QT_MOC_LITERAL(7, 120, 8), // "notified"
QT_MOC_LITERAL(8, 129, 17), // "audioBufferRefill"
QT_MOC_LITERAL(9, 147, 12), // "stateChanged"
QT_MOC_LITERAL(10, 160, 13), // "QAudio::State"
QT_MOC_LITERAL(11, 174, 5) // "state"

    },
    "UIDebugSCSP\0on_sbSlotNumber_valueChanged\0"
    "\0i\0on_pbSaveAsWav_clicked\0"
    "on_pbSaveSlotRegisters_clicked\0"
    "on_pbPlaySlot_clicked\0notified\0"
    "audioBufferRefill\0stateChanged\0"
    "QAudio::State\0state"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UIDebugSCSP[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x09 /* Protected */,
       4,    0,   52,    2, 0x09 /* Protected */,
       5,    0,   53,    2, 0x09 /* Protected */,
       6,    0,   54,    2, 0x09 /* Protected */,
       7,    0,   55,    2, 0x09 /* Protected */,
       8,    0,   56,    2, 0x09 /* Protected */,
       9,    1,   57,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,

       0        // eod
};

void UIDebugSCSP::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UIDebugSCSP *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->on_sbSlotNumber_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->on_pbSaveAsWav_clicked(); break;
        case 2: _t->on_pbSaveSlotRegisters_clicked(); break;
        case 3: _t->on_pbPlaySlot_clicked(); break;
        case 4: _t->notified(); break;
        case 5: _t->audioBufferRefill(); break;
        case 6: _t->stateChanged((*reinterpret_cast< QAudio::State(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UIDebugSCSP::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_UIDebugSCSP.data,
    qt_meta_data_UIDebugSCSP,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UIDebugSCSP::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UIDebugSCSP::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UIDebugSCSP.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::UIDebugSCSP"))
        return static_cast< Ui::UIDebugSCSP*>(this);
    return QDialog::qt_metacast(_clname);
}

int UIDebugSCSP::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
