/********************************************************************************
** Form generated from reading UI file 'UIYabause.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIYABAUSE_H
#define UI_UIYABAUSE_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QSlider>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_UIYabause
{
public:
    QAction *aFileQuit;
    QAction *aYabauseStart;
    QAction *aYabauseStop;
    QAction *aFileSettings;
    QAction *aEmulationRun;
    QAction *aEmulationPause;
    QAction *aEmulationReset;
    QAction *aToolsTransfer;
    QAction *aFileScreenshot;
    QAction *aEmulationFrameSkipLimiter;
    QAction *aViewFPS;
    QAction *aViewLayerVdp1;
    QAction *aViewFullscreen;
    QAction *aViewLog;
    QAction *aViewDebugMSH2;
    QAction *aViewDebugSSH2;
    QAction *aViewDebugVDP1;
    QAction *aViewDebugVDP2;
    QAction *aViewDebugM68K;
    QAction *aViewDebugSCUDSP;
    QAction *aViewDebugSCSP;
    QAction *aViewDebugMemoryTransfer;
    QAction *aHelpAbout;
    QAction *aViewLayerNBG0;
    QAction *aViewLayerNBG1;
    QAction *aViewLayerNBG2;
    QAction *aViewLayerNBG3;
    QAction *aViewLayerRBG0;
    QAction *aFileSaveState1;
    QAction *aFileSaveState2;
    QAction *aFileSaveState3;
    QAction *aFileSaveState4;
    QAction *aFileSaveState5;
    QAction *aFileSaveState6;
    QAction *aFileSaveState7;
    QAction *aFileSaveState8;
    QAction *aFileSaveState9;
    QAction *aFileLoadState1;
    QAction *aFileLoadState2;
    QAction *aFileLoadState3;
    QAction *aFileLoadState4;
    QAction *aFileLoadState5;
    QAction *aFileLoadState6;
    QAction *aFileLoadState7;
    QAction *aFileLoadState8;
    QAction *aFileLoadState9;
    QAction *aFileSaveStateAs;
    QAction *aFileLoadStateAs;
    QAction *aToolsCheatsList;
    QAction *aToolsBackupManager;
    QAction *aFileOpenISO;
    QAction *aFileOpenCDRom;
    QAction *aHelpCompatibilityList;
    QAction *aSound;
    QAction *aVideoDriver;
    QAction *aFileSaveState0;
    QAction *aFileLoadState0;
    QAction *aViewDebugMemoryEditor;
    QAction *aToolsCheatSearch;
    QAction *aViewDebugSCSPDSP;
    QAction *aTraceLogging;
    QAction *aViewDebugSCSPChan;
    QAction *aFileOpenSSF;
    QAction *aViewDebugSH1;
    QAction *aHelpDocumentation;
    QWidget *centralwidget;
    QFrame *fSound;
    QVBoxLayout *vboxLayout;
    QSlider *sVolume;
    QCheckBox *cbSound;
    QFrame *fVideoDriver;
    QVBoxLayout *vboxLayout1;
    QComboBox *cbVideoDriver;
    QMenuBar *menubar;
    QMenu *mFile;
    QMenu *mFileSaveState;
    QMenu *mFileLoadState;
    QMenu *mView;
    QMenu *mViewLayer;
    QMenu *mViewDebug;
    QMenu *mHelp;
    QMenu *mTools;
    QMenu *mEmulation;
    QToolBar *toolBar;

    void setupUi(QMainWindow *UIYabause)
    {
        if (UIYabause->objectName().isEmpty())
            UIYabause->setObjectName(QString::fromUtf8("UIYabause"));
        UIYabause->resize(640, 480);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/icons/yabause.png"), QSize(), QIcon::Normal, QIcon::Off);
        UIYabause->setWindowIcon(icon);
        UIYabause->setIconSize(QSize(16, 16));
        aFileQuit = new QAction(UIYabause);
        aFileQuit->setObjectName(QString::fromUtf8("aFileQuit"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/actions/icons/actions/quit.png"), QSize(), QIcon::Normal, QIcon::Off);
        aFileQuit->setIcon(icon1);
        aYabauseStart = new QAction(UIYabause);
        aYabauseStart->setObjectName(QString::fromUtf8("aYabauseStart"));
        aYabauseStop = new QAction(UIYabause);
        aYabauseStop->setObjectName(QString::fromUtf8("aYabauseStop"));
        aFileSettings = new QAction(UIYabause);
        aFileSettings->setObjectName(QString::fromUtf8("aFileSettings"));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/actions/icons/actions/settings.png"), QSize(), QIcon::Normal, QIcon::Off);
        aFileSettings->setIcon(icon2);
        aEmulationRun = new QAction(UIYabause);
        aEmulationRun->setObjectName(QString::fromUtf8("aEmulationRun"));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/actions/icons/actions/play.png"), QSize(), QIcon::Normal, QIcon::Off);
        aEmulationRun->setIcon(icon3);
        aEmulationPause = new QAction(UIYabause);
        aEmulationPause->setObjectName(QString::fromUtf8("aEmulationPause"));
        aEmulationPause->setEnabled(false);
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/actions/icons/actions/pause.png"), QSize(), QIcon::Normal, QIcon::Off);
        aEmulationPause->setIcon(icon4);
        aEmulationReset = new QAction(UIYabause);
        aEmulationReset->setObjectName(QString::fromUtf8("aEmulationReset"));
        aEmulationReset->setEnabled(false);
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/actions/icons/actions/reset.png"), QSize(), QIcon::Normal, QIcon::Off);
        aEmulationReset->setIcon(icon5);
        aToolsTransfer = new QAction(UIYabause);
        aToolsTransfer->setObjectName(QString::fromUtf8("aToolsTransfer"));
        aToolsTransfer->setEnabled(true);
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/actions/icons/actions/transfert.png"), QSize(), QIcon::Normal, QIcon::Off);
        aToolsTransfer->setIcon(icon6);
        aFileScreenshot = new QAction(UIYabause);
        aFileScreenshot->setObjectName(QString::fromUtf8("aFileScreenshot"));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/actions/icons/actions/screenshot.png"), QSize(), QIcon::Normal, QIcon::Off);
        aFileScreenshot->setIcon(icon7);
        aEmulationFrameSkipLimiter = new QAction(UIYabause);
        aEmulationFrameSkipLimiter->setObjectName(QString::fromUtf8("aEmulationFrameSkipLimiter"));
        aEmulationFrameSkipLimiter->setCheckable(true);
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/actions/icons/actions/frame_skipping.png"), QSize(), QIcon::Normal, QIcon::Off);
        aEmulationFrameSkipLimiter->setIcon(icon8);
        aViewFPS = new QAction(UIYabause);
        aViewFPS->setObjectName(QString::fromUtf8("aViewFPS"));
        aViewFPS->setCheckable(true);
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/actions/icons/actions/fps.png"), QSize(), QIcon::Normal, QIcon::Off);
        aViewFPS->setIcon(icon9);
        aViewLayerVdp1 = new QAction(UIYabause);
        aViewLayerVdp1->setObjectName(QString::fromUtf8("aViewLayerVdp1"));
        aViewLayerVdp1->setCheckable(true);
        aViewLayerVdp1->setChecked(true);
        aViewFullscreen = new QAction(UIYabause);
        aViewFullscreen->setObjectName(QString::fromUtf8("aViewFullscreen"));
        aViewFullscreen->setCheckable(true);
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/actions/fullscreen.png"), QSize(), QIcon::Normal, QIcon::Off);
        aViewFullscreen->setIcon(icon10);
        aViewLog = new QAction(UIYabause);
        aViewLog->setObjectName(QString::fromUtf8("aViewLog"));
        aViewLog->setCheckable(true);
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/actions/icons/actions/log.png"), QSize(), QIcon::Normal, QIcon::Off);
        aViewLog->setIcon(icon11);
        aViewDebugMSH2 = new QAction(UIYabause);
        aViewDebugMSH2->setObjectName(QString::fromUtf8("aViewDebugMSH2"));
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/actions/icons/actions/debug_cpu.png"), QSize(), QIcon::Normal, QIcon::Off);
        aViewDebugMSH2->setIcon(icon12);
        aViewDebugSSH2 = new QAction(UIYabause);
        aViewDebugSSH2->setObjectName(QString::fromUtf8("aViewDebugSSH2"));
        aViewDebugSSH2->setEnabled(true);
        aViewDebugSSH2->setIcon(icon12);
        aViewDebugVDP1 = new QAction(UIYabause);
        aViewDebugVDP1->setObjectName(QString::fromUtf8("aViewDebugVDP1"));
        aViewDebugVDP1->setEnabled(true);
        aViewDebugVDP1->setIcon(icon12);
        aViewDebugVDP2 = new QAction(UIYabause);
        aViewDebugVDP2->setObjectName(QString::fromUtf8("aViewDebugVDP2"));
        aViewDebugVDP2->setEnabled(true);
        aViewDebugVDP2->setIcon(icon12);
        aViewDebugM68K = new QAction(UIYabause);
        aViewDebugM68K->setObjectName(QString::fromUtf8("aViewDebugM68K"));
        aViewDebugM68K->setEnabled(true);
        aViewDebugM68K->setIcon(icon12);
        aViewDebugSCUDSP = new QAction(UIYabause);
        aViewDebugSCUDSP->setObjectName(QString::fromUtf8("aViewDebugSCUDSP"));
        aViewDebugSCUDSP->setEnabled(true);
        aViewDebugSCUDSP->setIcon(icon12);
        aViewDebugSCSP = new QAction(UIYabause);
        aViewDebugSCSP->setObjectName(QString::fromUtf8("aViewDebugSCSP"));
        aViewDebugSCSP->setEnabled(true);
        aViewDebugSCSP->setIcon(icon12);
        aViewDebugMemoryTransfer = new QAction(UIYabause);
        aViewDebugMemoryTransfer->setObjectName(QString::fromUtf8("aViewDebugMemoryTransfer"));
        aViewDebugMemoryTransfer->setEnabled(true);
        aHelpAbout = new QAction(UIYabause);
        aHelpAbout->setObjectName(QString::fromUtf8("aHelpAbout"));
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/actions/icons/actions/about.png"), QSize(), QIcon::Normal, QIcon::Off);
        aHelpAbout->setIcon(icon13);
        aViewLayerNBG0 = new QAction(UIYabause);
        aViewLayerNBG0->setObjectName(QString::fromUtf8("aViewLayerNBG0"));
        aViewLayerNBG0->setCheckable(true);
        aViewLayerNBG0->setChecked(true);
        aViewLayerNBG1 = new QAction(UIYabause);
        aViewLayerNBG1->setObjectName(QString::fromUtf8("aViewLayerNBG1"));
        aViewLayerNBG1->setCheckable(true);
        aViewLayerNBG1->setChecked(true);
        aViewLayerNBG2 = new QAction(UIYabause);
        aViewLayerNBG2->setObjectName(QString::fromUtf8("aViewLayerNBG2"));
        aViewLayerNBG2->setCheckable(true);
        aViewLayerNBG2->setChecked(true);
        aViewLayerNBG3 = new QAction(UIYabause);
        aViewLayerNBG3->setObjectName(QString::fromUtf8("aViewLayerNBG3"));
        aViewLayerNBG3->setCheckable(true);
        aViewLayerNBG3->setChecked(true);
        aViewLayerRBG0 = new QAction(UIYabause);
        aViewLayerRBG0->setObjectName(QString::fromUtf8("aViewLayerRBG0"));
        aViewLayerRBG0->setCheckable(true);
        aViewLayerRBG0->setChecked(true);
        aFileSaveState1 = new QAction(UIYabause);
        aFileSaveState1->setObjectName(QString::fromUtf8("aFileSaveState1"));
        aFileSaveState2 = new QAction(UIYabause);
        aFileSaveState2->setObjectName(QString::fromUtf8("aFileSaveState2"));
        aFileSaveState3 = new QAction(UIYabause);
        aFileSaveState3->setObjectName(QString::fromUtf8("aFileSaveState3"));
        aFileSaveState4 = new QAction(UIYabause);
        aFileSaveState4->setObjectName(QString::fromUtf8("aFileSaveState4"));
        aFileSaveState5 = new QAction(UIYabause);
        aFileSaveState5->setObjectName(QString::fromUtf8("aFileSaveState5"));
        aFileSaveState6 = new QAction(UIYabause);
        aFileSaveState6->setObjectName(QString::fromUtf8("aFileSaveState6"));
        aFileSaveState7 = new QAction(UIYabause);
        aFileSaveState7->setObjectName(QString::fromUtf8("aFileSaveState7"));
        aFileSaveState8 = new QAction(UIYabause);
        aFileSaveState8->setObjectName(QString::fromUtf8("aFileSaveState8"));
        aFileSaveState9 = new QAction(UIYabause);
        aFileSaveState9->setObjectName(QString::fromUtf8("aFileSaveState9"));
        aFileLoadState1 = new QAction(UIYabause);
        aFileLoadState1->setObjectName(QString::fromUtf8("aFileLoadState1"));
        aFileLoadState2 = new QAction(UIYabause);
        aFileLoadState2->setObjectName(QString::fromUtf8("aFileLoadState2"));
        aFileLoadState3 = new QAction(UIYabause);
        aFileLoadState3->setObjectName(QString::fromUtf8("aFileLoadState3"));
        aFileLoadState4 = new QAction(UIYabause);
        aFileLoadState4->setObjectName(QString::fromUtf8("aFileLoadState4"));
        aFileLoadState5 = new QAction(UIYabause);
        aFileLoadState5->setObjectName(QString::fromUtf8("aFileLoadState5"));
        aFileLoadState6 = new QAction(UIYabause);
        aFileLoadState6->setObjectName(QString::fromUtf8("aFileLoadState6"));
        aFileLoadState7 = new QAction(UIYabause);
        aFileLoadState7->setObjectName(QString::fromUtf8("aFileLoadState7"));
        aFileLoadState8 = new QAction(UIYabause);
        aFileLoadState8->setObjectName(QString::fromUtf8("aFileLoadState8"));
        aFileLoadState9 = new QAction(UIYabause);
        aFileLoadState9->setObjectName(QString::fromUtf8("aFileLoadState9"));
        aFileSaveStateAs = new QAction(UIYabause);
        aFileSaveStateAs->setObjectName(QString::fromUtf8("aFileSaveStateAs"));
        aFileLoadStateAs = new QAction(UIYabause);
        aFileLoadStateAs->setObjectName(QString::fromUtf8("aFileLoadStateAs"));
        aToolsCheatsList = new QAction(UIYabause);
        aToolsCheatsList->setObjectName(QString::fromUtf8("aToolsCheatsList"));
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/actions/icons/actions/cheats.png"), QSize(), QIcon::Normal, QIcon::Off);
        aToolsCheatsList->setIcon(icon14);
        aToolsBackupManager = new QAction(UIYabause);
        aToolsBackupManager->setObjectName(QString::fromUtf8("aToolsBackupManager"));
        QIcon icon15;
        icon15.addFile(QString::fromUtf8(":/actions/icons/actions/backup_ram.png"), QSize(), QIcon::Normal, QIcon::Off);
        aToolsBackupManager->setIcon(icon15);
        aFileOpenISO = new QAction(UIYabause);
        aFileOpenISO->setObjectName(QString::fromUtf8("aFileOpenISO"));
        QIcon icon16;
        icon16.addFile(QString::fromUtf8(":/actions/icons/actions/iso.png"), QSize(), QIcon::Normal, QIcon::Off);
        aFileOpenISO->setIcon(icon16);
        aFileOpenCDRom = new QAction(UIYabause);
        aFileOpenCDRom->setObjectName(QString::fromUtf8("aFileOpenCDRom"));
        QIcon icon17;
        icon17.addFile(QString::fromUtf8(":/actions/icons/actions/cd.png"), QSize(), QIcon::Normal, QIcon::Off);
        aFileOpenCDRom->setIcon(icon17);
        aHelpCompatibilityList = new QAction(UIYabause);
        aHelpCompatibilityList->setObjectName(QString::fromUtf8("aHelpCompatibilityList"));
        QIcon icon18;
        icon18.addFile(QString::fromUtf8(":/actions/icons/actions/compatibility.png"), QSize(), QIcon::Normal, QIcon::Off);
        aHelpCompatibilityList->setIcon(icon18);
        aSound = new QAction(UIYabause);
        aSound->setObjectName(QString::fromUtf8("aSound"));
        QIcon icon19;
        icon19.addFile(QString::fromUtf8(":/actions/sound.png"), QSize(), QIcon::Normal, QIcon::Off);
        aSound->setIcon(icon19);
        aVideoDriver = new QAction(UIYabause);
        aVideoDriver->setObjectName(QString::fromUtf8("aVideoDriver"));
        QIcon icon20;
        icon20.addFile(QString::fromUtf8(":/actions/icons/actions/video.png"), QSize(), QIcon::Normal, QIcon::Off);
        aVideoDriver->setIcon(icon20);
        aFileSaveState0 = new QAction(UIYabause);
        aFileSaveState0->setObjectName(QString::fromUtf8("aFileSaveState0"));
        aFileLoadState0 = new QAction(UIYabause);
        aFileLoadState0->setObjectName(QString::fromUtf8("aFileLoadState0"));
        aViewDebugMemoryEditor = new QAction(UIYabause);
        aViewDebugMemoryEditor->setObjectName(QString::fromUtf8("aViewDebugMemoryEditor"));
        aViewDebugMemoryEditor->setEnabled(true);
        aViewDebugMemoryEditor->setIcon(icon12);
        aToolsCheatSearch = new QAction(UIYabause);
        aToolsCheatSearch->setObjectName(QString::fromUtf8("aToolsCheatSearch"));
        QIcon icon21;
        icon21.addFile(QString::fromUtf8(":/actions/icons/actions/cheat_search.png"), QSize(), QIcon::Normal, QIcon::Off);
        aToolsCheatSearch->setIcon(icon21);
        aViewDebugSCSPDSP = new QAction(UIYabause);
        aViewDebugSCSPDSP->setObjectName(QString::fromUtf8("aViewDebugSCSPDSP"));
        aViewDebugSCSPDSP->setIcon(icon12);
        aTraceLogging = new QAction(UIYabause);
        aTraceLogging->setObjectName(QString::fromUtf8("aTraceLogging"));
        aTraceLogging->setCheckable(true);
        aTraceLogging->setChecked(false);
        aViewDebugSCSPChan = new QAction(UIYabause);
        aViewDebugSCSPChan->setObjectName(QString::fromUtf8("aViewDebugSCSPChan"));
        aFileOpenSSF = new QAction(UIYabause);
        aFileOpenSSF->setObjectName(QString::fromUtf8("aFileOpenSSF"));
        aViewDebugSH1 = new QAction(UIYabause);
        aViewDebugSH1->setObjectName(QString::fromUtf8("aViewDebugSH1"));
        aViewDebugSH1->setIcon(icon12);
        aHelpDocumentation = new QAction(UIYabause);
        aHelpDocumentation->setObjectName(QString::fromUtf8("aHelpDocumentation"));
        centralwidget = new QWidget(UIYabause);
        centralwidget->setObjectName(QString::fromUtf8("centralwidget"));
        fSound = new QFrame(centralwidget);
        fSound->setObjectName(QString::fromUtf8("fSound"));
        fSound->setGeometry(QRect(10, 10, 64, 121));
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(fSound->sizePolicy().hasHeightForWidth());
        fSound->setSizePolicy(sizePolicy);
        fSound->setFrameShape(QFrame::StyledPanel);
        fSound->setFrameShadow(QFrame::Sunken);
        vboxLayout = new QVBoxLayout(fSound);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        sVolume = new QSlider(fSound);
        sVolume->setObjectName(QString::fromUtf8("sVolume"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(sVolume->sizePolicy().hasHeightForWidth());
        sVolume->setSizePolicy(sizePolicy1);
        sVolume->setMaximum(100);
        sVolume->setValue(100);
        sVolume->setOrientation(Qt::Vertical);
        sVolume->setTickPosition(QSlider::TicksBothSides);
        sVolume->setTickInterval(15);

        vboxLayout->addWidget(sVolume);

        cbSound = new QCheckBox(fSound);
        cbSound->setObjectName(QString::fromUtf8("cbSound"));
        cbSound->setIcon(icon19);
        cbSound->setChecked(true);

        vboxLayout->addWidget(cbSound);

        fVideoDriver = new QFrame(centralwidget);
        fVideoDriver->setObjectName(QString::fromUtf8("fVideoDriver"));
        fVideoDriver->setGeometry(QRect(80, 10, 250, 48));
        sizePolicy.setHeightForWidth(fVideoDriver->sizePolicy().hasHeightForWidth());
        fVideoDriver->setSizePolicy(sizePolicy);
        fVideoDriver->setFrameShape(QFrame::StyledPanel);
        fVideoDriver->setFrameShadow(QFrame::Sunken);
        vboxLayout1 = new QVBoxLayout(fVideoDriver);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        cbVideoDriver = new QComboBox(fVideoDriver);
        cbVideoDriver->setObjectName(QString::fromUtf8("cbVideoDriver"));

        vboxLayout1->addWidget(cbVideoDriver);

        UIYabause->setCentralWidget(centralwidget);
        menubar = new QMenuBar(UIYabause);
        menubar->setObjectName(QString::fromUtf8("menubar"));
        menubar->setGeometry(QRect(0, 0, 640, 21));
        mFile = new QMenu(menubar);
        mFile->setObjectName(QString::fromUtf8("mFile"));
        mFileSaveState = new QMenu(mFile);
        mFileSaveState->setObjectName(QString::fromUtf8("mFileSaveState"));
        QIcon icon22;
        icon22.addFile(QString::fromUtf8(":/actions/icons/actions/save_state.png"), QSize(), QIcon::Normal, QIcon::Off);
        mFileSaveState->setIcon(icon22);
        mFileLoadState = new QMenu(mFile);
        mFileLoadState->setObjectName(QString::fromUtf8("mFileLoadState"));
        QIcon icon23;
        icon23.addFile(QString::fromUtf8(":/actions/icons/actions/load_state.png"), QSize(), QIcon::Normal, QIcon::Off);
        mFileLoadState->setIcon(icon23);
        mView = new QMenu(menubar);
        mView->setObjectName(QString::fromUtf8("mView"));
        mViewLayer = new QMenu(mView);
        mViewLayer->setObjectName(QString::fromUtf8("mViewLayer"));
        QIcon icon24;
        icon24.addFile(QString::fromUtf8(":/actions/icons/actions/layer.png"), QSize(), QIcon::Normal, QIcon::Off);
        mViewLayer->setIcon(icon24);
        mViewDebug = new QMenu(menubar);
        mViewDebug->setObjectName(QString::fromUtf8("mViewDebug"));
        mViewDebug->setEnabled(true);
        mHelp = new QMenu(menubar);
        mHelp->setObjectName(QString::fromUtf8("mHelp"));
        mTools = new QMenu(menubar);
        mTools->setObjectName(QString::fromUtf8("mTools"));
        mEmulation = new QMenu(menubar);
        mEmulation->setObjectName(QString::fromUtf8("mEmulation"));
        UIYabause->setMenuBar(menubar);
        toolBar = new QToolBar(UIYabause);
        toolBar->setObjectName(QString::fromUtf8("toolBar"));
        toolBar->setMovable(false);
        toolBar->setFloatable(true);
        UIYabause->addToolBar(Qt::TopToolBarArea, toolBar);

        menubar->addAction(mFile->menuAction());
        menubar->addAction(mEmulation->menuAction());
        menubar->addAction(mTools->menuAction());
        menubar->addAction(mView->menuAction());
        menubar->addAction(mViewDebug->menuAction());
        menubar->addAction(mHelp->menuAction());
        mFile->addAction(aFileSettings);
        mFile->addSeparator();
        mFile->addAction(aFileOpenISO);
        mFile->addAction(aFileOpenCDRom);
        mFile->addAction(aFileOpenSSF);
        mFile->addSeparator();
        mFile->addAction(mFileSaveState->menuAction());
        mFile->addAction(mFileLoadState->menuAction());
        mFile->addSeparator();
        mFile->addAction(aFileScreenshot);
        mFile->addSeparator();
        mFile->addAction(aFileQuit);
        mFileSaveState->addAction(aFileSaveStateAs);
        mFileSaveState->addSeparator();
        mFileSaveState->addAction(aFileSaveState0);
        mFileSaveState->addAction(aFileSaveState1);
        mFileSaveState->addAction(aFileSaveState2);
        mFileSaveState->addAction(aFileSaveState3);
        mFileSaveState->addAction(aFileSaveState4);
        mFileSaveState->addAction(aFileSaveState5);
        mFileSaveState->addAction(aFileSaveState6);
        mFileSaveState->addAction(aFileSaveState7);
        mFileSaveState->addAction(aFileSaveState8);
        mFileSaveState->addAction(aFileSaveState9);
        mFileLoadState->addAction(aFileLoadStateAs);
        mFileLoadState->addSeparator();
        mFileLoadState->addAction(aFileLoadState0);
        mFileLoadState->addAction(aFileLoadState1);
        mFileLoadState->addAction(aFileLoadState2);
        mFileLoadState->addAction(aFileLoadState3);
        mFileLoadState->addAction(aFileLoadState4);
        mFileLoadState->addAction(aFileLoadState5);
        mFileLoadState->addAction(aFileLoadState6);
        mFileLoadState->addAction(aFileLoadState7);
        mFileLoadState->addAction(aFileLoadState8);
        mFileLoadState->addAction(aFileLoadState9);
        mView->addAction(aViewFPS);
        mView->addAction(mViewLayer->menuAction());
        mView->addAction(aViewFullscreen);
        mView->addAction(aViewLog);
        mViewLayer->addAction(aViewLayerVdp1);
        mViewLayer->addAction(aViewLayerNBG0);
        mViewLayer->addAction(aViewLayerNBG1);
        mViewLayer->addAction(aViewLayerNBG2);
        mViewLayer->addAction(aViewLayerNBG3);
        mViewLayer->addAction(aViewLayerRBG0);
        mViewDebug->addAction(aViewDebugMSH2);
        mViewDebug->addAction(aViewDebugSSH2);
        mViewDebug->addAction(aViewDebugVDP1);
        mViewDebug->addAction(aViewDebugVDP2);
        mViewDebug->addAction(aViewDebugM68K);
        mViewDebug->addAction(aViewDebugSCUDSP);
        mViewDebug->addAction(aViewDebugSCSP);
        mViewDebug->addAction(aViewDebugSCSPChan);
        mViewDebug->addAction(aViewDebugSCSPDSP);
        mViewDebug->addAction(aViewDebugSH1);
        mViewDebug->addSeparator();
        mViewDebug->addAction(aViewDebugMemoryEditor);
        mViewDebug->addSeparator();
        mViewDebug->addAction(aTraceLogging);
        mHelp->addAction(aHelpDocumentation);
        mHelp->addAction(aHelpCompatibilityList);
        mHelp->addAction(aHelpAbout);
        mTools->addAction(aToolsBackupManager);
        mTools->addAction(aToolsCheatsList);
        mTools->addAction(aToolsCheatSearch);
        mTools->addAction(aToolsTransfer);
        mEmulation->addAction(aEmulationRun);
        mEmulation->addAction(aEmulationPause);
        mEmulation->addAction(aEmulationReset);
        mEmulation->addSeparator();
        mEmulation->addAction(aEmulationFrameSkipLimiter);
        toolBar->addAction(aEmulationRun);
        toolBar->addAction(aEmulationPause);
        toolBar->addAction(aEmulationReset);
        toolBar->addSeparator();
        toolBar->addAction(aFileOpenISO);
        toolBar->addAction(aFileOpenCDRom);
        toolBar->addSeparator();
        toolBar->addAction(aFileSettings);
        toolBar->addAction(aVideoDriver);
        toolBar->addAction(aSound);
        toolBar->addAction(aEmulationFrameSkipLimiter);
        toolBar->addAction(aViewFPS);
        toolBar->addAction(aViewFullscreen);
        toolBar->addAction(aFileScreenshot);
        toolBar->addSeparator();
        toolBar->addAction(aToolsBackupManager);
        toolBar->addAction(aToolsCheatsList);
        toolBar->addSeparator();
        toolBar->addAction(aFileQuit);
        toolBar->addAction(aHelpAbout);

        retranslateUi(UIYabause);
        QObject::connect(aFileQuit, SIGNAL(triggered()), UIYabause, SLOT(close()));

        QMetaObject::connectSlotsByName(UIYabause);
    } // setupUi

    void retranslateUi(QMainWindow *UIYabause)
    {
        UIYabause->setWindowTitle(QCoreApplication::translate("UIYabause", "Yabause Qt GUI", nullptr));
        aFileQuit->setText(QCoreApplication::translate("UIYabause", "&Quit", nullptr));
#if QT_CONFIG(shortcut)
        aFileQuit->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        aYabauseStart->setText(QCoreApplication::translate("UIYabause", "Start", nullptr));
        aYabauseStop->setText(QCoreApplication::translate("UIYabause", "Stop", nullptr));
        aFileSettings->setText(QCoreApplication::translate("UIYabause", "&Settings...", nullptr));
#if QT_CONFIG(shortcut)
        aFileSettings->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        aEmulationRun->setText(QCoreApplication::translate("UIYabause", "R&un", nullptr));
#if QT_CONFIG(shortcut)
        aEmulationRun->setShortcut(QCoreApplication::translate("UIYabause", "F1", nullptr));
#endif // QT_CONFIG(shortcut)
        aEmulationPause->setText(QCoreApplication::translate("UIYabause", "&Pause", nullptr));
#if QT_CONFIG(shortcut)
        aEmulationPause->setShortcut(QCoreApplication::translate("UIYabause", "F2", nullptr));
#endif // QT_CONFIG(shortcut)
        aEmulationReset->setText(QCoreApplication::translate("UIYabause", "&Reset", nullptr));
#if QT_CONFIG(shortcut)
        aEmulationReset->setShortcut(QCoreApplication::translate("UIYabause", "F3", nullptr));
#endif // QT_CONFIG(shortcut)
        aToolsTransfer->setText(QCoreApplication::translate("UIYabause", "&Transfer", nullptr));
#if QT_CONFIG(shortcut)
        aToolsTransfer->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+T", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileScreenshot->setText(QCoreApplication::translate("UIYabause", "Sc&reenshot", nullptr));
#if QT_CONFIG(shortcut)
        aFileScreenshot->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+P", nullptr));
#endif // QT_CONFIG(shortcut)
        aEmulationFrameSkipLimiter->setText(QCoreApplication::translate("UIYabause", "&Frame Skip/Limiter", nullptr));
#if QT_CONFIG(shortcut)
        aEmulationFrameSkipLimiter->setShortcut(QCoreApplication::translate("UIYabause", "F4", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewFPS->setText(QCoreApplication::translate("UIYabause", "F&PS", nullptr));
        aViewFPS->setIconText(QCoreApplication::translate("UIYabause", "FPS", nullptr));
#if QT_CONFIG(shortcut)
        aViewFPS->setShortcut(QCoreApplication::translate("UIYabause", "F12", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewLayerVdp1->setText(QCoreApplication::translate("UIYabause", "Vdp1", nullptr));
#if QT_CONFIG(shortcut)
        aViewLayerVdp1->setShortcut(QCoreApplication::translate("UIYabause", "1", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewFullscreen->setText(QCoreApplication::translate("UIYabause", "&Fullscreen", nullptr));
#if QT_CONFIG(shortcut)
        aViewFullscreen->setShortcut(QCoreApplication::translate("UIYabause", "Alt+Return", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewLog->setText(QCoreApplication::translate("UIYabause", "&Log", nullptr));
#if QT_CONFIG(shortcut)
        aViewLog->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+L", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewDebugMSH2->setText(QCoreApplication::translate("UIYabause", "&Master SH2", nullptr));
        aViewDebugSSH2->setText(QCoreApplication::translate("UIYabause", "&Slave SH2", nullptr));
        aViewDebugVDP1->setText(QCoreApplication::translate("UIYabause", "VDP&1", nullptr));
        aViewDebugVDP2->setText(QCoreApplication::translate("UIYabause", "VDP&2", nullptr));
        aViewDebugM68K->setText(QCoreApplication::translate("UIYabause", "M&68K", nullptr));
        aViewDebugSCUDSP->setText(QCoreApplication::translate("UIYabause", "SCU-&DSP", nullptr));
        aViewDebugSCSP->setText(QCoreApplication::translate("UIYabause", "S&CSP", nullptr));
        aViewDebugMemoryTransfer->setText(QCoreApplication::translate("UIYabause", "Memory Transfer", nullptr));
        aHelpAbout->setText(QCoreApplication::translate("UIYabause", "&About...", nullptr));
        aViewLayerNBG0->setText(QCoreApplication::translate("UIYabause", "NBG0", nullptr));
#if QT_CONFIG(shortcut)
        aViewLayerNBG0->setShortcut(QCoreApplication::translate("UIYabause", "2", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewLayerNBG1->setText(QCoreApplication::translate("UIYabause", "NBG1", nullptr));
#if QT_CONFIG(shortcut)
        aViewLayerNBG1->setShortcut(QCoreApplication::translate("UIYabause", "3", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewLayerNBG2->setText(QCoreApplication::translate("UIYabause", "NBG2", nullptr));
#if QT_CONFIG(shortcut)
        aViewLayerNBG2->setShortcut(QCoreApplication::translate("UIYabause", "4", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewLayerNBG3->setText(QCoreApplication::translate("UIYabause", "NBG3", nullptr));
#if QT_CONFIG(shortcut)
        aViewLayerNBG3->setShortcut(QCoreApplication::translate("UIYabause", "5", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewLayerRBG0->setText(QCoreApplication::translate("UIYabause", "RBG0", nullptr));
#if QT_CONFIG(shortcut)
        aViewLayerRBG0->setShortcut(QCoreApplication::translate("UIYabause", "6", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveState1->setText(QCoreApplication::translate("UIYabause", "1", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState1->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+1", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveState2->setText(QCoreApplication::translate("UIYabause", "2", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState2->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+2", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveState3->setText(QCoreApplication::translate("UIYabause", "3", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState3->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+3", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveState4->setText(QCoreApplication::translate("UIYabause", "4", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState4->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+4", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveState5->setText(QCoreApplication::translate("UIYabause", "5", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState5->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+5", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveState6->setText(QCoreApplication::translate("UIYabause", "6", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState6->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+6", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveState7->setText(QCoreApplication::translate("UIYabause", "7", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState7->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+7", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveState8->setText(QCoreApplication::translate("UIYabause", "8", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState8->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+8", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveState9->setText(QCoreApplication::translate("UIYabause", "9", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState9->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+9", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState1->setText(QCoreApplication::translate("UIYabause", "1", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState1->setShortcut(QCoreApplication::translate("UIYabause", "Alt+1", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState2->setText(QCoreApplication::translate("UIYabause", "2", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState2->setShortcut(QCoreApplication::translate("UIYabause", "Alt+2", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState3->setText(QCoreApplication::translate("UIYabause", "3", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState3->setShortcut(QCoreApplication::translate("UIYabause", "Alt+3", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState4->setText(QCoreApplication::translate("UIYabause", "4", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState4->setShortcut(QCoreApplication::translate("UIYabause", "Alt+4", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState5->setText(QCoreApplication::translate("UIYabause", "5", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState5->setShortcut(QCoreApplication::translate("UIYabause", "Alt+5", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState6->setText(QCoreApplication::translate("UIYabause", "6", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState6->setShortcut(QCoreApplication::translate("UIYabause", "Alt+6", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState7->setText(QCoreApplication::translate("UIYabause", "7", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState7->setShortcut(QCoreApplication::translate("UIYabause", "Alt+7", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState8->setText(QCoreApplication::translate("UIYabause", "8", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState8->setShortcut(QCoreApplication::translate("UIYabause", "Alt+8", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState9->setText(QCoreApplication::translate("UIYabause", "9", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState9->setShortcut(QCoreApplication::translate("UIYabause", "Alt+9", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileSaveStateAs->setText(QCoreApplication::translate("UIYabause", "To File...", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveStateAs->setShortcut(QCoreApplication::translate("UIYabause", "F9", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadStateAs->setText(QCoreApplication::translate("UIYabause", "From File...", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadStateAs->setShortcut(QCoreApplication::translate("UIYabause", "F5", nullptr));
#endif // QT_CONFIG(shortcut)
        aToolsCheatsList->setText(QCoreApplication::translate("UIYabause", "&Cheats List...", nullptr));
#if QT_CONFIG(shortcut)
        aToolsCheatsList->setShortcut(QCoreApplication::translate("UIYabause", "Alt+L", nullptr));
#endif // QT_CONFIG(shortcut)
        aToolsBackupManager->setText(QCoreApplication::translate("UIYabause", "&Backup Manager...", nullptr));
#if QT_CONFIG(shortcut)
        aToolsBackupManager->setShortcut(QCoreApplication::translate("UIYabause", "Alt+B", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileOpenISO->setText(QCoreApplication::translate("UIYabause", "Open &ISO...", nullptr));
#if QT_CONFIG(shortcut)
        aFileOpenISO->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+I", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileOpenCDRom->setText(QCoreApplication::translate("UIYabause", "Open &CD Rom...", nullptr));
#if QT_CONFIG(shortcut)
        aFileOpenCDRom->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+C", nullptr));
#endif // QT_CONFIG(shortcut)
        aHelpCompatibilityList->setText(QCoreApplication::translate("UIYabause", "Compatibility List", nullptr));
#if QT_CONFIG(statustip)
        aHelpCompatibilityList->setStatusTip(QCoreApplication::translate("UIYabause", "http://wiki.yabause.org/index.php5?title=Compatibility_list", nullptr));
#endif // QT_CONFIG(statustip)
        aSound->setText(QCoreApplication::translate("UIYabause", "Sound...", nullptr));
        aVideoDriver->setText(QCoreApplication::translate("UIYabause", "Video Driver", nullptr));
        aFileSaveState0->setText(QCoreApplication::translate("UIYabause", "0", nullptr));
#if QT_CONFIG(shortcut)
        aFileSaveState0->setShortcut(QCoreApplication::translate("UIYabause", "Ctrl+0", nullptr));
#endif // QT_CONFIG(shortcut)
        aFileLoadState0->setText(QCoreApplication::translate("UIYabause", "0", nullptr));
#if QT_CONFIG(shortcut)
        aFileLoadState0->setShortcut(QCoreApplication::translate("UIYabause", "Alt+0", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewDebugMemoryEditor->setText(QCoreApplication::translate("UIYabause", "Memory &Editor", nullptr));
        aToolsCheatSearch->setText(QCoreApplication::translate("UIYabause", "Cheat &Search...", nullptr));
#if QT_CONFIG(shortcut)
        aToolsCheatSearch->setShortcut(QCoreApplication::translate("UIYabause", "Alt+S", nullptr));
#endif // QT_CONFIG(shortcut)
        aViewDebugSCSPDSP->setText(QCoreApplication::translate("UIYabause", "SCSP DSP", nullptr));
        aTraceLogging->setText(QCoreApplication::translate("UIYabause", "Trace Logging", nullptr));
        aViewDebugSCSPChan->setText(QCoreApplication::translate("UIYabause", "SCSP Channels", nullptr));
        aFileOpenSSF->setText(QCoreApplication::translate("UIYabause", "Open SSF", nullptr));
        aViewDebugSH1->setText(QCoreApplication::translate("UIYabause", "SH1", nullptr));
        aHelpDocumentation->setText(QCoreApplication::translate("UIYabause", "Documentation", nullptr));
#if QT_CONFIG(statustip)
        aHelpDocumentation->setStatusTip(QCoreApplication::translate("UIYabause", "http://wiki.yabause.org/index.php5?title=Documentations", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(tooltip)
        sVolume->setToolTip(QCoreApplication::translate("UIYabause", "Volume", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cbSound->setToolTip(QCoreApplication::translate("UIYabause", "Mute/UnMute Sound", nullptr));
#endif // QT_CONFIG(tooltip)
        mFile->setTitle(QCoreApplication::translate("UIYabause", "&File", nullptr));
        mFileSaveState->setTitle(QCoreApplication::translate("UIYabause", "S&ave State", nullptr));
        mFileLoadState->setTitle(QCoreApplication::translate("UIYabause", "L&oad State", nullptr));
        mView->setTitle(QCoreApplication::translate("UIYabause", "&View", nullptr));
        mViewLayer->setTitle(QCoreApplication::translate("UIYabause", "&Layer", nullptr));
        mViewDebug->setTitle(QCoreApplication::translate("UIYabause", "&Debug", nullptr));
        mHelp->setTitle(QCoreApplication::translate("UIYabause", "&Help", nullptr));
        mTools->setTitle(QCoreApplication::translate("UIYabause", "&Tools", nullptr));
        mEmulation->setTitle(QCoreApplication::translate("UIYabause", "&Emulation", nullptr));
        toolBar->setWindowTitle(QCoreApplication::translate("UIYabause", "toolBar", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIYabause: public Ui_UIYabause {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIYABAUSE_H
