/********************************************************************************
** Form generated from reading UI file 'UIDebugVDP2.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIDEBUGVDP2_H
#define UI_UIDEBUGVDP2_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIDebugVDP2
{
public:
    QVBoxLayout *verticalLayout_7;
    QGridLayout *gridLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QCheckBox *cbNBG0Enabled;
    QPlainTextEdit *pteNBG0Info;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *cbNBG1Enabled;
    QPlainTextEdit *pteNBG1Info;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *cbNBG2Enabled;
    QPlainTextEdit *pteNBG2Info;
    QGroupBox *groupBox_4;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *cbNBG3Enabled;
    QPlainTextEdit *pteNBG3Info;
    QGroupBox *groupBox_5;
    QVBoxLayout *verticalLayout_5;
    QCheckBox *cbRBG0Enabled;
    QPlainTextEdit *pteRBG0Info;
    QGroupBox *groupBox_6;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *cbDisplayEnabled;
    QPlainTextEdit *pteGeneralInfo;
    QHBoxLayout *horizontalLayout;
    QPushButton *pbViewer;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UIDebugVDP2)
    {
        if (UIDebugVDP2->objectName().isEmpty())
            UIDebugVDP2->setObjectName(QString::fromUtf8("UIDebugVDP2"));
        UIDebugVDP2->resize(788, 560);
        verticalLayout_7 = new QVBoxLayout(UIDebugVDP2);
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        groupBox = new QGroupBox(UIDebugVDP2);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        cbNBG0Enabled = new QCheckBox(groupBox);
        cbNBG0Enabled->setObjectName(QString::fromUtf8("cbNBG0Enabled"));
        cbNBG0Enabled->setCheckable(true);

        verticalLayout->addWidget(cbNBG0Enabled);

        pteNBG0Info = new QPlainTextEdit(groupBox);
        pteNBG0Info->setObjectName(QString::fromUtf8("pteNBG0Info"));
        pteNBG0Info->setReadOnly(true);

        verticalLayout->addWidget(pteNBG0Info);


        gridLayout->addWidget(groupBox, 0, 0, 1, 1);

        groupBox_2 = new QGroupBox(UIDebugVDP2);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        verticalLayout_2 = new QVBoxLayout(groupBox_2);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        cbNBG1Enabled = new QCheckBox(groupBox_2);
        cbNBG1Enabled->setObjectName(QString::fromUtf8("cbNBG1Enabled"));
        cbNBG1Enabled->setCheckable(true);

        verticalLayout_2->addWidget(cbNBG1Enabled);

        pteNBG1Info = new QPlainTextEdit(groupBox_2);
        pteNBG1Info->setObjectName(QString::fromUtf8("pteNBG1Info"));
        pteNBG1Info->setReadOnly(true);

        verticalLayout_2->addWidget(pteNBG1Info);


        gridLayout->addWidget(groupBox_2, 0, 1, 1, 1);

        groupBox_3 = new QGroupBox(UIDebugVDP2);
        groupBox_3->setObjectName(QString::fromUtf8("groupBox_3"));
        verticalLayout_3 = new QVBoxLayout(groupBox_3);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        cbNBG2Enabled = new QCheckBox(groupBox_3);
        cbNBG2Enabled->setObjectName(QString::fromUtf8("cbNBG2Enabled"));
        cbNBG2Enabled->setCheckable(true);

        verticalLayout_3->addWidget(cbNBG2Enabled);

        pteNBG2Info = new QPlainTextEdit(groupBox_3);
        pteNBG2Info->setObjectName(QString::fromUtf8("pteNBG2Info"));
        pteNBG2Info->setReadOnly(true);

        verticalLayout_3->addWidget(pteNBG2Info);


        gridLayout->addWidget(groupBox_3, 0, 2, 1, 1);

        groupBox_4 = new QGroupBox(UIDebugVDP2);
        groupBox_4->setObjectName(QString::fromUtf8("groupBox_4"));
        verticalLayout_4 = new QVBoxLayout(groupBox_4);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        cbNBG3Enabled = new QCheckBox(groupBox_4);
        cbNBG3Enabled->setObjectName(QString::fromUtf8("cbNBG3Enabled"));
        cbNBG3Enabled->setCheckable(true);

        verticalLayout_4->addWidget(cbNBG3Enabled);

        pteNBG3Info = new QPlainTextEdit(groupBox_4);
        pteNBG3Info->setObjectName(QString::fromUtf8("pteNBG3Info"));
        pteNBG3Info->setReadOnly(true);

        verticalLayout_4->addWidget(pteNBG3Info);


        gridLayout->addWidget(groupBox_4, 1, 0, 1, 1);

        groupBox_5 = new QGroupBox(UIDebugVDP2);
        groupBox_5->setObjectName(QString::fromUtf8("groupBox_5"));
        verticalLayout_5 = new QVBoxLayout(groupBox_5);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        cbRBG0Enabled = new QCheckBox(groupBox_5);
        cbRBG0Enabled->setObjectName(QString::fromUtf8("cbRBG0Enabled"));
        cbRBG0Enabled->setCheckable(true);

        verticalLayout_5->addWidget(cbRBG0Enabled);

        pteRBG0Info = new QPlainTextEdit(groupBox_5);
        pteRBG0Info->setObjectName(QString::fromUtf8("pteRBG0Info"));
        pteRBG0Info->setReadOnly(true);

        verticalLayout_5->addWidget(pteRBG0Info);


        gridLayout->addWidget(groupBox_5, 1, 1, 1, 1);

        groupBox_6 = new QGroupBox(UIDebugVDP2);
        groupBox_6->setObjectName(QString::fromUtf8("groupBox_6"));
        verticalLayout_6 = new QVBoxLayout(groupBox_6);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        cbDisplayEnabled = new QCheckBox(groupBox_6);
        cbDisplayEnabled->setObjectName(QString::fromUtf8("cbDisplayEnabled"));
        cbDisplayEnabled->setCheckable(true);

        verticalLayout_6->addWidget(cbDisplayEnabled);

        pteGeneralInfo = new QPlainTextEdit(groupBox_6);
        pteGeneralInfo->setObjectName(QString::fromUtf8("pteGeneralInfo"));
        pteGeneralInfo->setReadOnly(true);

        verticalLayout_6->addWidget(pteGeneralInfo);


        gridLayout->addWidget(groupBox_6, 1, 2, 1, 1);


        verticalLayout_7->addLayout(gridLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        pbViewer = new QPushButton(UIDebugVDP2);
        pbViewer->setObjectName(QString::fromUtf8("pbViewer"));

        horizontalLayout->addWidget(pbViewer);

        dbbButtons = new QDialogButtonBox(UIDebugVDP2);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout->addWidget(dbbButtons);


        verticalLayout_7->addLayout(horizontalLayout);


        retranslateUi(UIDebugVDP2);
        QObject::connect(dbbButtons, SIGNAL(rejected()), UIDebugVDP2, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIDebugVDP2);
    } // setupUi

    void retranslateUi(QDialog *UIDebugVDP2)
    {
        UIDebugVDP2->setWindowTitle(QCoreApplication::translate("UIDebugVDP2", "Debug VDP2Viewer", nullptr));
        groupBox->setTitle(QCoreApplication::translate("UIDebugVDP2", "NBG0/RBG1 Info", nullptr));
        cbNBG0Enabled->setText(QCoreApplication::translate("UIDebugVDP2", "Screen Enabled", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("UIDebugVDP2", "NBG1 Info", nullptr));
        cbNBG1Enabled->setText(QCoreApplication::translate("UIDebugVDP2", "Screen Enabled", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("UIDebugVDP2", "NBG2 Info", nullptr));
        cbNBG2Enabled->setText(QCoreApplication::translate("UIDebugVDP2", "Screen Enabled", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("UIDebugVDP2", "NBG3 Info", nullptr));
        cbNBG3Enabled->setText(QCoreApplication::translate("UIDebugVDP2", "Screen Enabled", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("UIDebugVDP2", "RBG0 Info", nullptr));
        cbRBG0Enabled->setText(QCoreApplication::translate("UIDebugVDP2", "Screen Enabled", nullptr));
        groupBox_6->setTitle(QCoreApplication::translate("UIDebugVDP2", "General Info", nullptr));
        cbDisplayEnabled->setText(QCoreApplication::translate("UIDebugVDP2", "Display Enabled", nullptr));
        pbViewer->setText(QCoreApplication::translate("UIDebugVDP2", "Viewer", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIDebugVDP2: public Ui_UIDebugVDP2 {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIDEBUGVDP2_H
