/****************************************************************************
** Meta object code from reading C++ file 'UIHexEditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/qt/ui/UIHexEditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UIHexEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_UIHexEditorWnd_t {
    QByteArrayData data[13];
    char stringdata0[131];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UIHexEditorWnd_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UIHexEditorWnd_t qt_meta_stringdata_UIHexEditorWnd = {
    {
QT_MOC_LITERAL(0, 0, 14), // "UIHexEditorWnd"
QT_MOC_LITERAL(1, 15, 12), // "updateCursor"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 10), // "autoScroll"
QT_MOC_LITERAL(4, 40, 12), // "sliderUpdate"
QT_MOC_LITERAL(5, 53, 5), // "value"
QT_MOC_LITERAL(6, 59, 18), // "focusNextPrevChild"
QT_MOC_LITERAL(7, 78, 4), // "next"
QT_MOC_LITERAL(8, 83, 10), // "saveMemory"
QT_MOC_LITERAL(9, 94, 8), // "filename"
QT_MOC_LITERAL(10, 103, 3), // "u32"
QT_MOC_LITERAL(11, 107, 12), // "startAddress"
QT_MOC_LITERAL(12, 120, 10) // "endAddress"

    },
    "UIHexEditorWnd\0updateCursor\0\0autoScroll\0"
    "sliderUpdate\0value\0focusNextPrevChild\0"
    "next\0saveMemory\0filename\0u32\0startAddress\0"
    "endAddress"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UIHexEditorWnd[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x08 /* Private */,
       3,    0,   40,    2, 0x08 /* Private */,
       4,    1,   41,    2, 0x08 /* Private */,
       6,    1,   44,    2, 0x08 /* Private */,
       8,    3,   47,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Bool, QMetaType::Bool,    7,
    QMetaType::Bool, QMetaType::QString, 0x80000000 | 10, 0x80000000 | 10,    9,   11,   12,

       0        // eod
};

void UIHexEditorWnd::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UIHexEditorWnd *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateCursor(); break;
        case 1: _t->autoScroll(); break;
        case 2: _t->sliderUpdate((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: { bool _r = _t->focusNextPrevChild((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->saveMemory((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< u32(*)>(_a[2])),(*reinterpret_cast< u32(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UIHexEditorWnd::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractScrollArea::staticMetaObject>(),
    qt_meta_stringdata_UIHexEditorWnd.data,
    qt_meta_data_UIHexEditorWnd,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UIHexEditorWnd::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UIHexEditorWnd::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UIHexEditorWnd.stringdata0))
        return static_cast<void*>(this);
    return QAbstractScrollArea::qt_metacast(_clname);
}

int UIHexEditorWnd::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
struct qt_meta_stringdata_UIHexEditor_t {
    QByteArrayData data[1];
    char stringdata0[12];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UIHexEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UIHexEditor_t qt_meta_stringdata_UIHexEditor = {
    {
QT_MOC_LITERAL(0, 0, 11) // "UIHexEditor"

    },
    "UIHexEditor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UIHexEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void UIHexEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject UIHexEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabWidget::staticMetaObject>(),
    qt_meta_stringdata_UIHexEditor.data,
    qt_meta_data_UIHexEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UIHexEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UIHexEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UIHexEditor.stringdata0))
        return static_cast<void*>(this);
    return QTabWidget::qt_metacast(_clname);
}

int UIHexEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabWidget::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
