/********************************************************************************
** Form generated from reading UI file 'UIWheelSetting.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIWHEELSETTING_H
#define UI_UIWHEELSETTING_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QToolButton>

QT_BEGIN_NAMESPACE

class Ui_UIWheelSetting
{
public:
    QLabel *label;
    QToolButton *tbUp;
    QToolButton *tbX;
    QToolButton *tbY;
    QToolButton *tbZ;
    QToolButton *tbC;
    QToolButton *tbB;
    QToolButton *tbA;
    QToolButton *tbStart;
    QToolButton *tbDown;
    QLabel *lInfos;
    QDialogButtonBox *padButtons;
    QToolButton *tbAxis1Left;
    QToolButton *tbAxis1Right;

    void setupUi(QDialog *UIWheelSetting)
    {
        if (UIWheelSetting->objectName().isEmpty())
            UIWheelSetting->setObjectName(QString::fromUtf8("UIWheelSetting"));
        UIWheelSetting->resize(535, 375);
        UIWheelSetting->setFocusPolicy(Qt::StrongFocus);
        label = new QLabel(UIWheelSetting);
        label->setObjectName(QString::fromUtf8("label"));
        label->setGeometry(QRect(0, 0, 531, 320));
        label->setPixmap(QPixmap(QString::fromUtf8(":/actions/wheel.png")));
        label->setScaledContents(true);
        label->setAlignment(Qt::AlignCenter);
        tbUp = new QToolButton(UIWheelSetting);
        tbUp->setObjectName(QString::fromUtf8("tbUp"));
        tbUp->setGeometry(QRect(130, 100, 31, 121));
        tbUp->setFocusPolicy(Qt::NoFocus);
        tbUp->setCheckable(true);
        tbUp->setAutoRaise(true);
        tbX = new QToolButton(UIWheelSetting);
        tbX->setObjectName(QString::fromUtf8("tbX"));
        tbX->setGeometry(QRect(140, 50, 21, 31));
        tbX->setFocusPolicy(Qt::NoFocus);
        tbX->setCheckable(true);
        tbX->setAutoRaise(true);
        tbY = new QToolButton(UIWheelSetting);
        tbY->setObjectName(QString::fromUtf8("tbY"));
        tbY->setGeometry(QRect(120, 30, 21, 31));
        tbY->setFocusPolicy(Qt::NoFocus);
        tbY->setCheckable(true);
        tbY->setAutoRaise(true);
        tbZ = new QToolButton(UIWheelSetting);
        tbZ->setObjectName(QString::fromUtf8("tbZ"));
        tbZ->setGeometry(QRect(90, 20, 21, 31));
        tbZ->setFocusPolicy(Qt::NoFocus);
        tbZ->setCheckable(true);
        tbZ->setAutoRaise(true);
        tbC = new QToolButton(UIWheelSetting);
        tbC->setObjectName(QString::fromUtf8("tbC"));
        tbC->setGeometry(QRect(420, 20, 21, 31));
        tbC->setFocusPolicy(Qt::NoFocus);
        tbC->setCheckable(true);
        tbC->setAutoRaise(true);
        tbB = new QToolButton(UIWheelSetting);
        tbB->setObjectName(QString::fromUtf8("tbB"));
        tbB->setGeometry(QRect(400, 40, 21, 31));
        tbB->setFocusPolicy(Qt::NoFocus);
        tbB->setCheckable(true);
        tbB->setAutoRaise(true);
        tbA = new QToolButton(UIWheelSetting);
        tbA->setObjectName(QString::fromUtf8("tbA"));
        tbA->setGeometry(QRect(370, 50, 21, 31));
        tbA->setFocusPolicy(Qt::NoFocus);
        tbA->setCheckable(true);
        tbA->setAutoRaise(true);
        tbStart = new QToolButton(UIWheelSetting);
        tbStart->setObjectName(QString::fromUtf8("tbStart"));
        tbStart->setGeometry(QRect(250, 90, 41, 30));
        tbStart->setFocusPolicy(Qt::NoFocus);
        tbStart->setCheckable(true);
        tbStart->setAutoRaise(true);
        tbDown = new QToolButton(UIWheelSetting);
        tbDown->setObjectName(QString::fromUtf8("tbDown"));
        tbDown->setGeometry(QRect(380, 100, 31, 131));
        tbDown->setFocusPolicy(Qt::NoFocus);
        tbDown->setCheckable(true);
        tbDown->setAutoRaise(true);
        lInfos = new QLabel(UIWheelSetting);
        lInfos->setObjectName(QString::fromUtf8("lInfos"));
        lInfos->setGeometry(QRect(50, 330, 191, 41));
        lInfos->setAlignment(Qt::AlignCenter);
        padButtons = new QDialogButtonBox(UIWheelSetting);
        padButtons->setObjectName(QString::fromUtf8("padButtons"));
        padButtons->setGeometry(QRect(240, 340, 60, 30));
        padButtons->setStandardButtons(QDialogButtonBox::Close);
        tbAxis1Left = new QToolButton(UIWheelSetting);
        tbAxis1Left->setObjectName(QString::fromUtf8("tbAxis1Left"));
        tbAxis1Left->setGeometry(QRect(30, 70, 30, 151));
        tbAxis1Left->setFocusPolicy(Qt::NoFocus);
        tbAxis1Left->setCheckable(true);
        tbAxis1Left->setAutoRaise(true);
        tbAxis1Right = new QToolButton(UIWheelSetting);
        tbAxis1Right->setObjectName(QString::fromUtf8("tbAxis1Right"));
        tbAxis1Right->setGeometry(QRect(470, 70, 30, 151));
        tbAxis1Right->setFocusPolicy(Qt::NoFocus);
        tbAxis1Right->setCheckable(true);
        tbAxis1Right->setAutoRaise(true);

        retranslateUi(UIWheelSetting);
        QObject::connect(padButtons, SIGNAL(rejected()), UIWheelSetting, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIWheelSetting);
    } // setupUi

    void retranslateUi(QDialog *UIWheelSetting)
    {
        UIWheelSetting->setWindowTitle(QCoreApplication::translate("UIWheelSetting", "Wheel Configuration", nullptr));
#if QT_CONFIG(tooltip)
        tbUp->setToolTip(QCoreApplication::translate("UIWheelSetting", "Up", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbX->setToolTip(QCoreApplication::translate("UIWheelSetting", "X", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbY->setToolTip(QCoreApplication::translate("UIWheelSetting", "Y", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbZ->setToolTip(QCoreApplication::translate("UIWheelSetting", "Z", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbC->setToolTip(QCoreApplication::translate("UIWheelSetting", "C", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbB->setToolTip(QCoreApplication::translate("UIWheelSetting", "B", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbA->setToolTip(QCoreApplication::translate("UIWheelSetting", "A", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbStart->setToolTip(QCoreApplication::translate("UIWheelSetting", "Start", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbDown->setToolTip(QCoreApplication::translate("UIWheelSetting", "Down", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis1Left->setToolTip(QCoreApplication::translate("UIWheelSetting", "Left", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis1Right->setToolTip(QCoreApplication::translate("UIWheelSetting", "Right", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class UIWheelSetting: public Ui_UIWheelSetting {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIWHEELSETTING_H
