/*  Copyright 2013 Theo Berkau
    Copyright 2013 Guillaume Duhamel

    This file is part of Yabause.

    Yabause is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Yabause is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Yabause; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
*/

#ifndef SCREEN_H
#define SCREEN_H

typedef struct
{
   int index;
   int width;
   int height;
   int bpp;
   int freq;
} supportedRes_struct;

typedef void * ResolutionList;

ResolutionList ScreenGetResolutions();
int ScreenNextResolution(ResolutionList rl, supportedRes_struct * res);

void ScreenChangeResolution(supportedRes_struct * res);
void ScreenRestoreResolution();

#endif
