/****************************************************************************
** Meta object code from reading C++ file 'UIMemoryEditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/qt/ui/UIMemoryEditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UIMemoryEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MemorySearch_t {
    QByteArrayData data[13];
    char stringdata0[109];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MemorySearch_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MemorySearch_t qt_meta_stringdata_MemorySearch = {
    {
QT_MOC_LITERAL(0, 0, 12), // "MemorySearch"
QT_MOC_LITERAL(1, 13, 11), // "setBarValue"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 8), // "progress"
QT_MOC_LITERAL(4, 35, 11), // "setBarRange"
QT_MOC_LITERAL(5, 47, 7), // "minimum"
QT_MOC_LITERAL(6, 55, 7), // "maximum"
QT_MOC_LITERAL(7, 63, 12), // "searchResult"
QT_MOC_LITERAL(8, 76, 5), // "found"
QT_MOC_LITERAL(9, 82, 6), // "cancel"
QT_MOC_LITERAL(10, 89, 3), // "u32"
QT_MOC_LITERAL(11, 93, 7), // "address"
QT_MOC_LITERAL(12, 101, 7) // "process"

    },
    "MemorySearch\0setBarValue\0\0progress\0"
    "setBarRange\0minimum\0maximum\0searchResult\0"
    "found\0cancel\0u32\0address\0process"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MemorySearch[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,
       4,    2,   42,    2, 0x06 /* Public */,
       7,    3,   47,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    0,   54,    2, 0x0a /* Public */,
       9,    0,   55,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    5,    6,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool, 0x80000000 | 10,    8,    9,   11,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void MemorySearch::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MemorySearch *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setBarValue((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setBarRange((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->searchResult((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< u32(*)>(_a[3]))); break;
        case 3: _t->process(); break;
        case 4: _t->cancel(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MemorySearch::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MemorySearch::setBarValue)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MemorySearch::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MemorySearch::setBarRange)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (MemorySearch::*)(bool , bool , u32 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MemorySearch::searchResult)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MemorySearch::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_MemorySearch.data,
    qt_meta_data_MemorySearch,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MemorySearch::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MemorySearch::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MemorySearch.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MemorySearch::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void MemorySearch::setBarValue(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void MemorySearch::setBarRange(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void MemorySearch::searchResult(bool _t1, bool _t2, u32 _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
struct qt_meta_stringdata_UIMemoryEditor_t {
    QByteArrayData data[12];
    char stringdata0[171];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UIMemoryEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UIMemoryEditor_t qt_meta_stringdata_UIMemoryEditor = {
    {
QT_MOC_LITERAL(0, 0, 14), // "UIMemoryEditor"
QT_MOC_LITERAL(1, 15, 18), // "killProgressDialog"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 24), // "on_pbGotoAddress_clicked"
QT_MOC_LITERAL(4, 60, 25), // "on_pbSaveSelected_clicked"
QT_MOC_LITERAL(5, 86, 20), // "on_pbSaveTab_clicked"
QT_MOC_LITERAL(6, 107, 25), // "on_pbSearchMemory_clicked"
QT_MOC_LITERAL(7, 133, 12), // "searchResult"
QT_MOC_LITERAL(8, 146, 5), // "found"
QT_MOC_LITERAL(9, 152, 6), // "cancel"
QT_MOC_LITERAL(10, 159, 3), // "u32"
QT_MOC_LITERAL(11, 163, 7) // "address"

    },
    "UIMemoryEditor\0killProgressDialog\0\0"
    "on_pbGotoAddress_clicked\0"
    "on_pbSaveSelected_clicked\0"
    "on_pbSaveTab_clicked\0on_pbSearchMemory_clicked\0"
    "searchResult\0found\0cancel\0u32\0address"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UIMemoryEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   45,    2, 0x09 /* Protected */,
       4,    0,   46,    2, 0x09 /* Protected */,
       5,    0,   47,    2, 0x09 /* Protected */,
       6,    0,   48,    2, 0x09 /* Protected */,
       7,    3,   49,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool, 0x80000000 | 10,    8,    9,   11,

       0        // eod
};

void UIMemoryEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UIMemoryEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->killProgressDialog(); break;
        case 1: _t->on_pbGotoAddress_clicked(); break;
        case 2: _t->on_pbSaveSelected_clicked(); break;
        case 3: _t->on_pbSaveTab_clicked(); break;
        case 4: _t->on_pbSearchMemory_clicked(); break;
        case 5: _t->searchResult((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< u32(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (UIMemoryEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&UIMemoryEditor::killProgressDialog)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UIMemoryEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_UIMemoryEditor.data,
    qt_meta_data_UIMemoryEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UIMemoryEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UIMemoryEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UIMemoryEditor.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::UIMemoryEditor"))
        return static_cast< Ui::UIMemoryEditor*>(this);
    return QDialog::qt_metacast(_clname);
}

int UIMemoryEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void UIMemoryEditor::killProgressDialog()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
