/****************************************************************************
** Meta object code from reading C++ file 'UISettings.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/qt/ui/UISettings.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UISettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_UISettings_t {
    QByteArrayData data[12];
    char stringdata0[197];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UISettings_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UISettings_t qt_meta_stringdata_UISettings = {
    {
QT_MOC_LITERAL(0, 0, 10), // "UISettings"
QT_MOC_LITERAL(1, 11, 21), // "on_leBios_textChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 4), // "text"
QT_MOC_LITERAL(4, 39, 16), // "tbBrowse_clicked"
QT_MOC_LITERAL(5, 56, 30), // "on_cbInput_currentIndexChanged"
QT_MOC_LITERAL(6, 87, 2), // "id"
QT_MOC_LITERAL(7, 90, 30), // "on_cbCdRom_currentIndexChanged"
QT_MOC_LITERAL(8, 121, 27), // "on_cbClockSync_stateChanged"
QT_MOC_LITERAL(9, 149, 5), // "state"
QT_MOC_LITERAL(10, 155, 34), // "on_cbCartridge_currentIndexCh..."
QT_MOC_LITERAL(11, 190, 6) // "accept"

    },
    "UISettings\0on_leBios_textChanged\0\0"
    "text\0tbBrowse_clicked\0"
    "on_cbInput_currentIndexChanged\0id\0"
    "on_cbCdRom_currentIndexChanged\0"
    "on_cbClockSync_stateChanged\0state\0"
    "on_cbCartridge_currentIndexChanged\0"
    "accept"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UISettings[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x09 /* Protected */,
       4,    0,   52,    2, 0x09 /* Protected */,
       5,    1,   53,    2, 0x09 /* Protected */,
       7,    1,   56,    2, 0x09 /* Protected */,
       8,    1,   59,    2, 0x09 /* Protected */,
      10,    1,   62,    2, 0x09 /* Protected */,
      11,    0,   65,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void,

       0        // eod
};

void UISettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UISettings *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->on_leBios_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->tbBrowse_clicked(); break;
        case 2: _t->on_cbInput_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->on_cbCdRom_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->on_cbClockSync_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->on_cbCartridge_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->accept(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UISettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_UISettings.data,
    qt_meta_data_UISettings,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UISettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UISettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UISettings.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::UISettings"))
        return static_cast< Ui::UISettings*>(this);
    return QDialog::qt_metacast(_clname);
}

int UISettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
