/********************************************************************************
** Form generated from reading UI file 'UIDebugVDP1.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIDEBUGVDP1_H
#define UI_UIDEBUGVDP1_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIDebugVDP1
{
public:
    QVBoxLayout *verticalLayout_12;
    QHBoxLayout *horizontalLayout_6;
    QGroupBox *gbCommandList;
    QVBoxLayout *verticalLayout_10;
    QListWidget *lwCommandList;
    QGroupBox *gbCommandInfo;
    QVBoxLayout *verticalLayout_11;
    QPlainTextEdit *pteCommandInfo;
    QVBoxLayout *verticalLayout_9;
    QGroupBox *gbTexture;
    QVBoxLayout *verticalLayout_5;
    QGraphicsView *gvTexture;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *pbSaveBitmap;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout_7;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UIDebugVDP1)
    {
        if (UIDebugVDP1->objectName().isEmpty())
            UIDebugVDP1->setObjectName(QString::fromUtf8("UIDebugVDP1"));
        UIDebugVDP1->resize(715, 367);
        verticalLayout_12 = new QVBoxLayout(UIDebugVDP1);
        verticalLayout_12->setObjectName(QString::fromUtf8("verticalLayout_12"));
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        gbCommandList = new QGroupBox(UIDebugVDP1);
        gbCommandList->setObjectName(QString::fromUtf8("gbCommandList"));
        verticalLayout_10 = new QVBoxLayout(gbCommandList);
        verticalLayout_10->setObjectName(QString::fromUtf8("verticalLayout_10"));
        lwCommandList = new QListWidget(gbCommandList);
        lwCommandList->setObjectName(QString::fromUtf8("lwCommandList"));

        verticalLayout_10->addWidget(lwCommandList);


        horizontalLayout_6->addWidget(gbCommandList);

        gbCommandInfo = new QGroupBox(UIDebugVDP1);
        gbCommandInfo->setObjectName(QString::fromUtf8("gbCommandInfo"));
        verticalLayout_11 = new QVBoxLayout(gbCommandInfo);
        verticalLayout_11->setObjectName(QString::fromUtf8("verticalLayout_11"));
        pteCommandInfo = new QPlainTextEdit(gbCommandInfo);
        pteCommandInfo->setObjectName(QString::fromUtf8("pteCommandInfo"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pteCommandInfo->sizePolicy().hasHeightForWidth());
        pteCommandInfo->setSizePolicy(sizePolicy);
        pteCommandInfo->setReadOnly(true);

        verticalLayout_11->addWidget(pteCommandInfo);


        horizontalLayout_6->addWidget(gbCommandInfo);

        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        gbTexture = new QGroupBox(UIDebugVDP1);
        gbTexture->setObjectName(QString::fromUtf8("gbTexture"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(gbTexture->sizePolicy().hasHeightForWidth());
        gbTexture->setSizePolicy(sizePolicy1);
        verticalLayout_5 = new QVBoxLayout(gbTexture);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        gvTexture = new QGraphicsView(gbTexture);
        gvTexture->setObjectName(QString::fromUtf8("gvTexture"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(gvTexture->sizePolicy().hasHeightForWidth());
        gvTexture->setSizePolicy(sizePolicy2);
        gvTexture->setSizeIncrement(QSize(0, 0));

        verticalLayout_5->addWidget(gvTexture);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_5->addItem(verticalSpacer);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        pbSaveBitmap = new QPushButton(gbTexture);
        pbSaveBitmap->setObjectName(QString::fromUtf8("pbSaveBitmap"));

        horizontalLayout_2->addWidget(pbSaveBitmap);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_5->addLayout(horizontalLayout_2);


        verticalLayout_9->addWidget(gbTexture);


        horizontalLayout_6->addLayout(verticalLayout_9);


        verticalLayout_12->addLayout(horizontalLayout_6);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        dbbButtons = new QDialogButtonBox(UIDebugVDP1);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout_7->addWidget(dbbButtons);


        verticalLayout_12->addLayout(horizontalLayout_7);


        retranslateUi(UIDebugVDP1);
        QObject::connect(dbbButtons, SIGNAL(rejected()), UIDebugVDP1, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIDebugVDP1);
    } // setupUi

    void retranslateUi(QDialog *UIDebugVDP1)
    {
        UIDebugVDP1->setWindowTitle(QCoreApplication::translate("UIDebugVDP1", "Debug VDP1", nullptr));
        gbCommandList->setTitle(QCoreApplication::translate("UIDebugVDP1", "VDP1 Command List", nullptr));
        gbCommandInfo->setTitle(QCoreApplication::translate("UIDebugVDP1", "VDP1 Command Info", nullptr));
        gbTexture->setTitle(QCoreApplication::translate("UIDebugVDP1", "Texture", nullptr));
        pbSaveBitmap->setText(QCoreApplication::translate("UIDebugVDP1", "Save Bitmap", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIDebugVDP1: public Ui_UIDebugVDP1 {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIDEBUGVDP1_H
