/****************************************************************************
** Meta object code from reading C++ file 'YabauseThread.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../src/qt/YabauseThread.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'YabauseThread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_YabauseThread_t {
    QByteArrayData data[20];
    char stringdata0[223];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_YabauseThread_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_YabauseThread_t qt_meta_stringdata_YabauseThread = {
    {
QT_MOC_LITERAL(0, 0, 13), // "YabauseThread"
QT_MOC_LITERAL(1, 14, 11), // "requestSize"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 4), // "size"
QT_MOC_LITERAL(4, 32, 17), // "requestFullscreen"
QT_MOC_LITERAL(5, 50, 10), // "fullscreen"
QT_MOC_LITERAL(6, 61, 19), // "requestVolumeChange"
QT_MOC_LITERAL(7, 81, 6), // "volume"
QT_MOC_LITERAL(8, 88, 5), // "error"
QT_MOC_LITERAL(9, 94, 8), // "internal"
QT_MOC_LITERAL(10, 103, 5), // "pause"
QT_MOC_LITERAL(11, 109, 6), // "paused"
QT_MOC_LITERAL(12, 116, 5), // "reset"
QT_MOC_LITERAL(13, 122, 18), // "toggleEmulateMouse"
QT_MOC_LITERAL(14, 141, 6), // "enable"
QT_MOC_LITERAL(15, 148, 14), // "pauseEmulation"
QT_MOC_LITERAL(16, 163, 14), // "resetEmulation"
QT_MOC_LITERAL(17, 178, 17), // "reloadControllers"
QT_MOC_LITERAL(18, 196, 11), // "reloadClock"
QT_MOC_LITERAL(19, 208, 14) // "reloadSettings"

    },
    "YabauseThread\0requestSize\0\0size\0"
    "requestFullscreen\0fullscreen\0"
    "requestVolumeChange\0volume\0error\0"
    "internal\0pause\0paused\0reset\0"
    "toggleEmulateMouse\0enable\0pauseEmulation\0"
    "resetEmulation\0reloadControllers\0"
    "reloadClock\0reloadSettings"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_YabauseThread[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,
       4,    1,   82,    2, 0x06 /* Public */,
       6,    1,   85,    2, 0x06 /* Public */,
       8,    2,   88,    2, 0x06 /* Public */,
       8,    1,   93,    2, 0x26 /* Public | MethodCloned */,
      10,    1,   96,    2, 0x06 /* Public */,
      12,    0,   99,    2, 0x06 /* Public */,
      13,    1,  100,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    2,  103,    2, 0x0a /* Public */,
      16,    0,  108,    2, 0x0a /* Public */,
      17,    0,  109,    2, 0x0a /* Public */,
      18,    0,  110,    2, 0x0a /* Public */,
      19,    0,  111,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QSize,    3,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    8,    9,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   14,

 // slots: parameters
    QMetaType::Bool, QMetaType::Bool, QMetaType::Bool,   10,   12,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void YabauseThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<YabauseThread *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->requestSize((*reinterpret_cast< const QSize(*)>(_a[1]))); break;
        case 1: _t->requestFullscreen((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->requestVolumeChange((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->error((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->pause((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->reset(); break;
        case 7: _t->toggleEmulateMouse((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: { bool _r = _t->pauseEmulation((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->resetEmulation();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->reloadControllers(); break;
        case 11: _t->reloadClock(); break;
        case 12: _t->reloadSettings(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (YabauseThread::*)(const QSize & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YabauseThread::requestSize)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (YabauseThread::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YabauseThread::requestFullscreen)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (YabauseThread::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YabauseThread::requestVolumeChange)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (YabauseThread::*)(const QString & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YabauseThread::error)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (YabauseThread::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YabauseThread::pause)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (YabauseThread::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YabauseThread::reset)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (YabauseThread::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&YabauseThread::toggleEmulateMouse)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject YabauseThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_YabauseThread.data,
    qt_meta_data_YabauseThread,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *YabauseThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *YabauseThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_YabauseThread.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int YabauseThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void YabauseThread::requestSize(const QSize & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void YabauseThread::requestFullscreen(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void YabauseThread::requestVolumeChange(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void YabauseThread::error(const QString & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 5
void YabauseThread::pause(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void YabauseThread::reset()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void YabauseThread::toggleEmulateMouse(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
