/********************************************************************************
** Form generated from reading UI file 'UIHexInput.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIHEXINPUT_H
#define UI_UIHEXINPUT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIHexInput
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *lInformations;
    QLineEdit *leValue;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UIHexInput)
    {
        if (UIHexInput->objectName().isEmpty())
            UIHexInput->setObjectName(QString::fromUtf8("UIHexInput"));
        UIHexInput->resize(255, 86);
        vboxLayout = new QVBoxLayout(UIHexInput);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        lInformations = new QLabel(UIHexInput);
        lInformations->setObjectName(QString::fromUtf8("lInformations"));

        vboxLayout->addWidget(lInformations);

        leValue = new QLineEdit(UIHexInput);
        leValue->setObjectName(QString::fromUtf8("leValue"));

        vboxLayout->addWidget(leValue);

        dbbButtons = new QDialogButtonBox(UIHexInput);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setOrientation(Qt::Horizontal);
        dbbButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        dbbButtons->setCenterButtons(true);

        vboxLayout->addWidget(dbbButtons);


        retranslateUi(UIHexInput);
        QObject::connect(dbbButtons, SIGNAL(accepted()), UIHexInput, SLOT(accept()));
        QObject::connect(dbbButtons, SIGNAL(rejected()), UIHexInput, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIHexInput);
    } // setupUi

    void retranslateUi(QDialog *UIHexInput)
    {
        UIHexInput->setWindowTitle(QCoreApplication::translate("UIHexInput", "Enter New Value", nullptr));
        lInformations->setText(QCoreApplication::translate("UIHexInput", "Value:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIHexInput: public Ui_UIHexInput {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIHEXINPUT_H
