/********************************************************************************
** Form generated from reading UI file 'UIDebugVDP2Viewer.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIDEBUGVDP2VIEWER_H
#define UI_UIDEBUGVDP2VIEWER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIDebugVDP2Viewer
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    QComboBox *cbScreen;
    QSpacerItem *horizontalSpacer;
    QGraphicsView *gvScreen;
    QHBoxLayout *horizontalLayout;
    QPushButton *pbSaveAsBitmap;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UIDebugVDP2Viewer)
    {
        if (UIDebugVDP2Viewer->objectName().isEmpty())
            UIDebugVDP2Viewer->setObjectName(QString::fromUtf8("UIDebugVDP2Viewer"));
        UIDebugVDP2Viewer->resize(537, 511);
        verticalLayout = new QVBoxLayout(UIDebugVDP2Viewer);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label = new QLabel(UIDebugVDP2Viewer);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout_2->addWidget(label);

        cbScreen = new QComboBox(UIDebugVDP2Viewer);
        cbScreen->setObjectName(QString::fromUtf8("cbScreen"));

        horizontalLayout_2->addWidget(cbScreen);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_2);

        gvScreen = new QGraphicsView(UIDebugVDP2Viewer);
        gvScreen->setObjectName(QString::fromUtf8("gvScreen"));

        verticalLayout->addWidget(gvScreen);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        pbSaveAsBitmap = new QPushButton(UIDebugVDP2Viewer);
        pbSaveAsBitmap->setObjectName(QString::fromUtf8("pbSaveAsBitmap"));

        horizontalLayout->addWidget(pbSaveAsBitmap);

        dbbButtons = new QDialogButtonBox(UIDebugVDP2Viewer);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout->addWidget(dbbButtons);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(UIDebugVDP2Viewer);
        QObject::connect(dbbButtons, SIGNAL(rejected()), UIDebugVDP2Viewer, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIDebugVDP2Viewer);
    } // setupUi

    void retranslateUi(QDialog *UIDebugVDP2Viewer)
    {
        UIDebugVDP2Viewer->setWindowTitle(QCoreApplication::translate("UIDebugVDP2Viewer", "VDP2Viewer Viewer", nullptr));
        label->setText(QCoreApplication::translate("UIDebugVDP2Viewer", "Screen Select:", nullptr));
        pbSaveAsBitmap->setText(QCoreApplication::translate("UIDebugVDP2Viewer", "Save As Bitmap", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIDebugVDP2Viewer: public Ui_UIDebugVDP2Viewer {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIDEBUGVDP2VIEWER_H
