/********************************************************************************
** Form generated from reading UI file 'UICheatAR.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UICHEATAR_H
#define UI_UICHEATAR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTextEdit>

QT_BEGIN_NAMESPACE

class Ui_UICheatAR
{
public:
    QGridLayout *gridLayout;
    QLabel *lCode;
    QLineEdit *leCode;
    QLabel *lDescription;
    QTextEdit *teDescription;
    QDialogButtonBox *dbbButtons;
    QLabel *lMasterCodesInformations;

    void setupUi(QDialog *UICheatAR)
    {
        if (UICheatAR->objectName().isEmpty())
            UICheatAR->setObjectName(QString::fromUtf8("UICheatAR"));
        UICheatAR->resize(400, 160);
        gridLayout = new QGridLayout(UICheatAR);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        lCode = new QLabel(UICheatAR);
        lCode->setObjectName(QString::fromUtf8("lCode"));

        gridLayout->addWidget(lCode, 1, 0, 1, 1);

        leCode = new QLineEdit(UICheatAR);
        leCode->setObjectName(QString::fromUtf8("leCode"));
        leCode->setMaxLength(13);

        gridLayout->addWidget(leCode, 1, 1, 1, 1);

        lDescription = new QLabel(UICheatAR);
        lDescription->setObjectName(QString::fromUtf8("lDescription"));
        lDescription->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(lDescription, 2, 0, 1, 1);

        teDescription = new QTextEdit(UICheatAR);
        teDescription->setObjectName(QString::fromUtf8("teDescription"));
        teDescription->setAcceptRichText(false);

        gridLayout->addWidget(teDescription, 2, 1, 1, 1);

        dbbButtons = new QDialogButtonBox(UICheatAR);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setOrientation(Qt::Horizontal);
        dbbButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::NoButton|QDialogButtonBox::Ok);

        gridLayout->addWidget(dbbButtons, 3, 0, 1, 2);

        lMasterCodesInformations = new QLabel(UICheatAR);
        lMasterCodesInformations->setObjectName(QString::fromUtf8("lMasterCodesInformations"));
        lMasterCodesInformations->setTextFormat(Qt::RichText);
        lMasterCodesInformations->setAlignment(Qt::AlignCenter);
        lMasterCodesInformations->setWordWrap(true);

        gridLayout->addWidget(lMasterCodesInformations, 0, 0, 1, 2);


        retranslateUi(UICheatAR);
        QObject::connect(dbbButtons, SIGNAL(accepted()), UICheatAR, SLOT(accept()));
        QObject::connect(dbbButtons, SIGNAL(rejected()), UICheatAR, SLOT(reject()));

        QMetaObject::connectSlotsByName(UICheatAR);
    } // setupUi

    void retranslateUi(QDialog *UICheatAR)
    {
        UICheatAR->setWindowTitle(QCoreApplication::translate("UICheatAR", "Add Action Replay Code", nullptr));
        lCode->setText(QCoreApplication::translate("UICheatAR", "Action Replay Code :", nullptr));
        lDescription->setText(QCoreApplication::translate("UICheatAR", "Description :", nullptr));
        lMasterCodesInformations->setText(QCoreApplication::translate("UICheatAR", "<font color=\"red\"><b>WARNING:</b> Master Codes are <b>NOT</b> supported.</font>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UICheatAR: public Ui_UICheatAR {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UICHEATAR_H
