/********************************************************************************
** Form generated from reading UI file 'UIPortManager.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIPORTMANAGER_H
#define UI_UIPORTMANAGER_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIPortManager
{
public:
    QVBoxLayout *verticalLayout;
    QFrame *fController1;
    QHBoxLayout *horizontalLayout;
    QLabel *lJoystick1;
    QComboBox *cbTypeController1;
    QToolButton *tbSetJoystick1;
    QToolButton *tbClearJoystick1;
    QToolButton *tbRemoveJoystick1;
    QFrame *fController2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *lJoystick2;
    QComboBox *cbTypeController2;
    QToolButton *tbSetJoystick2;
    QToolButton *tbClearJoystick2;
    QToolButton *tbRemoveJoystick2;
    QFrame *fController3;
    QHBoxLayout *horizontalLayout_3;
    QLabel *lJoystick3;
    QComboBox *cbTypeController3;
    QToolButton *tbSetJoystick3;
    QToolButton *tbClearJoystick3;
    QToolButton *tbRemoveJoystick3;
    QFrame *fController4;
    QHBoxLayout *horizontalLayout_4;
    QLabel *lJoystick4;
    QComboBox *cbTypeController4;
    QToolButton *tbSetJoystick4;
    QToolButton *tbClearJoystick4;
    QToolButton *tbRemoveJoystick4;
    QFrame *fController5;
    QHBoxLayout *horizontalLayout_5;
    QLabel *lJoystick5;
    QComboBox *cbTypeController5;
    QToolButton *tbSetJoystick5;
    QToolButton *tbClearJoystick5;
    QToolButton *tbRemoveJoystick5;
    QFrame *fController6;
    QHBoxLayout *horizontalLayout_6;
    QLabel *lJoystick6;
    QComboBox *cbTypeController6;
    QToolButton *tbSetJoystick6;
    QToolButton *tbClearJoystick6;
    QToolButton *tbRemoveJoystick6;

    void setupUi(QGroupBox *UIPortManager)
    {
        if (UIPortManager->objectName().isEmpty())
            UIPortManager->setObjectName(QString::fromUtf8("UIPortManager"));
        UIPortManager->resize(277, 302);
        verticalLayout = new QVBoxLayout(UIPortManager);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        fController1 = new QFrame(UIPortManager);
        fController1->setObjectName(QString::fromUtf8("fController1"));
        fController1->setFrameShape(QFrame::StyledPanel);
        fController1->setFrameShadow(QFrame::Raised);
        horizontalLayout = new QHBoxLayout(fController1);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        lJoystick1 = new QLabel(fController1);
        lJoystick1->setObjectName(QString::fromUtf8("lJoystick1"));
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lJoystick1->sizePolicy().hasHeightForWidth());
        lJoystick1->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(lJoystick1);

        cbTypeController1 = new QComboBox(fController1);
        cbTypeController1->setObjectName(QString::fromUtf8("cbTypeController1"));

        horizontalLayout->addWidget(cbTypeController1);

        tbSetJoystick1 = new QToolButton(fController1);
        tbSetJoystick1->setObjectName(QString::fromUtf8("tbSetJoystick1"));
        tbSetJoystick1->setEnabled(false);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/actions/icons/actions/settings.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbSetJoystick1->setIcon(icon);
        tbSetJoystick1->setAutoRaise(true);

        horizontalLayout->addWidget(tbSetJoystick1);

        tbClearJoystick1 = new QToolButton(fController1);
        tbClearJoystick1->setObjectName(QString::fromUtf8("tbClearJoystick1"));
        tbClearJoystick1->setEnabled(false);
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/actions/icons/actions/reset.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbClearJoystick1->setIcon(icon1);
        tbClearJoystick1->setAutoRaise(true);

        horizontalLayout->addWidget(tbClearJoystick1);

        tbRemoveJoystick1 = new QToolButton(fController1);
        tbRemoveJoystick1->setObjectName(QString::fromUtf8("tbRemoveJoystick1"));
        tbRemoveJoystick1->setEnabled(false);
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/actions/icons/actions/button_cancel.png"), QSize(), QIcon::Normal, QIcon::Off);
        tbRemoveJoystick1->setIcon(icon2);
        tbRemoveJoystick1->setAutoRaise(true);

        horizontalLayout->addWidget(tbRemoveJoystick1);


        verticalLayout->addWidget(fController1);

        fController2 = new QFrame(UIPortManager);
        fController2->setObjectName(QString::fromUtf8("fController2"));
        fController2->setFrameShape(QFrame::StyledPanel);
        fController2->setFrameShadow(QFrame::Raised);
        horizontalLayout_2 = new QHBoxLayout(fController2);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        lJoystick2 = new QLabel(fController2);
        lJoystick2->setObjectName(QString::fromUtf8("lJoystick2"));
        sizePolicy.setHeightForWidth(lJoystick2->sizePolicy().hasHeightForWidth());
        lJoystick2->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(lJoystick2);

        cbTypeController2 = new QComboBox(fController2);
        cbTypeController2->setObjectName(QString::fromUtf8("cbTypeController2"));

        horizontalLayout_2->addWidget(cbTypeController2);

        tbSetJoystick2 = new QToolButton(fController2);
        tbSetJoystick2->setObjectName(QString::fromUtf8("tbSetJoystick2"));
        tbSetJoystick2->setEnabled(false);
        tbSetJoystick2->setIcon(icon);
        tbSetJoystick2->setAutoRaise(true);

        horizontalLayout_2->addWidget(tbSetJoystick2);

        tbClearJoystick2 = new QToolButton(fController2);
        tbClearJoystick2->setObjectName(QString::fromUtf8("tbClearJoystick2"));
        tbClearJoystick2->setEnabled(false);
        tbClearJoystick2->setIcon(icon1);
        tbClearJoystick2->setAutoRaise(true);

        horizontalLayout_2->addWidget(tbClearJoystick2);

        tbRemoveJoystick2 = new QToolButton(fController2);
        tbRemoveJoystick2->setObjectName(QString::fromUtf8("tbRemoveJoystick2"));
        tbRemoveJoystick2->setEnabled(false);
        tbRemoveJoystick2->setIcon(icon2);
        tbRemoveJoystick2->setAutoRaise(true);

        horizontalLayout_2->addWidget(tbRemoveJoystick2);


        verticalLayout->addWidget(fController2);

        fController3 = new QFrame(UIPortManager);
        fController3->setObjectName(QString::fromUtf8("fController3"));
        fController3->setEnabled(true);
        fController3->setFrameShape(QFrame::StyledPanel);
        fController3->setFrameShadow(QFrame::Raised);
        horizontalLayout_3 = new QHBoxLayout(fController3);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        lJoystick3 = new QLabel(fController3);
        lJoystick3->setObjectName(QString::fromUtf8("lJoystick3"));
        sizePolicy.setHeightForWidth(lJoystick3->sizePolicy().hasHeightForWidth());
        lJoystick3->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(lJoystick3);

        cbTypeController3 = new QComboBox(fController3);
        cbTypeController3->setObjectName(QString::fromUtf8("cbTypeController3"));
        cbTypeController3->setEnabled(true);

        horizontalLayout_3->addWidget(cbTypeController3);

        tbSetJoystick3 = new QToolButton(fController3);
        tbSetJoystick3->setObjectName(QString::fromUtf8("tbSetJoystick3"));
        tbSetJoystick3->setEnabled(false);
        tbSetJoystick3->setIcon(icon);
        tbSetJoystick3->setAutoRaise(true);

        horizontalLayout_3->addWidget(tbSetJoystick3);

        tbClearJoystick3 = new QToolButton(fController3);
        tbClearJoystick3->setObjectName(QString::fromUtf8("tbClearJoystick3"));
        tbClearJoystick3->setEnabled(false);
        tbClearJoystick3->setIcon(icon1);
        tbClearJoystick3->setAutoRaise(true);

        horizontalLayout_3->addWidget(tbClearJoystick3);

        tbRemoveJoystick3 = new QToolButton(fController3);
        tbRemoveJoystick3->setObjectName(QString::fromUtf8("tbRemoveJoystick3"));
        tbRemoveJoystick3->setEnabled(false);
        tbRemoveJoystick3->setIcon(icon2);
        tbRemoveJoystick3->setAutoRaise(true);

        horizontalLayout_3->addWidget(tbRemoveJoystick3);


        verticalLayout->addWidget(fController3);

        fController4 = new QFrame(UIPortManager);
        fController4->setObjectName(QString::fromUtf8("fController4"));
        fController4->setFrameShape(QFrame::StyledPanel);
        fController4->setFrameShadow(QFrame::Raised);
        horizontalLayout_4 = new QHBoxLayout(fController4);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        lJoystick4 = new QLabel(fController4);
        lJoystick4->setObjectName(QString::fromUtf8("lJoystick4"));
        sizePolicy.setHeightForWidth(lJoystick4->sizePolicy().hasHeightForWidth());
        lJoystick4->setSizePolicy(sizePolicy);

        horizontalLayout_4->addWidget(lJoystick4);

        cbTypeController4 = new QComboBox(fController4);
        cbTypeController4->setObjectName(QString::fromUtf8("cbTypeController4"));

        horizontalLayout_4->addWidget(cbTypeController4);

        tbSetJoystick4 = new QToolButton(fController4);
        tbSetJoystick4->setObjectName(QString::fromUtf8("tbSetJoystick4"));
        tbSetJoystick4->setEnabled(false);
        tbSetJoystick4->setIcon(icon);
        tbSetJoystick4->setAutoRaise(true);

        horizontalLayout_4->addWidget(tbSetJoystick4);

        tbClearJoystick4 = new QToolButton(fController4);
        tbClearJoystick4->setObjectName(QString::fromUtf8("tbClearJoystick4"));
        tbClearJoystick4->setEnabled(false);
        tbClearJoystick4->setIcon(icon1);
        tbClearJoystick4->setAutoRaise(true);

        horizontalLayout_4->addWidget(tbClearJoystick4);

        tbRemoveJoystick4 = new QToolButton(fController4);
        tbRemoveJoystick4->setObjectName(QString::fromUtf8("tbRemoveJoystick4"));
        tbRemoveJoystick4->setEnabled(false);
        tbRemoveJoystick4->setIcon(icon2);
        tbRemoveJoystick4->setAutoRaise(true);

        horizontalLayout_4->addWidget(tbRemoveJoystick4);


        verticalLayout->addWidget(fController4);

        fController5 = new QFrame(UIPortManager);
        fController5->setObjectName(QString::fromUtf8("fController5"));
        fController5->setFrameShape(QFrame::StyledPanel);
        fController5->setFrameShadow(QFrame::Raised);
        horizontalLayout_5 = new QHBoxLayout(fController5);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        lJoystick5 = new QLabel(fController5);
        lJoystick5->setObjectName(QString::fromUtf8("lJoystick5"));
        sizePolicy.setHeightForWidth(lJoystick5->sizePolicy().hasHeightForWidth());
        lJoystick5->setSizePolicy(sizePolicy);

        horizontalLayout_5->addWidget(lJoystick5);

        cbTypeController5 = new QComboBox(fController5);
        cbTypeController5->setObjectName(QString::fromUtf8("cbTypeController5"));

        horizontalLayout_5->addWidget(cbTypeController5);

        tbSetJoystick5 = new QToolButton(fController5);
        tbSetJoystick5->setObjectName(QString::fromUtf8("tbSetJoystick5"));
        tbSetJoystick5->setEnabled(false);
        tbSetJoystick5->setIcon(icon);
        tbSetJoystick5->setAutoRaise(true);

        horizontalLayout_5->addWidget(tbSetJoystick5);

        tbClearJoystick5 = new QToolButton(fController5);
        tbClearJoystick5->setObjectName(QString::fromUtf8("tbClearJoystick5"));
        tbClearJoystick5->setEnabled(false);
        tbClearJoystick5->setIcon(icon1);
        tbClearJoystick5->setAutoRaise(true);

        horizontalLayout_5->addWidget(tbClearJoystick5);

        tbRemoveJoystick5 = new QToolButton(fController5);
        tbRemoveJoystick5->setObjectName(QString::fromUtf8("tbRemoveJoystick5"));
        tbRemoveJoystick5->setEnabled(false);
        tbRemoveJoystick5->setIcon(icon2);
        tbRemoveJoystick5->setAutoRaise(true);

        horizontalLayout_5->addWidget(tbRemoveJoystick5);


        verticalLayout->addWidget(fController5);

        fController6 = new QFrame(UIPortManager);
        fController6->setObjectName(QString::fromUtf8("fController6"));
        fController6->setFrameShape(QFrame::StyledPanel);
        fController6->setFrameShadow(QFrame::Raised);
        horizontalLayout_6 = new QHBoxLayout(fController6);
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        lJoystick6 = new QLabel(fController6);
        lJoystick6->setObjectName(QString::fromUtf8("lJoystick6"));
        sizePolicy.setHeightForWidth(lJoystick6->sizePolicy().hasHeightForWidth());
        lJoystick6->setSizePolicy(sizePolicy);

        horizontalLayout_6->addWidget(lJoystick6);

        cbTypeController6 = new QComboBox(fController6);
        cbTypeController6->setObjectName(QString::fromUtf8("cbTypeController6"));

        horizontalLayout_6->addWidget(cbTypeController6);

        tbSetJoystick6 = new QToolButton(fController6);
        tbSetJoystick6->setObjectName(QString::fromUtf8("tbSetJoystick6"));
        tbSetJoystick6->setEnabled(false);
        tbSetJoystick6->setIcon(icon);
        tbSetJoystick6->setAutoRaise(true);

        horizontalLayout_6->addWidget(tbSetJoystick6);

        tbClearJoystick6 = new QToolButton(fController6);
        tbClearJoystick6->setObjectName(QString::fromUtf8("tbClearJoystick6"));
        tbClearJoystick6->setEnabled(false);
        tbClearJoystick6->setIcon(icon1);
        tbClearJoystick6->setAutoRaise(true);

        horizontalLayout_6->addWidget(tbClearJoystick6);

        tbRemoveJoystick6 = new QToolButton(fController6);
        tbRemoveJoystick6->setObjectName(QString::fromUtf8("tbRemoveJoystick6"));
        tbRemoveJoystick6->setEnabled(false);
        tbRemoveJoystick6->setIcon(icon2);
        tbRemoveJoystick6->setAutoRaise(true);

        horizontalLayout_6->addWidget(tbRemoveJoystick6);


        verticalLayout->addWidget(fController6);


        retranslateUi(UIPortManager);

        QMetaObject::connectSlotsByName(UIPortManager);
    } // setupUi

    void retranslateUi(QGroupBox *UIPortManager)
    {
        lJoystick1->setText(QCoreApplication::translate("UIPortManager", "1", nullptr));
#if QT_CONFIG(tooltip)
        tbSetJoystick1->setToolTip(QCoreApplication::translate("UIPortManager", "Edit configuration", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbClearJoystick1->setToolTip(QCoreApplication::translate("UIPortManager", "Clear configuration", nullptr));
#endif // QT_CONFIG(tooltip)
        tbClearJoystick1->setText(QCoreApplication::translate("UIPortManager", "...", nullptr));
#if QT_CONFIG(tooltip)
        tbRemoveJoystick1->setToolTip(QCoreApplication::translate("UIPortManager", "Remove Peripheral", nullptr));
#endif // QT_CONFIG(tooltip)
        lJoystick2->setText(QCoreApplication::translate("UIPortManager", "2", nullptr));
#if QT_CONFIG(tooltip)
        tbSetJoystick2->setToolTip(QCoreApplication::translate("UIPortManager", "Edit configuration", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbClearJoystick2->setToolTip(QCoreApplication::translate("UIPortManager", "Clear configuration", nullptr));
#endif // QT_CONFIG(tooltip)
        tbClearJoystick2->setText(QCoreApplication::translate("UIPortManager", "...", nullptr));
#if QT_CONFIG(tooltip)
        tbRemoveJoystick2->setToolTip(QCoreApplication::translate("UIPortManager", "Remove Peripheral", nullptr));
#endif // QT_CONFIG(tooltip)
        lJoystick3->setText(QCoreApplication::translate("UIPortManager", "3", nullptr));
#if QT_CONFIG(tooltip)
        tbSetJoystick3->setToolTip(QCoreApplication::translate("UIPortManager", "Edit configuration", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbClearJoystick3->setToolTip(QCoreApplication::translate("UIPortManager", "Clear configuration", nullptr));
#endif // QT_CONFIG(tooltip)
        tbClearJoystick3->setText(QCoreApplication::translate("UIPortManager", "...", nullptr));
#if QT_CONFIG(tooltip)
        tbRemoveJoystick3->setToolTip(QCoreApplication::translate("UIPortManager", "Remove Peripheral", nullptr));
#endif // QT_CONFIG(tooltip)
        lJoystick4->setText(QCoreApplication::translate("UIPortManager", "4", nullptr));
#if QT_CONFIG(tooltip)
        tbSetJoystick4->setToolTip(QCoreApplication::translate("UIPortManager", "Edit configuration", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbClearJoystick4->setToolTip(QCoreApplication::translate("UIPortManager", "Clear configuration", nullptr));
#endif // QT_CONFIG(tooltip)
        tbClearJoystick4->setText(QCoreApplication::translate("UIPortManager", "...", nullptr));
#if QT_CONFIG(tooltip)
        tbRemoveJoystick4->setToolTip(QCoreApplication::translate("UIPortManager", "Remove Peripheral", nullptr));
#endif // QT_CONFIG(tooltip)
        lJoystick5->setText(QCoreApplication::translate("UIPortManager", "5", nullptr));
#if QT_CONFIG(tooltip)
        tbSetJoystick5->setToolTip(QCoreApplication::translate("UIPortManager", "Edit configuration", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbClearJoystick5->setToolTip(QCoreApplication::translate("UIPortManager", "Clear configuration", nullptr));
#endif // QT_CONFIG(tooltip)
        tbClearJoystick5->setText(QCoreApplication::translate("UIPortManager", "...", nullptr));
#if QT_CONFIG(tooltip)
        tbRemoveJoystick5->setToolTip(QCoreApplication::translate("UIPortManager", "Remove Peripheral", nullptr));
#endif // QT_CONFIG(tooltip)
        lJoystick6->setText(QCoreApplication::translate("UIPortManager", "6", nullptr));
#if QT_CONFIG(tooltip)
        tbSetJoystick6->setToolTip(QCoreApplication::translate("UIPortManager", "Edit configuration", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbClearJoystick6->setToolTip(QCoreApplication::translate("UIPortManager", "Clear configuration", nullptr));
#endif // QT_CONFIG(tooltip)
        tbClearJoystick6->setText(QCoreApplication::translate("UIPortManager", "...", nullptr));
#if QT_CONFIG(tooltip)
        tbRemoveJoystick6->setToolTip(QCoreApplication::translate("UIPortManager", "Remove Peripheral", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)UIPortManager;
    } // retranslateUi

};

namespace Ui {
    class UIPortManager: public Ui_UIPortManager {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIPORTMANAGER_H
