/****************************************************************************
** Meta object code from reading C++ file 'UIDebugCPU.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/qt/ui/UIDebugCPU.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UIDebugCPU.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_UIDebugCPU_t {
    QByteArrayData data[39];
    char stringdata0[882];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UIDebugCPU_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UIDebugCPU_t qt_meta_stringdata_UIDebugCPU = {
    {
QT_MOC_LITERAL(0, 0, 10), // "UIDebugCPU"
QT_MOC_LITERAL(1, 11, 32), // "on_lwRegisters_itemDoubleClicked"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(4, 62, 4), // "item"
QT_MOC_LITERAL(5, 67, 32), // "on_lwBackTrace_itemDoubleClicked"
QT_MOC_LITERAL(6, 100, 35), // "on_twTrackInfLoop_itemDoubleC..."
QT_MOC_LITERAL(7, 136, 17), // "QTableWidgetItem*"
QT_MOC_LITERAL(8, 154, 31), // "on_leCodeBreakpoint_textChanged"
QT_MOC_LITERAL(9, 186, 4), // "text"
QT_MOC_LITERAL(10, 191, 33), // "on_leMemoryBreakpoint_textCha..."
QT_MOC_LITERAL(11, 225, 41), // "on_lwCodeBreakpoints_itemSele..."
QT_MOC_LITERAL(12, 267, 30), // "on_pbAddCodeBreakpoint_clicked"
QT_MOC_LITERAL(13, 298, 30), // "on_pbDelCodeBreakpoint_clicked"
QT_MOC_LITERAL(14, 329, 43), // "on_lwMemoryBreakpoints_itemSe..."
QT_MOC_LITERAL(15, 373, 32), // "on_pbAddMemoryBreakpoint_clicked"
QT_MOC_LITERAL(16, 406, 32), // "on_pbDelMemoryBreakpoint_clicked"
QT_MOC_LITERAL(17, 439, 17), // "on_cbRead_toggled"
QT_MOC_LITERAL(18, 457, 6), // "enable"
QT_MOC_LITERAL(19, 464, 18), // "on_cbWrite_toggled"
QT_MOC_LITERAL(20, 483, 21), // "on_cbReadByte_toggled"
QT_MOC_LITERAL(21, 505, 21), // "on_cbReadWord_toggled"
QT_MOC_LITERAL(22, 527, 21), // "on_cbReadLong_toggled"
QT_MOC_LITERAL(23, 549, 22), // "on_cbWriteByte_toggled"
QT_MOC_LITERAL(24, 572, 22), // "on_cbWriteWord_toggled"
QT_MOC_LITERAL(25, 595, 22), // "on_cbWriteLong_toggled"
QT_MOC_LITERAL(26, 618, 21), // "on_pbStepInto_clicked"
QT_MOC_LITERAL(27, 640, 21), // "on_pbStepOver_clicked"
QT_MOC_LITERAL(28, 662, 20), // "on_pbStepOut_clicked"
QT_MOC_LITERAL(29, 683, 27), // "on_pbMemoryTransfer_clicked"
QT_MOC_LITERAL(30, 711, 25), // "on_pbMemoryEditor_clicked"
QT_MOC_LITERAL(31, 737, 22), // "on_pbReserved1_clicked"
QT_MOC_LITERAL(32, 760, 22), // "on_pbReserved2_clicked"
QT_MOC_LITERAL(33, 783, 22), // "on_pbReserved3_clicked"
QT_MOC_LITERAL(34, 806, 22), // "on_pbReserved4_clicked"
QT_MOC_LITERAL(35, 829, 22), // "on_pbReserved5_clicked"
QT_MOC_LITERAL(36, 852, 20), // "toggleCodeBreakpoint"
QT_MOC_LITERAL(37, 873, 3), // "u32"
QT_MOC_LITERAL(38, 877, 4) // "addr"

    },
    "UIDebugCPU\0on_lwRegisters_itemDoubleClicked\0"
    "\0QListWidgetItem*\0item\0"
    "on_lwBackTrace_itemDoubleClicked\0"
    "on_twTrackInfLoop_itemDoubleClicked\0"
    "QTableWidgetItem*\0on_leCodeBreakpoint_textChanged\0"
    "text\0on_leMemoryBreakpoint_textChanged\0"
    "on_lwCodeBreakpoints_itemSelectionChanged\0"
    "on_pbAddCodeBreakpoint_clicked\0"
    "on_pbDelCodeBreakpoint_clicked\0"
    "on_lwMemoryBreakpoints_itemSelectionChanged\0"
    "on_pbAddMemoryBreakpoint_clicked\0"
    "on_pbDelMemoryBreakpoint_clicked\0"
    "on_cbRead_toggled\0enable\0on_cbWrite_toggled\0"
    "on_cbReadByte_toggled\0on_cbReadWord_toggled\0"
    "on_cbReadLong_toggled\0on_cbWriteByte_toggled\0"
    "on_cbWriteWord_toggled\0on_cbWriteLong_toggled\0"
    "on_pbStepInto_clicked\0on_pbStepOver_clicked\0"
    "on_pbStepOut_clicked\0on_pbMemoryTransfer_clicked\0"
    "on_pbMemoryEditor_clicked\0"
    "on_pbReserved1_clicked\0on_pbReserved2_clicked\0"
    "on_pbReserved3_clicked\0on_pbReserved4_clicked\0"
    "on_pbReserved5_clicked\0toggleCodeBreakpoint\0"
    "u32\0addr"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UIDebugCPU[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      30,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  164,    2, 0x09 /* Protected */,
       5,    1,  167,    2, 0x09 /* Protected */,
       6,    1,  170,    2, 0x09 /* Protected */,
       8,    1,  173,    2, 0x09 /* Protected */,
      10,    1,  176,    2, 0x09 /* Protected */,
      11,    0,  179,    2, 0x09 /* Protected */,
      12,    0,  180,    2, 0x09 /* Protected */,
      13,    0,  181,    2, 0x09 /* Protected */,
      14,    0,  182,    2, 0x09 /* Protected */,
      15,    0,  183,    2, 0x09 /* Protected */,
      16,    0,  184,    2, 0x09 /* Protected */,
      17,    1,  185,    2, 0x09 /* Protected */,
      19,    1,  188,    2, 0x09 /* Protected */,
      20,    1,  191,    2, 0x09 /* Protected */,
      21,    1,  194,    2, 0x09 /* Protected */,
      22,    1,  197,    2, 0x09 /* Protected */,
      23,    1,  200,    2, 0x09 /* Protected */,
      24,    1,  203,    2, 0x09 /* Protected */,
      25,    1,  206,    2, 0x09 /* Protected */,
      26,    0,  209,    2, 0x09 /* Protected */,
      27,    0,  210,    2, 0x09 /* Protected */,
      28,    0,  211,    2, 0x09 /* Protected */,
      29,    0,  212,    2, 0x09 /* Protected */,
      30,    0,  213,    2, 0x09 /* Protected */,
      31,    0,  214,    2, 0x09 /* Protected */,
      32,    0,  215,    2, 0x09 /* Protected */,
      33,    0,  216,    2, 0x09 /* Protected */,
      34,    0,  217,    2, 0x09 /* Protected */,
      35,    0,  218,    2, 0x09 /* Protected */,
      36,    1,  219,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 7,    4,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 37,   38,

       0        // eod
};

void UIDebugCPU::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UIDebugCPU *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->on_lwRegisters_itemDoubleClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 1: _t->on_lwBackTrace_itemDoubleClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 2: _t->on_twTrackInfLoop_itemDoubleClicked((*reinterpret_cast< QTableWidgetItem*(*)>(_a[1]))); break;
        case 3: _t->on_leCodeBreakpoint_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->on_leMemoryBreakpoint_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->on_lwCodeBreakpoints_itemSelectionChanged(); break;
        case 6: _t->on_pbAddCodeBreakpoint_clicked(); break;
        case 7: _t->on_pbDelCodeBreakpoint_clicked(); break;
        case 8: _t->on_lwMemoryBreakpoints_itemSelectionChanged(); break;
        case 9: _t->on_pbAddMemoryBreakpoint_clicked(); break;
        case 10: _t->on_pbDelMemoryBreakpoint_clicked(); break;
        case 11: _t->on_cbRead_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->on_cbWrite_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->on_cbReadByte_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->on_cbReadWord_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->on_cbReadLong_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->on_cbWriteByte_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->on_cbWriteWord_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->on_cbWriteLong_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->on_pbStepInto_clicked(); break;
        case 20: _t->on_pbStepOver_clicked(); break;
        case 21: _t->on_pbStepOut_clicked(); break;
        case 22: _t->on_pbMemoryTransfer_clicked(); break;
        case 23: _t->on_pbMemoryEditor_clicked(); break;
        case 24: _t->on_pbReserved1_clicked(); break;
        case 25: _t->on_pbReserved2_clicked(); break;
        case 26: _t->on_pbReserved3_clicked(); break;
        case 27: _t->on_pbReserved4_clicked(); break;
        case 28: _t->on_pbReserved5_clicked(); break;
        case 29: _t->toggleCodeBreakpoint((*reinterpret_cast< u32(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UIDebugCPU::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_UIDebugCPU.data,
    qt_meta_data_UIDebugCPU,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UIDebugCPU::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UIDebugCPU::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UIDebugCPU.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::UIDebugCPU"))
        return static_cast< Ui::UIDebugCPU*>(this);
    return QDialog::qt_metacast(_clname);
}

int UIDebugCPU::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 30;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
