/********************************************************************************
** Form generated from reading UI file 'UIMemorySearch.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIMEMORYSEARCH_H
#define UI_UIMEMORYSEARCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIMemorySearch
{
public:
    QVBoxLayout *verticalLayout_4;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QLabel *label;
    QComboBox *cbType;
    QLabel *label_2;
    QLineEdit *leValue;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_2;
    QLabel *label_3;
    QLineEdit *leStartAddress;
    QLabel *label_4;
    QLineEdit *leEndAddress;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UIMemorySearch)
    {
        if (UIMemorySearch->objectName().isEmpty())
            UIMemorySearch->setObjectName(QString::fromUtf8("UIMemorySearch"));
        UIMemorySearch->resize(333, 217);
        verticalLayout_4 = new QVBoxLayout(UIMemorySearch);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        groupBox = new QGroupBox(UIMemorySearch);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label = new QLabel(groupBox);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        cbType = new QComboBox(groupBox);
        cbType->setObjectName(QString::fromUtf8("cbType"));

        gridLayout->addWidget(cbType, 0, 1, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        leValue = new QLineEdit(groupBox);
        leValue->setObjectName(QString::fromUtf8("leValue"));

        gridLayout->addWidget(leValue, 1, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout);


        verticalLayout_3->addWidget(groupBox);

        groupBox_2 = new QGroupBox(UIMemorySearch);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout_2->addWidget(label_3, 0, 0, 1, 1);

        leStartAddress = new QLineEdit(groupBox_2);
        leStartAddress->setObjectName(QString::fromUtf8("leStartAddress"));

        gridLayout_2->addWidget(leStartAddress, 0, 1, 1, 1);

        label_4 = new QLabel(groupBox_2);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout_2->addWidget(label_4, 1, 0, 1, 1);

        leEndAddress = new QLineEdit(groupBox_2);
        leEndAddress->setObjectName(QString::fromUtf8("leEndAddress"));

        gridLayout_2->addWidget(leEndAddress, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout_2);


        verticalLayout_3->addWidget(groupBox_2);

        dbbButtons = new QDialogButtonBox(UIMemorySearch);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_3->addWidget(dbbButtons);


        verticalLayout_4->addLayout(verticalLayout_3);


        retranslateUi(UIMemorySearch);
        QObject::connect(dbbButtons, SIGNAL(accepted()), UIMemorySearch, SLOT(accept()));
        QObject::connect(dbbButtons, SIGNAL(rejected()), UIMemorySearch, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIMemorySearch);
    } // setupUi

    void retranslateUi(QDialog *UIMemorySearch)
    {
        UIMemorySearch->setWindowTitle(QCoreApplication::translate("UIMemorySearch", "Memory Search", nullptr));
        groupBox->setTitle(QCoreApplication::translate("UIMemorySearch", "Search Criteria", nullptr));
        label->setText(QCoreApplication::translate("UIMemorySearch", "Type:", nullptr));
        label_2->setText(QCoreApplication::translate("UIMemorySearch", "Value:", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("UIMemorySearch", "Options", nullptr));
        label_3->setText(QCoreApplication::translate("UIMemorySearch", "Start Address:", nullptr));
        label_4->setText(QCoreApplication::translate("UIMemorySearch", "End Address:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIMemorySearch: public Ui_UIMemorySearch {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIMEMORYSEARCH_H
