/********************************************************************************
** Form generated from reading UI file 'UIAbout.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIABOUT_H
#define UI_UIABOUT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIAbout
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *lInformations;
    QLabel *lMonkeyStudio;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *pbOk;
    QSpacerItem *spacerItem1;

    void setupUi(QDialog *UIAbout)
    {
        if (UIAbout->objectName().isEmpty())
            UIAbout->setObjectName(QString::fromUtf8("UIAbout"));
        UIAbout->resize(234, 179);
        vboxLayout = new QVBoxLayout(UIAbout);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        lInformations = new QLabel(UIAbout);
        lInformations->setObjectName(QString::fromUtf8("lInformations"));
        lInformations->setTextFormat(Qt::RichText);
        lInformations->setAlignment(Qt::AlignCenter);
        lInformations->setOpenExternalLinks(true);
        lInformations->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::NoTextInteraction|Qt::TextSelectableByMouse);

        vboxLayout->addWidget(lInformations);

        lMonkeyStudio = new QLabel(UIAbout);
        lMonkeyStudio->setObjectName(QString::fromUtf8("lMonkeyStudio"));
        lMonkeyStudio->setAlignment(Qt::AlignCenter);
        lMonkeyStudio->setOpenExternalLinks(true);

        vboxLayout->addWidget(lMonkeyStudio);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        pbOk = new QPushButton(UIAbout);
        pbOk->setObjectName(QString::fromUtf8("pbOk"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pbOk->sizePolicy().hasHeightForWidth());
        pbOk->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(pbOk);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(UIAbout);
        QObject::connect(pbOk, SIGNAL(clicked()), UIAbout, SLOT(accept()));

        QMetaObject::connectSlotsByName(UIAbout);
    } // setupUi

    void retranslateUi(QDialog *UIAbout)
    {
        UIAbout->setWindowTitle(QCoreApplication::translate("UIAbout", "About...", nullptr));
        lInformations->setText(QCoreApplication::translate("UIAbout", "Yabause Qt Gui<br />Based on Yabause %1<br /><a href=\"http://yabause.org\">http://yabause.org</a><br />The Yabause Team<br /><a href=\"mailto:pasnox@gmail.com\">Filipe AZEVEDO</a>", nullptr));
#if QT_CONFIG(tooltip)
        lMonkeyStudio->setToolTip(QCoreApplication::translate("UIAbout", "http://www.monkeystudio.org", nullptr));
#endif // QT_CONFIG(tooltip)
        lMonkeyStudio->setText(QCoreApplication::translate("UIAbout", "<a href=\"http://www.monkeystudio.org\"><img src=\":/icons/icons/made.png\"></a>", nullptr));
        pbOk->setText(QCoreApplication::translate("UIAbout", "Ok", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIAbout: public Ui_UIAbout {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIABOUT_H
