/********************************************************************************
** Form generated from reading UI file 'UIMemoryEditor.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIMEMORYEDITOR_H
#define UI_UIMEMORYEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "UIHexEditor.h"

QT_BEGIN_NAMESPACE

class Ui_UIMemoryEditor
{
public:
    QVBoxLayout *verticalLayout;
    UIHexEditor *saMemoryEditor;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *pbGotoAddress;
    QPushButton *pbSaveSelected;
    QPushButton *pbSaveTab;
    QPushButton *pbSearchMemory;
    QSpacerItem *horizontalSpacer_3;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UIMemoryEditor)
    {
        if (UIMemoryEditor->objectName().isEmpty())
            UIMemoryEditor->setObjectName(QString::fromUtf8("UIMemoryEditor"));
        UIMemoryEditor->resize(518, 363);
        verticalLayout = new QVBoxLayout(UIMemoryEditor);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        saMemoryEditor = new UIHexEditor(UIMemoryEditor);
        saMemoryEditor->setObjectName(QString::fromUtf8("saMemoryEditor"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(saMemoryEditor->sizePolicy().hasHeightForWidth());
        saMemoryEditor->setSizePolicy(sizePolicy);
        saMemoryEditor->setMinimumSize(QSize(0, 0));
        QFont font;
        font.setFamily(QString::fromUtf8("Courier New"));
        font.setPointSize(9);
        saMemoryEditor->setFont(font);
        saMemoryEditor->setFocusPolicy(Qt::StrongFocus);
        saMemoryEditor->setAcceptDrops(false);

        verticalLayout->addWidget(saMemoryEditor);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        horizontalLayout_7->setSizeConstraint(QLayout::SetDefaultConstraint);
        pbGotoAddress = new QPushButton(UIMemoryEditor);
        pbGotoAddress->setObjectName(QString::fromUtf8("pbGotoAddress"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pbGotoAddress->sizePolicy().hasHeightForWidth());
        pbGotoAddress->setSizePolicy(sizePolicy1);
        pbGotoAddress->setAutoDefault(false);

        horizontalLayout_7->addWidget(pbGotoAddress);

        pbSaveSelected = new QPushButton(UIMemoryEditor);
        pbSaveSelected->setObjectName(QString::fromUtf8("pbSaveSelected"));
        pbSaveSelected->setAutoDefault(false);

        horizontalLayout_7->addWidget(pbSaveSelected);

        pbSaveTab = new QPushButton(UIMemoryEditor);
        pbSaveTab->setObjectName(QString::fromUtf8("pbSaveTab"));

        horizontalLayout_7->addWidget(pbSaveTab);

        pbSearchMemory = new QPushButton(UIMemoryEditor);
        pbSearchMemory->setObjectName(QString::fromUtf8("pbSearchMemory"));
        pbSearchMemory->setAutoDefault(false);

        horizontalLayout_7->addWidget(pbSearchMemory);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_3);

        dbbButtons = new QDialogButtonBox(UIMemoryEditor);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout_7->addWidget(dbbButtons);


        verticalLayout->addLayout(horizontalLayout_7);


        retranslateUi(UIMemoryEditor);
        QObject::connect(dbbButtons, SIGNAL(rejected()), UIMemoryEditor, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIMemoryEditor);
    } // setupUi

    void retranslateUi(QDialog *UIMemoryEditor)
    {
        UIMemoryEditor->setWindowTitle(QCoreApplication::translate("UIMemoryEditor", "Memory Editor", nullptr));
        pbGotoAddress->setText(QCoreApplication::translate("UIMemoryEditor", "Goto Address", nullptr));
        pbSaveSelected->setText(QCoreApplication::translate("UIMemoryEditor", "Save Selected", nullptr));
        pbSaveTab->setText(QCoreApplication::translate("UIMemoryEditor", "Save Tab", nullptr));
        pbSearchMemory->setText(QCoreApplication::translate("UIMemoryEditor", "Search Memory", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIMemoryEditor: public Ui_UIMemoryEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIMEMORYEDITOR_H
