/********************************************************************************
** Form generated from reading UI file 'UICheatSearch.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UICHEATSEARCH_H
#define UI_UICHEATSEARCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UICheatSearch
{
public:
    QVBoxLayout *verticalLayout_5;
    QTreeWidget *twSearchResults;
    QHBoxLayout *horizontalLayout;
    QGroupBox *gbCompareType;
    QVBoxLayout *verticalLayout;
    QRadioButton *rbExact;
    QRadioButton *rbLessThan;
    QRadioButton *rbGreaterThan;
    QGroupBox *gbDataType;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *rbUnsigned;
    QRadioButton *rbSigned;
    QSpacerItem *verticalSpacer;
    QGroupBox *rbDataSize;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *rb8Bit;
    QRadioButton *rb16Bit;
    QRadioButton *rb32Bit;
    QGroupBox *gbSearchAddCheats;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label;
    QLineEdit *leSearchValue;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *pbRestart;
    QPushButton *pbSearch;
    QPushButton *pbAddCheat;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UICheatSearch)
    {
        if (UICheatSearch->objectName().isEmpty())
            UICheatSearch->setObjectName(QString::fromUtf8("UICheatSearch"));
        UICheatSearch->resize(378, 525);
        verticalLayout_5 = new QVBoxLayout(UICheatSearch);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        twSearchResults = new QTreeWidget(UICheatSearch);
        twSearchResults->setObjectName(QString::fromUtf8("twSearchResults"));
        twSearchResults->setColumnCount(2);

        verticalLayout_5->addWidget(twSearchResults);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        gbCompareType = new QGroupBox(UICheatSearch);
        gbCompareType->setObjectName(QString::fromUtf8("gbCompareType"));
        verticalLayout = new QVBoxLayout(gbCompareType);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        rbExact = new QRadioButton(gbCompareType);
        rbExact->setObjectName(QString::fromUtf8("rbExact"));

        verticalLayout->addWidget(rbExact);

        rbLessThan = new QRadioButton(gbCompareType);
        rbLessThan->setObjectName(QString::fromUtf8("rbLessThan"));

        verticalLayout->addWidget(rbLessThan);

        rbGreaterThan = new QRadioButton(gbCompareType);
        rbGreaterThan->setObjectName(QString::fromUtf8("rbGreaterThan"));

        verticalLayout->addWidget(rbGreaterThan);


        horizontalLayout->addWidget(gbCompareType);

        gbDataType = new QGroupBox(UICheatSearch);
        gbDataType->setObjectName(QString::fromUtf8("gbDataType"));
        verticalLayout_2 = new QVBoxLayout(gbDataType);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        rbUnsigned = new QRadioButton(gbDataType);
        rbUnsigned->setObjectName(QString::fromUtf8("rbUnsigned"));

        verticalLayout_2->addWidget(rbUnsigned);

        rbSigned = new QRadioButton(gbDataType);
        rbSigned->setObjectName(QString::fromUtf8("rbSigned"));

        verticalLayout_2->addWidget(rbSigned);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout->addWidget(gbDataType);

        rbDataSize = new QGroupBox(UICheatSearch);
        rbDataSize->setObjectName(QString::fromUtf8("rbDataSize"));
        verticalLayout_3 = new QVBoxLayout(rbDataSize);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        rb8Bit = new QRadioButton(rbDataSize);
        rb8Bit->setObjectName(QString::fromUtf8("rb8Bit"));

        verticalLayout_3->addWidget(rb8Bit);

        rb16Bit = new QRadioButton(rbDataSize);
        rb16Bit->setObjectName(QString::fromUtf8("rb16Bit"));

        verticalLayout_3->addWidget(rb16Bit);

        rb32Bit = new QRadioButton(rbDataSize);
        rb32Bit->setObjectName(QString::fromUtf8("rb32Bit"));

        verticalLayout_3->addWidget(rb32Bit);


        horizontalLayout->addWidget(rbDataSize);


        verticalLayout_5->addLayout(horizontalLayout);

        gbSearchAddCheats = new QGroupBox(UICheatSearch);
        gbSearchAddCheats->setObjectName(QString::fromUtf8("gbSearchAddCheats"));
        verticalLayout_4 = new QVBoxLayout(gbSearchAddCheats);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        label = new QLabel(gbSearchAddCheats);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout_3->addWidget(label);

        leSearchValue = new QLineEdit(gbSearchAddCheats);
        leSearchValue->setObjectName(QString::fromUtf8("leSearchValue"));

        horizontalLayout_3->addWidget(leSearchValue);


        verticalLayout_4->addLayout(horizontalLayout_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        pbRestart = new QPushButton(gbSearchAddCheats);
        pbRestart->setObjectName(QString::fromUtf8("pbRestart"));
        pbRestart->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pbRestart->sizePolicy().hasHeightForWidth());
        pbRestart->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(pbRestart);

        pbSearch = new QPushButton(gbSearchAddCheats);
        pbSearch->setObjectName(QString::fromUtf8("pbSearch"));
        pbSearch->setEnabled(true);
        sizePolicy.setHeightForWidth(pbSearch->sizePolicy().hasHeightForWidth());
        pbSearch->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(pbSearch);

        pbAddCheat = new QPushButton(gbSearchAddCheats);
        pbAddCheat->setObjectName(QString::fromUtf8("pbAddCheat"));
        sizePolicy.setHeightForWidth(pbAddCheat->sizePolicy().hasHeightForWidth());
        pbAddCheat->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(pbAddCheat);


        verticalLayout_4->addLayout(horizontalLayout_2);


        verticalLayout_5->addWidget(gbSearchAddCheats);

        dbbButtons = new QDialogButtonBox(UICheatSearch);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout_5->addWidget(dbbButtons);


        retranslateUi(UICheatSearch);
        QObject::connect(dbbButtons, SIGNAL(rejected()), UICheatSearch, SLOT(reject()));

        QMetaObject::connectSlotsByName(UICheatSearch);
    } // setupUi

    void retranslateUi(QDialog *UICheatSearch)
    {
        UICheatSearch->setWindowTitle(QCoreApplication::translate("UICheatSearch", "Cheat Search", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = twSearchResults->headerItem();
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("UICheatSearch", "Value", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("UICheatSearch", "Address", nullptr));
        gbCompareType->setTitle(QCoreApplication::translate("UICheatSearch", "Compare Type", nullptr));
        rbExact->setText(QCoreApplication::translate("UICheatSearch", "Exact", nullptr));
        rbLessThan->setText(QCoreApplication::translate("UICheatSearch", "Less than", nullptr));
        rbGreaterThan->setText(QCoreApplication::translate("UICheatSearch", "Greater then", nullptr));
        gbDataType->setTitle(QCoreApplication::translate("UICheatSearch", "Data Type", nullptr));
        rbUnsigned->setText(QCoreApplication::translate("UICheatSearch", "Unsigned", nullptr));
        rbSigned->setText(QCoreApplication::translate("UICheatSearch", "Signed", nullptr));
        rbDataSize->setTitle(QCoreApplication::translate("UICheatSearch", "Data Size", nullptr));
        rb8Bit->setText(QCoreApplication::translate("UICheatSearch", "8-bit", nullptr));
        rb16Bit->setText(QCoreApplication::translate("UICheatSearch", "16-bit", nullptr));
        rb32Bit->setText(QCoreApplication::translate("UICheatSearch", "32-bit", nullptr));
        gbSearchAddCheats->setTitle(QCoreApplication::translate("UICheatSearch", "Search/Add Cheats", nullptr));
        label->setText(QCoreApplication::translate("UICheatSearch", "Search Value:", nullptr));
        pbRestart->setText(QCoreApplication::translate("UICheatSearch", "Restart", nullptr));
        pbSearch->setText(QCoreApplication::translate("UICheatSearch", "Search", nullptr));
        pbAddCheat->setText(QCoreApplication::translate("UICheatSearch", "Add Cheat", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UICheatSearch: public Ui_UICheatSearch {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UICHEATSEARCH_H
