/********************************************************************************
** Form generated from reading UI file 'UIDebugSCSP.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIDEBUGSCSP_H
#define UI_UIDEBUGSCSP_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIDebugSCSP
{
public:
    QVBoxLayout *verticalLayout_12;
    QHBoxLayout *horizontalLayout_6;
    QGroupBox *gbSlotInfo;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpinBox *sbSlotNumber;
    QPlainTextEdit *pteSlotInfo;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *pbPlaySlot;
    QPushButton *pbSaveAsWav;
    QPushButton *pbSaveSlotRegisters;
    QGroupBox *gbCommonControlRegisters;
    QVBoxLayout *verticalLayout_11;
    QPlainTextEdit *pteCommonControlRegisters;
    QVBoxLayout *verticalLayout_9;
    QGroupBox *gbDSPControlRegisters;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_2;
    QListWidget *pteDSPControlRegisters;
    QHBoxLayout *horizontalLayout_7;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UIDebugSCSP)
    {
        if (UIDebugSCSP->objectName().isEmpty())
            UIDebugSCSP->setObjectName(QString::fromUtf8("UIDebugSCSP"));
        UIDebugSCSP->resize(766, 417);
        verticalLayout_12 = new QVBoxLayout(UIDebugSCSP);
        verticalLayout_12->setObjectName(QString::fromUtf8("verticalLayout_12"));
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        gbSlotInfo = new QGroupBox(UIDebugSCSP);
        gbSlotInfo->setObjectName(QString::fromUtf8("gbSlotInfo"));
        verticalLayout = new QVBoxLayout(gbSlotInfo);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label = new QLabel(gbSlotInfo);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        sbSlotNumber = new QSpinBox(gbSlotInfo);
        sbSlotNumber->setObjectName(QString::fromUtf8("sbSlotNumber"));

        horizontalLayout->addWidget(sbSlotNumber);


        verticalLayout->addLayout(horizontalLayout);

        pteSlotInfo = new QPlainTextEdit(gbSlotInfo);
        pteSlotInfo->setObjectName(QString::fromUtf8("pteSlotInfo"));
        pteSlotInfo->setReadOnly(true);

        verticalLayout->addWidget(pteSlotInfo);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        pbPlaySlot = new QPushButton(gbSlotInfo);
        pbPlaySlot->setObjectName(QString::fromUtf8("pbPlaySlot"));

        horizontalLayout_3->addWidget(pbPlaySlot);

        pbSaveAsWav = new QPushButton(gbSlotInfo);
        pbSaveAsWav->setObjectName(QString::fromUtf8("pbSaveAsWav"));

        horizontalLayout_3->addWidget(pbSaveAsWav);

        pbSaveSlotRegisters = new QPushButton(gbSlotInfo);
        pbSaveSlotRegisters->setObjectName(QString::fromUtf8("pbSaveSlotRegisters"));

        horizontalLayout_3->addWidget(pbSaveSlotRegisters);


        verticalLayout->addLayout(horizontalLayout_3);


        horizontalLayout_6->addWidget(gbSlotInfo);

        gbCommonControlRegisters = new QGroupBox(UIDebugSCSP);
        gbCommonControlRegisters->setObjectName(QString::fromUtf8("gbCommonControlRegisters"));
        verticalLayout_11 = new QVBoxLayout(gbCommonControlRegisters);
        verticalLayout_11->setObjectName(QString::fromUtf8("verticalLayout_11"));
        pteCommonControlRegisters = new QPlainTextEdit(gbCommonControlRegisters);
        pteCommonControlRegisters->setObjectName(QString::fromUtf8("pteCommonControlRegisters"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pteCommonControlRegisters->sizePolicy().hasHeightForWidth());
        pteCommonControlRegisters->setSizePolicy(sizePolicy);
        pteCommonControlRegisters->setReadOnly(true);

        verticalLayout_11->addWidget(pteCommonControlRegisters);


        horizontalLayout_6->addWidget(gbCommonControlRegisters);

        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        gbDSPControlRegisters = new QGroupBox(UIDebugSCSP);
        gbDSPControlRegisters->setObjectName(QString::fromUtf8("gbDSPControlRegisters"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(gbDSPControlRegisters->sizePolicy().hasHeightForWidth());
        gbDSPControlRegisters->setSizePolicy(sizePolicy1);
        verticalLayout_5 = new QVBoxLayout(gbDSPControlRegisters);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        pteDSPControlRegisters = new QListWidget(gbDSPControlRegisters);
        pteDSPControlRegisters->setObjectName(QString::fromUtf8("pteDSPControlRegisters"));

        horizontalLayout_2->addWidget(pteDSPControlRegisters);


        verticalLayout_5->addLayout(horizontalLayout_2);


        verticalLayout_9->addWidget(gbDSPControlRegisters);


        horizontalLayout_6->addLayout(verticalLayout_9);


        verticalLayout_12->addLayout(horizontalLayout_6);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        dbbButtons = new QDialogButtonBox(UIDebugSCSP);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout_7->addWidget(dbbButtons);


        verticalLayout_12->addLayout(horizontalLayout_7);


        retranslateUi(UIDebugSCSP);
        QObject::connect(dbbButtons, SIGNAL(rejected()), UIDebugSCSP, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIDebugSCSP);
    } // setupUi

    void retranslateUi(QDialog *UIDebugSCSP)
    {
        UIDebugSCSP->setWindowTitle(QCoreApplication::translate("UIDebugSCSP", "Debug SCSP", nullptr));
        gbSlotInfo->setTitle(QCoreApplication::translate("UIDebugSCSP", "Slot Info", nullptr));
        label->setText(QCoreApplication::translate("UIDebugSCSP", "Slot Number:", nullptr));
        pbPlaySlot->setText(QCoreApplication::translate("UIDebugSCSP", "Play Slot", nullptr));
        pbSaveAsWav->setText(QCoreApplication::translate("UIDebugSCSP", "Save As WAV", nullptr));
        pbSaveSlotRegisters->setText(QCoreApplication::translate("UIDebugSCSP", "Save Slot Registers", nullptr));
        gbCommonControlRegisters->setTitle(QCoreApplication::translate("UIDebugSCSP", "Common Control Registers", nullptr));
        gbDSPControlRegisters->setTitle(QCoreApplication::translate("UIDebugSCSP", "DSP Control Registers", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIDebugSCSP: public Ui_UIDebugSCSP {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIDEBUGSCSP_H
