/********************************************************************************
** Form generated from reading UI file 'UIBackupRam.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIBACKUPRAM_H
#define UI_UIBACKUPRAM_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIBackupRam
{
public:
    QGridLayout *gridLayout;
    QGroupBox *gbDeviceList;
    QHBoxLayout *hboxLayout;
    QComboBox *cbDeviceList;
    QGroupBox *gbSaveList;
    QVBoxLayout *vboxLayout;
    QListWidget *lwSaveList;
    QLabel *lBlocks;
    QHBoxLayout *hboxLayout1;
    QPushButton *pbDelete;
    QPushButton *pbFormat;
    QGroupBox *gbSaveInformations;
    QVBoxLayout *vboxLayout1;
    QLabel *lFileName;
    QLineEdit *leFileName;
    QLabel *lComment;
    QLineEdit *leComment;
    QLabel *lLanguage;
    QLineEdit *leLanguage;
    QLabel *lDataSize;
    QLineEdit *leDataSize;
    QLabel *lBlockSize;
    QLineEdit *leBlockSize;
    QSpacerItem *spacerItem;
    QDialogButtonBox *dbbButton;

    void setupUi(QDialog *UIBackupRam)
    {
        if (UIBackupRam->objectName().isEmpty())
            UIBackupRam->setObjectName(QString::fromUtf8("UIBackupRam"));
        UIBackupRam->resize(450, 460);
        gridLayout = new QGridLayout(UIBackupRam);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gbDeviceList = new QGroupBox(UIBackupRam);
        gbDeviceList->setObjectName(QString::fromUtf8("gbDeviceList"));
        hboxLayout = new QHBoxLayout(gbDeviceList);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        cbDeviceList = new QComboBox(gbDeviceList);
        cbDeviceList->setObjectName(QString::fromUtf8("cbDeviceList"));

        hboxLayout->addWidget(cbDeviceList);


        gridLayout->addWidget(gbDeviceList, 0, 0, 1, 2);

        gbSaveList = new QGroupBox(UIBackupRam);
        gbSaveList->setObjectName(QString::fromUtf8("gbSaveList"));
        gbSaveList->setMaximumSize(QSize(220, 16777215));
        vboxLayout = new QVBoxLayout(gbSaveList);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        lwSaveList = new QListWidget(gbSaveList);
        lwSaveList->setObjectName(QString::fromUtf8("lwSaveList"));

        vboxLayout->addWidget(lwSaveList);

        lBlocks = new QLabel(gbSaveList);
        lBlocks->setObjectName(QString::fromUtf8("lBlocks"));

        vboxLayout->addWidget(lBlocks);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        pbDelete = new QPushButton(gbSaveList);
        pbDelete->setObjectName(QString::fromUtf8("pbDelete"));

        hboxLayout1->addWidget(pbDelete);

        pbFormat = new QPushButton(gbSaveList);
        pbFormat->setObjectName(QString::fromUtf8("pbFormat"));

        hboxLayout1->addWidget(pbFormat);


        vboxLayout->addLayout(hboxLayout1);


        gridLayout->addWidget(gbSaveList, 1, 0, 1, 1);

        gbSaveInformations = new QGroupBox(UIBackupRam);
        gbSaveInformations->setObjectName(QString::fromUtf8("gbSaveInformations"));
        vboxLayout1 = new QVBoxLayout(gbSaveInformations);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        lFileName = new QLabel(gbSaveInformations);
        lFileName->setObjectName(QString::fromUtf8("lFileName"));

        vboxLayout1->addWidget(lFileName);

        leFileName = new QLineEdit(gbSaveInformations);
        leFileName->setObjectName(QString::fromUtf8("leFileName"));
        leFileName->setReadOnly(true);

        vboxLayout1->addWidget(leFileName);

        lComment = new QLabel(gbSaveInformations);
        lComment->setObjectName(QString::fromUtf8("lComment"));

        vboxLayout1->addWidget(lComment);

        leComment = new QLineEdit(gbSaveInformations);
        leComment->setObjectName(QString::fromUtf8("leComment"));
        leComment->setReadOnly(true);

        vboxLayout1->addWidget(leComment);

        lLanguage = new QLabel(gbSaveInformations);
        lLanguage->setObjectName(QString::fromUtf8("lLanguage"));

        vboxLayout1->addWidget(lLanguage);

        leLanguage = new QLineEdit(gbSaveInformations);
        leLanguage->setObjectName(QString::fromUtf8("leLanguage"));
        leLanguage->setReadOnly(true);

        vboxLayout1->addWidget(leLanguage);

        lDataSize = new QLabel(gbSaveInformations);
        lDataSize->setObjectName(QString::fromUtf8("lDataSize"));

        vboxLayout1->addWidget(lDataSize);

        leDataSize = new QLineEdit(gbSaveInformations);
        leDataSize->setObjectName(QString::fromUtf8("leDataSize"));
        leDataSize->setReadOnly(true);

        vboxLayout1->addWidget(leDataSize);

        lBlockSize = new QLabel(gbSaveInformations);
        lBlockSize->setObjectName(QString::fromUtf8("lBlockSize"));

        vboxLayout1->addWidget(lBlockSize);

        leBlockSize = new QLineEdit(gbSaveInformations);
        leBlockSize->setObjectName(QString::fromUtf8("leBlockSize"));
        leBlockSize->setReadOnly(true);

        vboxLayout1->addWidget(leBlockSize);

        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout1->addItem(spacerItem);


        gridLayout->addWidget(gbSaveInformations, 1, 1, 1, 1);

        dbbButton = new QDialogButtonBox(UIBackupRam);
        dbbButton->setObjectName(QString::fromUtf8("dbbButton"));
        dbbButton->setOrientation(Qt::Horizontal);
        dbbButton->setStandardButtons(QDialogButtonBox::Close);

        gridLayout->addWidget(dbbButton, 2, 0, 1, 2);


        retranslateUi(UIBackupRam);
        QObject::connect(dbbButton, SIGNAL(rejected()), UIBackupRam, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIBackupRam);
    } // setupUi

    void retranslateUi(QDialog *UIBackupRam)
    {
        UIBackupRam->setWindowTitle(QCoreApplication::translate("UIBackupRam", "Backup Ram Manager", nullptr));
        gbDeviceList->setTitle(QCoreApplication::translate("UIBackupRam", "Device List", nullptr));
        gbSaveList->setTitle(QCoreApplication::translate("UIBackupRam", "Save List", nullptr));
        pbDelete->setText(QCoreApplication::translate("UIBackupRam", "Delete", nullptr));
        pbFormat->setText(QCoreApplication::translate("UIBackupRam", "Format", nullptr));
        gbSaveInformations->setTitle(QCoreApplication::translate("UIBackupRam", "Save Information", nullptr));
        lFileName->setText(QCoreApplication::translate("UIBackupRam", "File Name :", nullptr));
        lComment->setText(QCoreApplication::translate("UIBackupRam", "Comment :", nullptr));
        lLanguage->setText(QCoreApplication::translate("UIBackupRam", "Language :", nullptr));
        lDataSize->setText(QCoreApplication::translate("UIBackupRam", "Data Size :", nullptr));
        lBlockSize->setText(QCoreApplication::translate("UIBackupRam", "Block Size :", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIBackupRam: public Ui_UIBackupRam {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIBACKUPRAM_H
