/********************************************************************************
** Form generated from reading UI file 'UICheatRaw.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UICHEATRAW_H
#define UI_UICHEATRAW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QTextEdit>

QT_BEGIN_NAMESPACE

class Ui_UICheatRaw
{
public:
    QGridLayout *gridLayout;
    QGroupBox *gbType;
    QHBoxLayout *hboxLayout;
    QRadioButton *rbEnable;
    QRadioButton *rbByte;
    QRadioButton *rbWord;
    QRadioButton *rbLong;
    QLabel *lAddress;
    QLineEdit *leAddress;
    QLabel *lValue;
    QLineEdit *leValue;
    QLabel *leDescription;
    QTextEdit *teDescription;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UICheatRaw)
    {
        if (UICheatRaw->objectName().isEmpty())
            UICheatRaw->setObjectName(QString::fromUtf8("UICheatRaw"));
        UICheatRaw->resize(458, 246);
        gridLayout = new QGridLayout(UICheatRaw);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gbType = new QGroupBox(UICheatRaw);
        gbType->setObjectName(QString::fromUtf8("gbType"));
        hboxLayout = new QHBoxLayout(gbType);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        rbEnable = new QRadioButton(gbType);
        rbEnable->setObjectName(QString::fromUtf8("rbEnable"));

        hboxLayout->addWidget(rbEnable);

        rbByte = new QRadioButton(gbType);
        rbByte->setObjectName(QString::fromUtf8("rbByte"));

        hboxLayout->addWidget(rbByte);

        rbWord = new QRadioButton(gbType);
        rbWord->setObjectName(QString::fromUtf8("rbWord"));

        hboxLayout->addWidget(rbWord);

        rbLong = new QRadioButton(gbType);
        rbLong->setObjectName(QString::fromUtf8("rbLong"));

        hboxLayout->addWidget(rbLong);


        gridLayout->addWidget(gbType, 0, 0, 1, 2);

        lAddress = new QLabel(UICheatRaw);
        lAddress->setObjectName(QString::fromUtf8("lAddress"));

        gridLayout->addWidget(lAddress, 1, 0, 1, 1);

        leAddress = new QLineEdit(UICheatRaw);
        leAddress->setObjectName(QString::fromUtf8("leAddress"));
        leAddress->setMaxLength(8);

        gridLayout->addWidget(leAddress, 1, 1, 1, 1);

        lValue = new QLabel(UICheatRaw);
        lValue->setObjectName(QString::fromUtf8("lValue"));

        gridLayout->addWidget(lValue, 2, 0, 1, 1);

        leValue = new QLineEdit(UICheatRaw);
        leValue->setObjectName(QString::fromUtf8("leValue"));
        leValue->setMaxLength(23);

        gridLayout->addWidget(leValue, 2, 1, 1, 1);

        leDescription = new QLabel(UICheatRaw);
        leDescription->setObjectName(QString::fromUtf8("leDescription"));
        leDescription->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(leDescription, 3, 0, 1, 1);

        teDescription = new QTextEdit(UICheatRaw);
        teDescription->setObjectName(QString::fromUtf8("teDescription"));
        teDescription->setAcceptRichText(false);

        gridLayout->addWidget(teDescription, 3, 1, 1, 1);

        dbbButtons = new QDialogButtonBox(UICheatRaw);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setOrientation(Qt::Horizontal);
        dbbButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::NoButton|QDialogButtonBox::Ok);

        gridLayout->addWidget(dbbButtons, 4, 0, 1, 2);


        retranslateUi(UICheatRaw);
        QObject::connect(dbbButtons, SIGNAL(accepted()), UICheatRaw, SLOT(accept()));
        QObject::connect(dbbButtons, SIGNAL(rejected()), UICheatRaw, SLOT(reject()));

        QMetaObject::connectSlotsByName(UICheatRaw);
    } // setupUi

    void retranslateUi(QDialog *UICheatRaw)
    {
        UICheatRaw->setWindowTitle(QCoreApplication::translate("UICheatRaw", "Add Raw Memory Code", nullptr));
        gbType->setTitle(QCoreApplication::translate("UICheatRaw", "Cheat Type", nullptr));
        rbEnable->setText(QCoreApplication::translate("UICheatRaw", "Enable", nullptr));
        rbByte->setText(QCoreApplication::translate("UICheatRaw", "Byte Write", nullptr));
        rbWord->setText(QCoreApplication::translate("UICheatRaw", "Word Write", nullptr));
        rbLong->setText(QCoreApplication::translate("UICheatRaw", "Long Write", nullptr));
        lAddress->setText(QCoreApplication::translate("UICheatRaw", "Address :", nullptr));
        lValue->setText(QCoreApplication::translate("UICheatRaw", "Value :", nullptr));
        leDescription->setText(QCoreApplication::translate("UICheatRaw", "Description :", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UICheatRaw: public Ui_UICheatRaw {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UICHEATRAW_H
