/********************************************************************************
** Form generated from reading UI file 'UIDoubleMissionStickSetting.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIDOUBLEMISSIONSTICKSETTING_H
#define UI_UIDOUBLEMISSIONSTICKSETTING_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QToolButton>

QT_BEGIN_NAMESPACE

class Ui_UIDoubleMissionStickSetting
{
public:
    QLabel *label;
    QToolButton *tbX;
    QToolButton *tbY;
    QToolButton *tbZ;
    QToolButton *tbC;
    QToolButton *tbB;
    QToolButton *tbA;
    QToolButton *tbStart;
    QToolButton *tbRightTrigger;
    QToolButton *tbLeftTrigger;
    QLabel *lInfos;
    QDialogButtonBox *padButtons;
    QToolButton *tbAxis2Up;
    QToolButton *tbAxis1Left;
    QToolButton *tbAxis2Down;
    QToolButton *tbAxis1Right;
    QToolButton *tbAxis3Up;
    QToolButton *tbAxis3Down;
    QLabel *label_2;
    QLabel *label_3;
    QToolButton *tbAxis5Left;
    QToolButton *tbAxis5Right;
    QToolButton *tbAxis6Up;
    QToolButton *tbAxis6Down;
    QToolButton *tbAxis7Up;
    QToolButton *tbAxis7Down;

    void setupUi(QDialog *UIDoubleMissionStickSetting)
    {
        if (UIDoubleMissionStickSetting->objectName().isEmpty())
            UIDoubleMissionStickSetting->setObjectName(QString::fromUtf8("UIDoubleMissionStickSetting"));
        UIDoubleMissionStickSetting->resize(575, 486);
        UIDoubleMissionStickSetting->setFocusPolicy(Qt::StrongFocus);
        label = new QLabel(UIDoubleMissionStickSetting);
        label->setObjectName(QString::fromUtf8("label"));
        label->setGeometry(QRect(0, 0, 573, 405));
        label->setPixmap(QPixmap(QString::fromUtf8(":/actions/doublemissionstick.png")));
        label->setScaledContents(true);
        label->setAlignment(Qt::AlignCenter);
        tbX = new QToolButton(UIDoubleMissionStickSetting);
        tbX->setObjectName(QString::fromUtf8("tbX"));
        tbX->setGeometry(QRect(240, 160, 21, 31));
        tbX->setFocusPolicy(Qt::NoFocus);
        tbX->setCheckable(true);
        tbX->setAutoRaise(true);
        tbY = new QToolButton(UIDoubleMissionStickSetting);
        tbY->setObjectName(QString::fromUtf8("tbY"));
        tbY->setGeometry(QRect(270, 160, 21, 31));
        tbY->setFocusPolicy(Qt::NoFocus);
        tbY->setCheckable(true);
        tbY->setAutoRaise(true);
        tbZ = new QToolButton(UIDoubleMissionStickSetting);
        tbZ->setObjectName(QString::fromUtf8("tbZ"));
        tbZ->setGeometry(QRect(310, 160, 21, 31));
        tbZ->setFocusPolicy(Qt::NoFocus);
        tbZ->setCheckable(true);
        tbZ->setAutoRaise(true);
        tbC = new QToolButton(UIDoubleMissionStickSetting);
        tbC->setObjectName(QString::fromUtf8("tbC"));
        tbC->setGeometry(QRect(310, 190, 21, 31));
        tbC->setFocusPolicy(Qt::NoFocus);
        tbC->setCheckable(true);
        tbC->setAutoRaise(true);
        tbB = new QToolButton(UIDoubleMissionStickSetting);
        tbB->setObjectName(QString::fromUtf8("tbB"));
        tbB->setGeometry(QRect(270, 190, 21, 31));
        tbB->setFocusPolicy(Qt::NoFocus);
        tbB->setCheckable(true);
        tbB->setAutoRaise(true);
        tbA = new QToolButton(UIDoubleMissionStickSetting);
        tbA->setObjectName(QString::fromUtf8("tbA"));
        tbA->setGeometry(QRect(240, 190, 21, 31));
        tbA->setFocusPolicy(Qt::NoFocus);
        tbA->setCheckable(true);
        tbA->setAutoRaise(true);
        tbStart = new QToolButton(UIDoubleMissionStickSetting);
        tbStart->setObjectName(QString::fromUtf8("tbStart"));
        tbStart->setGeometry(QRect(270, 110, 31, 30));
        tbStart->setFocusPolicy(Qt::NoFocus);
        tbStart->setCheckable(true);
        tbStart->setAutoRaise(true);
        tbRightTrigger = new QToolButton(UIDoubleMissionStickSetting);
        tbRightTrigger->setObjectName(QString::fromUtf8("tbRightTrigger"));
        tbRightTrigger->setGeometry(QRect(340, 190, 41, 51));
        tbRightTrigger->setFocusPolicy(Qt::NoFocus);
        tbRightTrigger->setCheckable(true);
        tbRightTrigger->setAutoRaise(true);
        tbLeftTrigger = new QToolButton(UIDoubleMissionStickSetting);
        tbLeftTrigger->setObjectName(QString::fromUtf8("tbLeftTrigger"));
        tbLeftTrigger->setGeometry(QRect(190, 190, 41, 51));
        tbLeftTrigger->setFocusPolicy(Qt::NoFocus);
        tbLeftTrigger->setCheckable(true);
        tbLeftTrigger->setAutoRaise(true);
        lInfos = new QLabel(UIDoubleMissionStickSetting);
        lInfos->setObjectName(QString::fromUtf8("lInfos"));
        lInfos->setGeometry(QRect(10, 440, 191, 41));
        lInfos->setAlignment(Qt::AlignCenter);
        padButtons = new QDialogButtonBox(UIDoubleMissionStickSetting);
        padButtons->setObjectName(QString::fromUtf8("padButtons"));
        padButtons->setGeometry(QRect(220, 440, 60, 30));
        padButtons->setStandardButtons(QDialogButtonBox::Close);
        tbAxis2Up = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis2Up->setObjectName(QString::fromUtf8("tbAxis2Up"));
        tbAxis2Up->setGeometry(QRect(440, 160, 31, 30));
        tbAxis2Up->setFocusPolicy(Qt::NoFocus);
        tbAxis2Up->setCheckable(true);
        tbAxis2Up->setAutoRaise(true);
        tbAxis1Left = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis1Left->setObjectName(QString::fromUtf8("tbAxis1Left"));
        tbAxis1Left->setGeometry(QRect(430, 210, 30, 31));
        tbAxis1Left->setFocusPolicy(Qt::NoFocus);
        tbAxis1Left->setCheckable(true);
        tbAxis1Left->setAutoRaise(true);
        tbAxis2Down = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis2Down->setObjectName(QString::fromUtf8("tbAxis2Down"));
        tbAxis2Down->setGeometry(QRect(470, 230, 31, 30));
        tbAxis2Down->setFocusPolicy(Qt::NoFocus);
        tbAxis2Down->setCheckable(true);
        tbAxis2Down->setAutoRaise(true);
        tbAxis1Right = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis1Right->setObjectName(QString::fromUtf8("tbAxis1Right"));
        tbAxis1Right->setGeometry(QRect(500, 210, 30, 31));
        tbAxis1Right->setFocusPolicy(Qt::NoFocus);
        tbAxis1Right->setCheckable(true);
        tbAxis1Right->setAutoRaise(true);
        tbAxis3Up = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis3Up->setObjectName(QString::fromUtf8("tbAxis3Up"));
        tbAxis3Up->setGeometry(QRect(500, 50, 31, 30));
        tbAxis3Up->setFocusPolicy(Qt::NoFocus);
        tbAxis3Up->setCheckable(true);
        tbAxis3Up->setAutoRaise(true);
        tbAxis3Down = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis3Down->setObjectName(QString::fromUtf8("tbAxis3Down"));
        tbAxis3Down->setGeometry(QRect(510, 110, 31, 30));
        tbAxis3Down->setFocusPolicy(Qt::NoFocus);
        tbAxis3Down->setCheckable(true);
        tbAxis3Down->setAutoRaise(true);
        label_2 = new QLabel(UIDoubleMissionStickSetting);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setGeometry(QRect(290, 410, 111, 71));
        QFont font;
        font.setKerning(false);
        label_2->setFont(font);
        label_3 = new QLabel(UIDoubleMissionStickSetting);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setGeometry(QRect(420, 410, 161, 71));
        label_3->setFont(font);
        tbAxis5Left = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis5Left->setObjectName(QString::fromUtf8("tbAxis5Left"));
        tbAxis5Left->setGeometry(QRect(40, 190, 31, 30));
        tbAxis5Left->setFocusPolicy(Qt::NoFocus);
        tbAxis5Left->setCheckable(true);
        tbAxis5Left->setAutoRaise(true);
        tbAxis5Right = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis5Right->setObjectName(QString::fromUtf8("tbAxis5Right"));
        tbAxis5Right->setGeometry(QRect(110, 200, 31, 30));
        tbAxis5Right->setFocusPolicy(Qt::NoFocus);
        tbAxis5Right->setCheckable(true);
        tbAxis5Right->setAutoRaise(true);
        tbAxis6Up = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis6Up->setObjectName(QString::fromUtf8("tbAxis6Up"));
        tbAxis6Up->setGeometry(QRect(80, 160, 31, 30));
        tbAxis6Up->setFocusPolicy(Qt::NoFocus);
        tbAxis6Up->setCheckable(true);
        tbAxis6Up->setAutoRaise(true);
        tbAxis6Down = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis6Down->setObjectName(QString::fromUtf8("tbAxis6Down"));
        tbAxis6Down->setGeometry(QRect(60, 230, 31, 30));
        tbAxis6Down->setFocusPolicy(Qt::NoFocus);
        tbAxis6Down->setCheckable(true);
        tbAxis6Down->setAutoRaise(true);
        tbAxis7Up = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis7Up->setObjectName(QString::fromUtf8("tbAxis7Up"));
        tbAxis7Up->setGeometry(QRect(30, 60, 31, 30));
        tbAxis7Up->setFocusPolicy(Qt::NoFocus);
        tbAxis7Up->setCheckable(true);
        tbAxis7Up->setAutoRaise(true);
        tbAxis7Down = new QToolButton(UIDoubleMissionStickSetting);
        tbAxis7Down->setObjectName(QString::fromUtf8("tbAxis7Down"));
        tbAxis7Down->setGeometry(QRect(20, 110, 31, 30));
        tbAxis7Down->setFocusPolicy(Qt::NoFocus);
        tbAxis7Down->setCheckable(true);
        tbAxis7Down->setAutoRaise(true);

        retranslateUi(UIDoubleMissionStickSetting);
        QObject::connect(padButtons, SIGNAL(rejected()), UIDoubleMissionStickSetting, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIDoubleMissionStickSetting);
    } // setupUi

    void retranslateUi(QDialog *UIDoubleMissionStickSetting)
    {
        UIDoubleMissionStickSetting->setWindowTitle(QCoreApplication::translate("UIDoubleMissionStickSetting", "3D Control Pad Configuration", nullptr));
#if QT_CONFIG(tooltip)
        tbX->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "X", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbY->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Y", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbZ->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Z", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbC->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "C", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbB->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "B", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbA->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "A", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbStart->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Start", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbRightTrigger->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Right trigger", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbLeftTrigger->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Left trigger", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis2Up->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Stick Up", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis1Left->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Stick Left", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis2Down->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Stick Down", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis1Right->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Stick Right", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis3Up->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Throttle Up", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis3Down->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Throttle Down", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QCoreApplication::translate("UIDoubleMissionStickSetting", "Left stick:\n"
"Trigger:      X\n"
"Left button:  Y\n"
"Right button: Z", nullptr));
        label_3->setText(QCoreApplication::translate("UIDoubleMissionStickSetting", "Right Stick:\n"
"Trigger:      A\n"
"Left button:  B\n"
"Right button: C", nullptr));
#if QT_CONFIG(tooltip)
        tbAxis5Left->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Left Stick X Left", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis5Right->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Left Stick X Right", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis6Up->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Left Stick Y Up", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis6Down->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Left Stick Y Down", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis7Up->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Left Stick Throttle Up", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis7Down->setToolTip(QCoreApplication::translate("UIDoubleMissionStickSetting", "Left Stick Throttle Down", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class UIDoubleMissionStickSetting: public Ui_UIDoubleMissionStickSetting {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIDOUBLEMISSIONSTICKSETTING_H
