/********************************************************************************
** Form generated from reading UI file 'UIMemoryTransfer.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIMEMORYTRANSFER_H
#define UI_UIMEMORYTRANSFER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UIMemoryTransfer
{
public:
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QLineEdit *leStartAddress;
    QLineEdit *leEndAddress;
    QLabel *label_3;
    QLabel *label;
    QLineEdit *leFile;
    QToolButton *tbBrowse;
    QHBoxLayout *horizontalLayout;
    QRadioButton *rbDownload;
    QRadioButton *rbUpload;
    QCheckBox *cbPC;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UIMemoryTransfer)
    {
        if (UIMemoryTransfer->objectName().isEmpty())
            UIMemoryTransfer->setObjectName(QString::fromUtf8("UIMemoryTransfer"));
        UIMemoryTransfer->resize(270, 184);
        verticalLayout_2 = new QVBoxLayout(UIMemoryTransfer);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_2 = new QLabel(UIMemoryTransfer);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        leStartAddress = new QLineEdit(UIMemoryTransfer);
        leStartAddress->setObjectName(QString::fromUtf8("leStartAddress"));

        gridLayout->addWidget(leStartAddress, 1, 1, 1, 1);

        leEndAddress = new QLineEdit(UIMemoryTransfer);
        leEndAddress->setObjectName(QString::fromUtf8("leEndAddress"));

        gridLayout->addWidget(leEndAddress, 2, 1, 1, 1);

        label_3 = new QLabel(UIMemoryTransfer);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        label = new QLabel(UIMemoryTransfer);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        leFile = new QLineEdit(UIMemoryTransfer);
        leFile->setObjectName(QString::fromUtf8("leFile"));

        gridLayout->addWidget(leFile, 0, 1, 1, 1);

        tbBrowse = new QToolButton(UIMemoryTransfer);
        tbBrowse->setObjectName(QString::fromUtf8("tbBrowse"));

        gridLayout->addWidget(tbBrowse, 0, 2, 1, 1);


        verticalLayout->addLayout(gridLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        rbDownload = new QRadioButton(UIMemoryTransfer);
        rbDownload->setObjectName(QString::fromUtf8("rbDownload"));

        horizontalLayout->addWidget(rbDownload);

        rbUpload = new QRadioButton(UIMemoryTransfer);
        rbUpload->setObjectName(QString::fromUtf8("rbUpload"));

        horizontalLayout->addWidget(rbUpload);


        verticalLayout->addLayout(horizontalLayout);

        cbPC = new QCheckBox(UIMemoryTransfer);
        cbPC->setObjectName(QString::fromUtf8("cbPC"));

        verticalLayout->addWidget(cbPC);

        dbbButtons = new QDialogButtonBox(UIMemoryTransfer);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(dbbButtons);


        verticalLayout_2->addLayout(verticalLayout);


        retranslateUi(UIMemoryTransfer);
        QObject::connect(dbbButtons, SIGNAL(accepted()), UIMemoryTransfer, SLOT(accept()));
        QObject::connect(dbbButtons, SIGNAL(rejected()), UIMemoryTransfer, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIMemoryTransfer);
    } // setupUi

    void retranslateUi(QDialog *UIMemoryTransfer)
    {
        UIMemoryTransfer->setWindowTitle(QCoreApplication::translate("UIMemoryTransfer", "Memory Transfer", nullptr));
        label_2->setText(QCoreApplication::translate("UIMemoryTransfer", "Start Address:", nullptr));
        label_3->setText(QCoreApplication::translate("UIMemoryTransfer", "End Address:", nullptr));
        label->setText(QCoreApplication::translate("UIMemoryTransfer", "File:", nullptr));
        tbBrowse->setText(QCoreApplication::translate("UIMemoryTransfer", "...", nullptr));
        rbDownload->setText(QCoreApplication::translate("UIMemoryTransfer", "Download", nullptr));
        rbUpload->setText(QCoreApplication::translate("UIMemoryTransfer", "Upload", nullptr));
        cbPC->setText(QCoreApplication::translate("UIMemoryTransfer", "Set PC to Start Address", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIMemoryTransfer: public Ui_UIMemoryTransfer {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIMEMORYTRANSFER_H
