/********************************************************************************
** Form generated from reading UI file 'UICheats.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UICHEATS_H
#define UI_UICHEATS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_UICheats
{
public:
    QVBoxLayout *vboxLayout;
    QTreeWidget *twCheats;
    QHBoxLayout *hboxLayout;
    QPushButton *pbDelete;
    QSpacerItem *spacerItem;
    QPushButton *pbClear;
    QGroupBox *gbAddCodes;
    QHBoxLayout *hboxLayout1;
    QPushButton *pbAR;
    QSpacerItem *spacerItem1;
    QPushButton *pbRaw;
    QGroupBox *groupBox;
    QHBoxLayout *hboxLayout2;
    QPushButton *pbSaveFile;
    QSpacerItem *spacerItem2;
    QPushButton *pbLoadFile;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UICheats)
    {
        if (UICheats->objectName().isEmpty())
            UICheats->setObjectName(QString::fromUtf8("UICheats"));
        UICheats->resize(448, 402);
        vboxLayout = new QVBoxLayout(UICheats);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        twCheats = new QTreeWidget(UICheats);
        twCheats->setObjectName(QString::fromUtf8("twCheats"));
        twCheats->setColumnCount(3);

        vboxLayout->addWidget(twCheats);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        pbDelete = new QPushButton(UICheats);
        pbDelete->setObjectName(QString::fromUtf8("pbDelete"));
        pbDelete->setEnabled(false);

        hboxLayout->addWidget(pbDelete);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        pbClear = new QPushButton(UICheats);
        pbClear->setObjectName(QString::fromUtf8("pbClear"));
        pbClear->setEnabled(false);

        hboxLayout->addWidget(pbClear);


        vboxLayout->addLayout(hboxLayout);

        gbAddCodes = new QGroupBox(UICheats);
        gbAddCodes->setObjectName(QString::fromUtf8("gbAddCodes"));
        hboxLayout1 = new QHBoxLayout(gbAddCodes);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        pbAR = new QPushButton(gbAddCodes);
        pbAR->setObjectName(QString::fromUtf8("pbAR"));

        hboxLayout1->addWidget(pbAR);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem1);

        pbRaw = new QPushButton(gbAddCodes);
        pbRaw->setObjectName(QString::fromUtf8("pbRaw"));

        hboxLayout1->addWidget(pbRaw);


        vboxLayout->addWidget(gbAddCodes);

        groupBox = new QGroupBox(UICheats);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        hboxLayout2 = new QHBoxLayout(groupBox);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        pbSaveFile = new QPushButton(groupBox);
        pbSaveFile->setObjectName(QString::fromUtf8("pbSaveFile"));
        pbSaveFile->setEnabled(false);

        hboxLayout2->addWidget(pbSaveFile);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem2);

        pbLoadFile = new QPushButton(groupBox);
        pbLoadFile->setObjectName(QString::fromUtf8("pbLoadFile"));

        hboxLayout2->addWidget(pbLoadFile);


        vboxLayout->addWidget(groupBox);

        dbbButtons = new QDialogButtonBox(UICheats);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Close);

        vboxLayout->addWidget(dbbButtons);


        retranslateUi(UICheats);
        QObject::connect(dbbButtons, SIGNAL(rejected()), UICheats, SLOT(reject()));

        QMetaObject::connectSlotsByName(UICheats);
    } // setupUi

    void retranslateUi(QDialog *UICheats)
    {
        UICheats->setWindowTitle(QCoreApplication::translate("UICheats", "Cheats", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = twCheats->headerItem();
        ___qtreewidgetitem->setText(2, QCoreApplication::translate("UICheats", "Status", nullptr));
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("UICheats", "Description", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("UICheats", "Code", nullptr));
        pbDelete->setText(QCoreApplication::translate("UICheats", "&Delete", nullptr));
        pbClear->setText(QCoreApplication::translate("UICheats", "&Clear", nullptr));
        gbAddCodes->setTitle(QCoreApplication::translate("UICheats", "Add Codes...", nullptr));
        pbAR->setText(QCoreApplication::translate("UICheats", "&Action Replay", nullptr));
        pbRaw->setText(QCoreApplication::translate("UICheats", "&Raw Memory Address", nullptr));
        groupBox->setTitle(QCoreApplication::translate("UICheats", "Cheats File", nullptr));
        pbSaveFile->setText(QCoreApplication::translate("UICheats", "&Save To File", nullptr));
        pbLoadFile->setText(QCoreApplication::translate("UICheats", "&Load From File", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UICheats: public Ui_UICheats {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UICHEATS_H
