/****************************************************************************
** Meta object code from reading C++ file 'UIYabause.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/qt/ui/UIYabause.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UIYabause.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_UIYabause_t {
    QByteArrayData data[92];
    char stringdata0[1860];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UIYabause_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UIYabause_t qt_meta_stringdata_UIYabause = {
    {
QT_MOC_LITERAL(0, 0, 9), // "UIYabause"
QT_MOC_LITERAL(1, 10, 9), // "appendLog"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 11), // "const char*"
QT_MOC_LITERAL(4, 33, 3), // "msg"
QT_MOC_LITERAL(5, 37, 5), // "pause"
QT_MOC_LITERAL(6, 43, 6), // "paused"
QT_MOC_LITERAL(7, 50, 5), // "reset"
QT_MOC_LITERAL(8, 56, 9), // "hideMouse"
QT_MOC_LITERAL(9, 66, 13), // "cursorRestore"
QT_MOC_LITERAL(10, 80, 18), // "toggleEmulateMouse"
QT_MOC_LITERAL(11, 99, 6), // "enable"
QT_MOC_LITERAL(12, 106, 20), // "breakpointHandlerSH1"
QT_MOC_LITERAL(13, 127, 14), // "displayMessage"
QT_MOC_LITERAL(14, 142, 21), // "breakpointHandlerMSH2"
QT_MOC_LITERAL(15, 164, 21), // "breakpointHandlerSSH2"
QT_MOC_LITERAL(16, 186, 21), // "breakpointHandlerM68K"
QT_MOC_LITERAL(17, 208, 23), // "breakpointHandlerSCUDSP"
QT_MOC_LITERAL(18, 232, 24), // "breakpointHandlerSCSPDSP"
QT_MOC_LITERAL(19, 257, 13), // "errorReceived"
QT_MOC_LITERAL(20, 271, 5), // "error"
QT_MOC_LITERAL(21, 277, 8), // "internal"
QT_MOC_LITERAL(22, 286, 13), // "sizeRequested"
QT_MOC_LITERAL(23, 300, 4), // "size"
QT_MOC_LITERAL(24, 305, 14), // "fixAspectRatio"
QT_MOC_LITERAL(25, 320, 5), // "width"
QT_MOC_LITERAL(26, 326, 23), // "getSupportedResolutions"
QT_MOC_LITERAL(27, 350, 17), // "isResolutionValid"
QT_MOC_LITERAL(28, 368, 6), // "height"
QT_MOC_LITERAL(29, 375, 3), // "bpp"
QT_MOC_LITERAL(30, 379, 4), // "freq"
QT_MOC_LITERAL(31, 384, 17), // "findBestVideoFreq"
QT_MOC_LITERAL(32, 402, 11), // "videoFormat"
QT_MOC_LITERAL(33, 414, 16), // "toggleFullscreen"
QT_MOC_LITERAL(34, 431, 1), // "f"
QT_MOC_LITERAL(35, 433, 19), // "fullscreenRequested"
QT_MOC_LITERAL(36, 453, 10), // "fullscreen"
QT_MOC_LITERAL(37, 464, 20), // "refreshStatesActions"
QT_MOC_LITERAL(38, 485, 12), // "adjustHeight"
QT_MOC_LITERAL(39, 498, 4), // "int&"
QT_MOC_LITERAL(40, 503, 20), // "resizeIntegerScaling"
QT_MOC_LITERAL(41, 524, 26), // "on_aFileSettings_triggered"
QT_MOC_LITERAL(42, 551, 25), // "on_aFileOpenISO_triggered"
QT_MOC_LITERAL(43, 577, 27), // "on_aFileOpenCDRom_triggered"
QT_MOC_LITERAL(44, 605, 25), // "on_aFileOpenSSF_triggered"
QT_MOC_LITERAL(45, 631, 27), // "on_mFileSaveState_triggered"
QT_MOC_LITERAL(46, 659, 8), // "QAction*"
QT_MOC_LITERAL(47, 668, 27), // "on_mFileLoadState_triggered"
QT_MOC_LITERAL(48, 696, 29), // "on_aFileSaveStateAs_triggered"
QT_MOC_LITERAL(49, 726, 29), // "on_aFileLoadStateAs_triggered"
QT_MOC_LITERAL(50, 756, 28), // "on_aFileScreenshot_triggered"
QT_MOC_LITERAL(51, 785, 22), // "on_aFileQuit_triggered"
QT_MOC_LITERAL(52, 808, 26), // "on_aEmulationRun_triggered"
QT_MOC_LITERAL(53, 835, 28), // "on_aEmulationPause_triggered"
QT_MOC_LITERAL(54, 864, 28), // "on_aEmulationReset_triggered"
QT_MOC_LITERAL(55, 893, 37), // "on_aEmulationFrameSkipLimiter..."
QT_MOC_LITERAL(56, 931, 7), // "toggled"
QT_MOC_LITERAL(57, 939, 32), // "on_aToolsBackupManager_triggered"
QT_MOC_LITERAL(58, 972, 29), // "on_aToolsCheatsList_triggered"
QT_MOC_LITERAL(59, 1002, 30), // "on_aToolsCheatSearch_triggered"
QT_MOC_LITERAL(60, 1033, 27), // "on_aToolsTransfer_triggered"
QT_MOC_LITERAL(61, 1061, 21), // "on_aViewFPS_triggered"
QT_MOC_LITERAL(62, 1083, 27), // "on_aViewLayerVdp1_triggered"
QT_MOC_LITERAL(63, 1111, 27), // "on_aViewLayerNBG0_triggered"
QT_MOC_LITERAL(64, 1139, 27), // "on_aViewLayerNBG1_triggered"
QT_MOC_LITERAL(65, 1167, 27), // "on_aViewLayerNBG2_triggered"
QT_MOC_LITERAL(66, 1195, 27), // "on_aViewLayerNBG3_triggered"
QT_MOC_LITERAL(67, 1223, 27), // "on_aViewLayerRBG0_triggered"
QT_MOC_LITERAL(68, 1251, 28), // "on_aViewFullscreen_triggered"
QT_MOC_LITERAL(69, 1280, 1), // "b"
QT_MOC_LITERAL(70, 1282, 27), // "on_aViewDebugMSH2_triggered"
QT_MOC_LITERAL(71, 1310, 27), // "on_aViewDebugSSH2_triggered"
QT_MOC_LITERAL(72, 1338, 27), // "on_aViewDebugVDP1_triggered"
QT_MOC_LITERAL(73, 1366, 27), // "on_aViewDebugVDP2_triggered"
QT_MOC_LITERAL(74, 1394, 27), // "on_aViewDebugM68K_triggered"
QT_MOC_LITERAL(75, 1422, 27), // "on_aViewDebugSCSP_triggered"
QT_MOC_LITERAL(76, 1450, 31), // "on_aViewDebugSCSPChan_triggered"
QT_MOC_LITERAL(77, 1482, 30), // "on_aViewDebugSCSPDSP_triggered"
QT_MOC_LITERAL(78, 1513, 29), // "on_aViewDebugSCUDSP_triggered"
QT_MOC_LITERAL(79, 1543, 26), // "on_aViewDebugSH1_triggered"
QT_MOC_LITERAL(80, 1570, 35), // "on_aViewDebugMemoryEditor_tri..."
QT_MOC_LITERAL(81, 1606, 26), // "on_aTraceLogging_triggered"
QT_MOC_LITERAL(82, 1633, 31), // "on_aHelpDocumentation_triggered"
QT_MOC_LITERAL(83, 1665, 35), // "on_aHelpCompatibilityList_tri..."
QT_MOC_LITERAL(84, 1701, 23), // "on_aHelpAbout_triggered"
QT_MOC_LITERAL(85, 1725, 19), // "on_aSound_triggered"
QT_MOC_LITERAL(86, 1745, 25), // "on_aVideoDriver_triggered"
QT_MOC_LITERAL(87, 1771, 18), // "on_cbSound_toggled"
QT_MOC_LITERAL(88, 1790, 23), // "on_sVolume_valueChanged"
QT_MOC_LITERAL(89, 1814, 5), // "value"
QT_MOC_LITERAL(90, 1820, 36), // "on_cbVideoDriver_currentIndex..."
QT_MOC_LITERAL(91, 1857, 2) // "id"

    },
    "UIYabause\0appendLog\0\0const char*\0msg\0"
    "pause\0paused\0reset\0hideMouse\0cursorRestore\0"
    "toggleEmulateMouse\0enable\0"
    "breakpointHandlerSH1\0displayMessage\0"
    "breakpointHandlerMSH2\0breakpointHandlerSSH2\0"
    "breakpointHandlerM68K\0breakpointHandlerSCUDSP\0"
    "breakpointHandlerSCSPDSP\0errorReceived\0"
    "error\0internal\0sizeRequested\0size\0"
    "fixAspectRatio\0width\0getSupportedResolutions\0"
    "isResolutionValid\0height\0bpp\0freq\0"
    "findBestVideoFreq\0videoFormat\0"
    "toggleFullscreen\0f\0fullscreenRequested\0"
    "fullscreen\0refreshStatesActions\0"
    "adjustHeight\0int&\0resizeIntegerScaling\0"
    "on_aFileSettings_triggered\0"
    "on_aFileOpenISO_triggered\0"
    "on_aFileOpenCDRom_triggered\0"
    "on_aFileOpenSSF_triggered\0"
    "on_mFileSaveState_triggered\0QAction*\0"
    "on_mFileLoadState_triggered\0"
    "on_aFileSaveStateAs_triggered\0"
    "on_aFileLoadStateAs_triggered\0"
    "on_aFileScreenshot_triggered\0"
    "on_aFileQuit_triggered\0"
    "on_aEmulationRun_triggered\0"
    "on_aEmulationPause_triggered\0"
    "on_aEmulationReset_triggered\0"
    "on_aEmulationFrameSkipLimiter_toggled\0"
    "toggled\0on_aToolsBackupManager_triggered\0"
    "on_aToolsCheatsList_triggered\0"
    "on_aToolsCheatSearch_triggered\0"
    "on_aToolsTransfer_triggered\0"
    "on_aViewFPS_triggered\0on_aViewLayerVdp1_triggered\0"
    "on_aViewLayerNBG0_triggered\0"
    "on_aViewLayerNBG1_triggered\0"
    "on_aViewLayerNBG2_triggered\0"
    "on_aViewLayerNBG3_triggered\0"
    "on_aViewLayerRBG0_triggered\0"
    "on_aViewFullscreen_triggered\0b\0"
    "on_aViewDebugMSH2_triggered\0"
    "on_aViewDebugSSH2_triggered\0"
    "on_aViewDebugVDP1_triggered\0"
    "on_aViewDebugVDP2_triggered\0"
    "on_aViewDebugM68K_triggered\0"
    "on_aViewDebugSCSP_triggered\0"
    "on_aViewDebugSCSPChan_triggered\0"
    "on_aViewDebugSCSPDSP_triggered\0"
    "on_aViewDebugSCUDSP_triggered\0"
    "on_aViewDebugSH1_triggered\0"
    "on_aViewDebugMemoryEditor_triggered\0"
    "on_aTraceLogging_triggered\0"
    "on_aHelpDocumentation_triggered\0"
    "on_aHelpCompatibilityList_triggered\0"
    "on_aHelpAbout_triggered\0on_aSound_triggered\0"
    "on_aVideoDriver_triggered\0on_cbSound_toggled\0"
    "on_sVolume_valueChanged\0value\0"
    "on_cbVideoDriver_currentIndexChanged\0"
    "id"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UIYabause[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      70,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  364,    2, 0x0a /* Public */,
       5,    1,  367,    2, 0x0a /* Public */,
       7,    0,  370,    2, 0x0a /* Public */,
       8,    0,  371,    2, 0x0a /* Public */,
       9,    0,  372,    2, 0x0a /* Public */,
      10,    1,  373,    2, 0x0a /* Public */,
      12,    1,  376,    2, 0x0a /* Public */,
      14,    1,  379,    2, 0x0a /* Public */,
      15,    1,  382,    2, 0x0a /* Public */,
      16,    0,  385,    2, 0x0a /* Public */,
      17,    0,  386,    2, 0x0a /* Public */,
      18,    0,  387,    2, 0x0a /* Public */,
      19,    2,  388,    2, 0x09 /* Protected */,
      19,    1,  393,    2, 0x29 /* Protected | MethodCloned */,
      22,    1,  396,    2, 0x09 /* Protected */,
      24,    1,  399,    2, 0x09 /* Protected */,
      26,    0,  402,    2, 0x09 /* Protected */,
      27,    4,  403,    2, 0x09 /* Protected */,
      31,    4,  412,    2, 0x09 /* Protected */,
      33,    4,  421,    2, 0x09 /* Protected */,
      35,    1,  430,    2, 0x09 /* Protected */,
      37,    0,  433,    2, 0x09 /* Protected */,
      38,    1,  434,    2, 0x09 /* Protected */,
      40,    0,  437,    2, 0x09 /* Protected */,
      41,    0,  438,    2, 0x09 /* Protected */,
      42,    0,  439,    2, 0x09 /* Protected */,
      43,    0,  440,    2, 0x09 /* Protected */,
      44,    0,  441,    2, 0x09 /* Protected */,
      45,    1,  442,    2, 0x09 /* Protected */,
      47,    1,  445,    2, 0x09 /* Protected */,
      48,    0,  448,    2, 0x09 /* Protected */,
      49,    0,  449,    2, 0x09 /* Protected */,
      50,    0,  450,    2, 0x09 /* Protected */,
      51,    0,  451,    2, 0x09 /* Protected */,
      52,    0,  452,    2, 0x09 /* Protected */,
      53,    0,  453,    2, 0x09 /* Protected */,
      54,    0,  454,    2, 0x09 /* Protected */,
      55,    1,  455,    2, 0x09 /* Protected */,
      57,    0,  458,    2, 0x09 /* Protected */,
      58,    0,  459,    2, 0x09 /* Protected */,
      59,    0,  460,    2, 0x09 /* Protected */,
      60,    0,  461,    2, 0x09 /* Protected */,
      61,    1,  462,    2, 0x09 /* Protected */,
      62,    0,  465,    2, 0x09 /* Protected */,
      63,    0,  466,    2, 0x09 /* Protected */,
      64,    0,  467,    2, 0x09 /* Protected */,
      65,    0,  468,    2, 0x09 /* Protected */,
      66,    0,  469,    2, 0x09 /* Protected */,
      67,    0,  470,    2, 0x09 /* Protected */,
      68,    1,  471,    2, 0x09 /* Protected */,
      70,    0,  474,    2, 0x09 /* Protected */,
      71,    0,  475,    2, 0x09 /* Protected */,
      72,    0,  476,    2, 0x09 /* Protected */,
      73,    0,  477,    2, 0x09 /* Protected */,
      74,    0,  478,    2, 0x09 /* Protected */,
      75,    0,  479,    2, 0x09 /* Protected */,
      76,    0,  480,    2, 0x09 /* Protected */,
      77,    0,  481,    2, 0x09 /* Protected */,
      78,    0,  482,    2, 0x09 /* Protected */,
      79,    0,  483,    2, 0x09 /* Protected */,
      80,    0,  484,    2, 0x09 /* Protected */,
      81,    1,  485,    2, 0x09 /* Protected */,
      82,    0,  488,    2, 0x09 /* Protected */,
      83,    0,  489,    2, 0x09 /* Protected */,
      84,    0,  490,    2, 0x09 /* Protected */,
      85,    0,  491,    2, 0x09 /* Protected */,
      86,    0,  492,    2, 0x09 /* Protected */,
      87,    1,  493,    2, 0x09 /* Protected */,
      88,    1,  496,    2, 0x09 /* Protected */,
      90,    1,  499,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   20,   21,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void, QMetaType::QSize,   23,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void,
    QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,   25,   28,   29,   30,
    QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,   25,   28,   29,   32,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Bool, QMetaType::Int,   25,   28,   34,   32,
    QMetaType::Void, QMetaType::Bool,   36,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 39,   28,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 46,    2,
    QMetaType::Void, 0x80000000 | 46,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   56,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   56,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   69,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   56,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   56,
    QMetaType::Void, QMetaType::Int,   89,
    QMetaType::Void, QMetaType::Int,   91,

       0        // eod
};

void UIYabause::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UIYabause *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->appendLog((*reinterpret_cast< const char*(*)>(_a[1]))); break;
        case 1: _t->pause((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->reset(); break;
        case 3: _t->hideMouse(); break;
        case 4: _t->cursorRestore(); break;
        case 5: _t->toggleEmulateMouse((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->breakpointHandlerSH1((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->breakpointHandlerMSH2((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->breakpointHandlerSSH2((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->breakpointHandlerM68K(); break;
        case 10: _t->breakpointHandlerSCUDSP(); break;
        case 11: _t->breakpointHandlerSCSPDSP(); break;
        case 12: _t->errorReceived((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 13: _t->errorReceived((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->sizeRequested((*reinterpret_cast< const QSize(*)>(_a[1]))); break;
        case 15: _t->fixAspectRatio((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->getSupportedResolutions(); break;
        case 17: { int _r = _t->isResolutionValid((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 18: { int _r = _t->findBestVideoFreq((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->toggleFullscreen((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 20: _t->fullscreenRequested((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->refreshStatesActions(); break;
        case 22: _t->adjustHeight((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->resizeIntegerScaling(); break;
        case 24: _t->on_aFileSettings_triggered(); break;
        case 25: _t->on_aFileOpenISO_triggered(); break;
        case 26: _t->on_aFileOpenCDRom_triggered(); break;
        case 27: _t->on_aFileOpenSSF_triggered(); break;
        case 28: _t->on_mFileSaveState_triggered((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 29: _t->on_mFileLoadState_triggered((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 30: _t->on_aFileSaveStateAs_triggered(); break;
        case 31: _t->on_aFileLoadStateAs_triggered(); break;
        case 32: _t->on_aFileScreenshot_triggered(); break;
        case 33: _t->on_aFileQuit_triggered(); break;
        case 34: _t->on_aEmulationRun_triggered(); break;
        case 35: _t->on_aEmulationPause_triggered(); break;
        case 36: _t->on_aEmulationReset_triggered(); break;
        case 37: _t->on_aEmulationFrameSkipLimiter_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 38: _t->on_aToolsBackupManager_triggered(); break;
        case 39: _t->on_aToolsCheatsList_triggered(); break;
        case 40: _t->on_aToolsCheatSearch_triggered(); break;
        case 41: _t->on_aToolsTransfer_triggered(); break;
        case 42: _t->on_aViewFPS_triggered((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 43: _t->on_aViewLayerVdp1_triggered(); break;
        case 44: _t->on_aViewLayerNBG0_triggered(); break;
        case 45: _t->on_aViewLayerNBG1_triggered(); break;
        case 46: _t->on_aViewLayerNBG2_triggered(); break;
        case 47: _t->on_aViewLayerNBG3_triggered(); break;
        case 48: _t->on_aViewLayerRBG0_triggered(); break;
        case 49: _t->on_aViewFullscreen_triggered((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 50: _t->on_aViewDebugMSH2_triggered(); break;
        case 51: _t->on_aViewDebugSSH2_triggered(); break;
        case 52: _t->on_aViewDebugVDP1_triggered(); break;
        case 53: _t->on_aViewDebugVDP2_triggered(); break;
        case 54: _t->on_aViewDebugM68K_triggered(); break;
        case 55: _t->on_aViewDebugSCSP_triggered(); break;
        case 56: _t->on_aViewDebugSCSPChan_triggered(); break;
        case 57: _t->on_aViewDebugSCSPDSP_triggered(); break;
        case 58: _t->on_aViewDebugSCUDSP_triggered(); break;
        case 59: _t->on_aViewDebugSH1_triggered(); break;
        case 60: _t->on_aViewDebugMemoryEditor_triggered(); break;
        case 61: _t->on_aTraceLogging_triggered((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 62: _t->on_aHelpDocumentation_triggered(); break;
        case 63: _t->on_aHelpCompatibilityList_triggered(); break;
        case 64: _t->on_aHelpAbout_triggered(); break;
        case 65: _t->on_aSound_triggered(); break;
        case 66: _t->on_aVideoDriver_triggered(); break;
        case 67: _t->on_cbSound_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 68: _t->on_sVolume_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 69: _t->on_cbVideoDriver_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UIYabause::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_UIYabause.data,
    qt_meta_data_UIYabause,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UIYabause::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UIYabause::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UIYabause.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::UIYabause"))
        return static_cast< Ui::UIYabause*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int UIYabause::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 70)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 70;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 70)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 70;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
