/********************************************************************************
** Form generated from reading UI file 'UIGunSetting.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIGUNSETTING_H
#define UI_UIGUNSETTING_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QToolButton>

QT_BEGIN_NAMESPACE

class Ui_UIGunSetting
{
public:
    QLabel *label;
    QToolButton *tbTrigger;
    QToolButton *tbStart;
    QLabel *lInfos;
    QDialogButtonBox *padButtons;
    QToolButton *tbAxis;

    void setupUi(QDialog *UIGunSetting)
    {
        if (UIGunSetting->objectName().isEmpty())
            UIGunSetting->setObjectName(QString::fromUtf8("UIGunSetting"));
        UIGunSetting->resize(211, 409);
        UIGunSetting->setFocusPolicy(Qt::StrongFocus);
        label = new QLabel(UIGunSetting);
        label->setObjectName(QString::fromUtf8("label"));
        label->setGeometry(QRect(10, 0, 191, 351));
        label->setPixmap(QPixmap(QString::fromUtf8(":/actions/mouse.png")));
        label->setScaledContents(true);
        label->setAlignment(Qt::AlignCenter);
        tbTrigger = new QToolButton(UIGunSetting);
        tbTrigger->setObjectName(QString::fromUtf8("tbTrigger"));
        tbTrigger->setGeometry(QRect(30, 70, 54, 58));
        tbTrigger->setFocusPolicy(Qt::NoFocus);
        tbTrigger->setCheckable(true);
        tbTrigger->setAutoRaise(true);
        tbStart = new QToolButton(UIGunSetting);
        tbStart->setObjectName(QString::fromUtf8("tbStart"));
        tbStart->setGeometry(QRect(90, 100, 31, 21));
        tbStart->setFocusPolicy(Qt::NoFocus);
        tbStart->setCheckable(true);
        tbStart->setAutoRaise(true);
        lInfos = new QLabel(UIGunSetting);
        lInfos->setObjectName(QString::fromUtf8("lInfos"));
        lInfos->setGeometry(QRect(10, 340, 191, 41));
        lInfos->setAlignment(Qt::AlignCenter);
        padButtons = new QDialogButtonBox(UIGunSetting);
        padButtons->setObjectName(QString::fromUtf8("padButtons"));
        padButtons->setGeometry(QRect(140, 380, 61, 23));
        padButtons->setStandardButtons(QDialogButtonBox::Close);
        tbAxis = new QToolButton(UIGunSetting);
        tbAxis->setObjectName(QString::fromUtf8("tbAxis"));
        tbAxis->setGeometry(QRect(80, 190, 51, 51));
        tbAxis->setFocusPolicy(Qt::NoFocus);
        tbAxis->setCheckable(true);
        tbAxis->setAutoRaise(true);

        retranslateUi(UIGunSetting);
        QObject::connect(padButtons, SIGNAL(rejected()), UIGunSetting, SLOT(reject()));

        QMetaObject::connectSlotsByName(UIGunSetting);
    } // setupUi

    void retranslateUi(QDialog *UIGunSetting)
    {
        UIGunSetting->setWindowTitle(QCoreApplication::translate("UIGunSetting", "Gun Configuration", nullptr));
#if QT_CONFIG(tooltip)
        tbTrigger->setToolTip(QCoreApplication::translate("UIGunSetting", "B", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbStart->setToolTip(QCoreApplication::translate("UIGunSetting", "Start", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis->setToolTip(QCoreApplication::translate("UIGunSetting", "Axis", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class UIGunSetting: public Ui_UIGunSetting {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIGUNSETTING_H
