/********************************************************************************
** Form generated from reading UI file 'UI3DControlPadSetting.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UI3DCONTROLPADSETTING_H
#define UI_UI3DCONTROLPADSETTING_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QToolButton>

QT_BEGIN_NAMESPACE

class Ui_UI3DControlPadSetting
{
public:
    QLabel *label;
    QToolButton *tbUp;
    QToolButton *tbX;
    QToolButton *tbY;
    QToolButton *tbZ;
    QToolButton *tbC;
    QToolButton *tbB;
    QToolButton *tbA;
    QToolButton *tbStart;
    QToolButton *tbRightTrigger;
    QToolButton *tbLeftTrigger;
    QToolButton *tbDown;
    QToolButton *tbRight;
    QToolButton *tbLeft;
    QLabel *lInfos;
    QDialogButtonBox *padButtons;
    QToolButton *tbAxis2Up;
    QToolButton *tbAxis1Left;
    QToolButton *tbAxis2Down;
    QToolButton *tbAxis1Right;

    void setupUi(QDialog *UI3DControlPadSetting)
    {
        if (UI3DControlPadSetting->objectName().isEmpty())
            UI3DControlPadSetting->setObjectName(QString::fromUtf8("UI3DControlPadSetting"));
        UI3DControlPadSetting->resize(308, 414);
        UI3DControlPadSetting->setFocusPolicy(Qt::StrongFocus);
        label = new QLabel(UI3DControlPadSetting);
        label->setObjectName(QString::fromUtf8("label"));
        label->setGeometry(QRect(10, 10, 281, 321));
        label->setPixmap(QPixmap(QString::fromUtf8(":/actions/3dcontroller.png")));
        label->setScaledContents(true);
        label->setAlignment(Qt::AlignCenter);
        tbUp = new QToolButton(UI3DControlPadSetting);
        tbUp->setObjectName(QString::fromUtf8("tbUp"));
        tbUp->setGeometry(QRect(60, 140, 31, 30));
        tbUp->setFocusPolicy(Qt::NoFocus);
        tbUp->setCheckable(true);
        tbUp->setAutoRaise(true);
        tbX = new QToolButton(UI3DControlPadSetting);
        tbX->setObjectName(QString::fromUtf8("tbX"));
        tbX->setGeometry(QRect(160, 130, 21, 31));
        tbX->setFocusPolicy(Qt::NoFocus);
        tbX->setCheckable(true);
        tbX->setAutoRaise(true);
        tbY = new QToolButton(UI3DControlPadSetting);
        tbY->setObjectName(QString::fromUtf8("tbY"));
        tbY->setGeometry(QRect(190, 122, 21, 31));
        tbY->setFocusPolicy(Qt::NoFocus);
        tbY->setCheckable(true);
        tbY->setAutoRaise(true);
        tbZ = new QToolButton(UI3DControlPadSetting);
        tbZ->setObjectName(QString::fromUtf8("tbZ"));
        tbZ->setGeometry(QRect(220, 122, 21, 31));
        tbZ->setFocusPolicy(Qt::NoFocus);
        tbZ->setCheckable(true);
        tbZ->setAutoRaise(true);
        tbC = new QToolButton(UI3DControlPadSetting);
        tbC->setObjectName(QString::fromUtf8("tbC"));
        tbC->setGeometry(QRect(230, 150, 21, 31));
        tbC->setFocusPolicy(Qt::NoFocus);
        tbC->setCheckable(true);
        tbC->setAutoRaise(true);
        tbB = new QToolButton(UI3DControlPadSetting);
        tbB->setObjectName(QString::fromUtf8("tbB"));
        tbB->setGeometry(QRect(190, 160, 21, 31));
        tbB->setFocusPolicy(Qt::NoFocus);
        tbB->setCheckable(true);
        tbB->setAutoRaise(true);
        tbA = new QToolButton(UI3DControlPadSetting);
        tbA->setObjectName(QString::fromUtf8("tbA"));
        tbA->setGeometry(QRect(160, 170, 21, 31));
        tbA->setFocusPolicy(Qt::NoFocus);
        tbA->setCheckable(true);
        tbA->setAutoRaise(true);
        tbStart = new QToolButton(UI3DControlPadSetting);
        tbStart->setObjectName(QString::fromUtf8("tbStart"));
        tbStart->setGeometry(QRect(110, 210, 31, 30));
        tbStart->setFocusPolicy(Qt::NoFocus);
        tbStart->setCheckable(true);
        tbStart->setAutoRaise(true);
        tbRightTrigger = new QToolButton(UI3DControlPadSetting);
        tbRightTrigger->setObjectName(QString::fromUtf8("tbRightTrigger"));
        tbRightTrigger->setGeometry(QRect(250, 60, 41, 51));
        tbRightTrigger->setFocusPolicy(Qt::NoFocus);
        tbRightTrigger->setCheckable(true);
        tbRightTrigger->setAutoRaise(true);
        tbLeftTrigger = new QToolButton(UI3DControlPadSetting);
        tbLeftTrigger->setObjectName(QString::fromUtf8("tbLeftTrigger"));
        tbLeftTrigger->setGeometry(QRect(80, 10, 41, 51));
        tbLeftTrigger->setFocusPolicy(Qt::NoFocus);
        tbLeftTrigger->setCheckable(true);
        tbLeftTrigger->setAutoRaise(true);
        tbDown = new QToolButton(UI3DControlPadSetting);
        tbDown->setObjectName(QString::fromUtf8("tbDown"));
        tbDown->setGeometry(QRect(60, 200, 31, 30));
        tbDown->setFocusPolicy(Qt::NoFocus);
        tbDown->setCheckable(true);
        tbDown->setAutoRaise(true);
        tbRight = new QToolButton(UI3DControlPadSetting);
        tbRight->setObjectName(QString::fromUtf8("tbRight"));
        tbRight->setGeometry(QRect(90, 169, 30, 31));
        tbRight->setFocusPolicy(Qt::NoFocus);
        tbRight->setCheckable(true);
        tbRight->setAutoRaise(true);
        tbLeft = new QToolButton(UI3DControlPadSetting);
        tbLeft->setObjectName(QString::fromUtf8("tbLeft"));
        tbLeft->setGeometry(QRect(30, 170, 30, 31));
        tbLeft->setFocusPolicy(Qt::NoFocus);
        tbLeft->setCheckable(true);
        tbLeft->setAutoRaise(true);
        lInfos = new QLabel(UI3DControlPadSetting);
        lInfos->setObjectName(QString::fromUtf8("lInfos"));
        lInfos->setGeometry(QRect(50, 330, 191, 41));
        lInfos->setAlignment(Qt::AlignCenter);
        padButtons = new QDialogButtonBox(UI3DControlPadSetting);
        padButtons->setObjectName(QString::fromUtf8("padButtons"));
        padButtons->setGeometry(QRect(230, 380, 60, 30));
        padButtons->setStandardButtons(QDialogButtonBox::Close);
        tbAxis2Up = new QToolButton(UI3DControlPadSetting);
        tbAxis2Up->setObjectName(QString::fromUtf8("tbAxis2Up"));
        tbAxis2Up->setGeometry(QRect(40, 50, 31, 30));
        tbAxis2Up->setFocusPolicy(Qt::NoFocus);
        tbAxis2Up->setCheckable(true);
        tbAxis2Up->setAutoRaise(true);
        tbAxis1Left = new QToolButton(UI3DControlPadSetting);
        tbAxis1Left->setObjectName(QString::fromUtf8("tbAxis1Left"));
        tbAxis1Left->setGeometry(QRect(10, 80, 30, 31));
        tbAxis1Left->setFocusPolicy(Qt::NoFocus);
        tbAxis1Left->setCheckable(true);
        tbAxis1Left->setAutoRaise(true);
        tbAxis2Down = new QToolButton(UI3DControlPadSetting);
        tbAxis2Down->setObjectName(QString::fromUtf8("tbAxis2Down"));
        tbAxis2Down->setGeometry(QRect(40, 110, 31, 30));
        tbAxis2Down->setFocusPolicy(Qt::NoFocus);
        tbAxis2Down->setCheckable(true);
        tbAxis2Down->setAutoRaise(true);
        tbAxis1Right = new QToolButton(UI3DControlPadSetting);
        tbAxis1Right->setObjectName(QString::fromUtf8("tbAxis1Right"));
        tbAxis1Right->setGeometry(QRect(70, 80, 30, 31));
        tbAxis1Right->setFocusPolicy(Qt::NoFocus);
        tbAxis1Right->setCheckable(true);
        tbAxis1Right->setAutoRaise(true);

        retranslateUi(UI3DControlPadSetting);
        QObject::connect(padButtons, SIGNAL(rejected()), UI3DControlPadSetting, SLOT(reject()));

        QMetaObject::connectSlotsByName(UI3DControlPadSetting);
    } // setupUi

    void retranslateUi(QDialog *UI3DControlPadSetting)
    {
        UI3DControlPadSetting->setWindowTitle(QCoreApplication::translate("UI3DControlPadSetting", "3D Control Pad Configuration", nullptr));
#if QT_CONFIG(tooltip)
        tbUp->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Up", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbX->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "X", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbY->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Y", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbZ->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Z", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbC->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "C", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbB->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "B", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbA->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "A", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbStart->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Start", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbRightTrigger->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Right trigger", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbLeftTrigger->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Left trigger", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbDown->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Down", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbRight->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Right", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbLeft->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Left", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis2Up->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Up", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis1Left->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Left", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis2Down->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Down", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tbAxis1Right->setToolTip(QCoreApplication::translate("UI3DControlPadSetting", "Right", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class UI3DControlPadSetting: public Ui_UI3DControlPadSetting {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UI3DCONTROLPADSETTING_H
