/****************************************************************************
** Meta object code from reading C++ file 'UICheatSearch.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../src/qt/ui/UICheatSearch.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'UICheatSearch.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_UICheatSearch_t {
    QByteArrayData data[14];
    char stringdata0[258];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UICheatSearch_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UICheatSearch_t qt_meta_stringdata_UICheatSearch = {
    {
QT_MOC_LITERAL(0, 0, 13), // "UICheatSearch"
QT_MOC_LITERAL(1, 14, 39), // "on_twSearchResults_itemSelect..."
QT_MOC_LITERAL(2, 54, 0), // ""
QT_MOC_LITERAL(3, 55, 28), // "on_leSearchValue_textChanged"
QT_MOC_LITERAL(4, 84, 4), // "text"
QT_MOC_LITERAL(5, 89, 20), // "on_pbRestart_clicked"
QT_MOC_LITERAL(6, 110, 19), // "on_pbSearch_clicked"
QT_MOC_LITERAL(7, 130, 21), // "on_pbAddCheat_clicked"
QT_MOC_LITERAL(8, 152, 21), // "on_rbUnsigned_toggled"
QT_MOC_LITERAL(9, 174, 7), // "checked"
QT_MOC_LITERAL(10, 182, 19), // "on_rbSigned_toggled"
QT_MOC_LITERAL(11, 202, 17), // "on_rb8Bit_toggled"
QT_MOC_LITERAL(12, 220, 18), // "on_rb16Bit_toggled"
QT_MOC_LITERAL(13, 239, 18) // "on_rb32Bit_toggled"

    },
    "UICheatSearch\0on_twSearchResults_itemSelectionChanged\0"
    "\0on_leSearchValue_textChanged\0text\0"
    "on_pbRestart_clicked\0on_pbSearch_clicked\0"
    "on_pbAddCheat_clicked\0on_rbUnsigned_toggled\0"
    "checked\0on_rbSigned_toggled\0"
    "on_rb8Bit_toggled\0on_rb16Bit_toggled\0"
    "on_rb32Bit_toggled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UICheatSearch[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x09 /* Protected */,
       3,    1,   65,    2, 0x09 /* Protected */,
       5,    0,   68,    2, 0x09 /* Protected */,
       6,    0,   69,    2, 0x09 /* Protected */,
       7,    0,   70,    2, 0x09 /* Protected */,
       8,    1,   71,    2, 0x09 /* Protected */,
      10,    1,   74,    2, 0x09 /* Protected */,
      11,    1,   77,    2, 0x09 /* Protected */,
      12,    1,   80,    2, 0x09 /* Protected */,
      13,    1,   83,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,

       0        // eod
};

void UICheatSearch::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UICheatSearch *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->on_twSearchResults_itemSelectionChanged(); break;
        case 1: _t->on_leSearchValue_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->on_pbRestart_clicked(); break;
        case 3: _t->on_pbSearch_clicked(); break;
        case 4: _t->on_pbAddCheat_clicked(); break;
        case 5: _t->on_rbUnsigned_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->on_rbSigned_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->on_rb8Bit_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->on_rb16Bit_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->on_rb32Bit_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UICheatSearch::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_UICheatSearch.data,
    qt_meta_data_UICheatSearch,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UICheatSearch::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UICheatSearch::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UICheatSearch.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::UICheatSearch"))
        return static_cast< Ui::UICheatSearch*>(this);
    return QDialog::qt_metacast(_clname);
}

int UICheatSearch::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
