/********************************************************************************
** Form generated from reading UI file 'UIDebugCPU.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UIDEBUGCPU_H
#define UI_UIDEBUGCPU_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "UIDisasm.h"

QT_BEGIN_NAMESPACE

class Ui_UIDebugCPU
{
public:
    QVBoxLayout *verticalLayout_13;
    QHBoxLayout *horizontalLayout_6;
    QGroupBox *gbRegisters;
    QVBoxLayout *verticalLayout_10;
    QListWidget *lwRegisters;
    QVBoxLayout *verticalLayout_12;
    QGroupBox *gbDisassembledCode;
    QVBoxLayout *verticalLayout_11;
    UIDisasm *lwDisassembledCode;
    QGroupBox *gbBackTrace;
    QHBoxLayout *horizontalLayout_8;
    QTabWidget *tabOtherDebug;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_14;
    QListWidget *lwBackTrace;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout_15;
    QTableWidget *twTrackInfLoop;
    QVBoxLayout *verticalLayout_9;
    QGroupBox *gbCodeBreakpoints;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLineEdit *leCodeBreakpoint;
    QSpacerItem *horizontalSpacer;
    QListWidget *lwCodeBreakpoints;
    QVBoxLayout *verticalLayout;
    QPushButton *pbAddCodeBreakpoint;
    QPushButton *pbDelCodeBreakpoint;
    QSpacerItem *verticalSpacer_4;
    QGroupBox *gbMemoryBreakpoints;
    QVBoxLayout *verticalLayout_8;
    QHBoxLayout *horizontalLayout_4;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *leMemoryBreakpoint;
    QSpacerItem *horizontalSpacer_2;
    QListWidget *lwMemoryBreakpoints;
    QVBoxLayout *verticalLayout_3;
    QPushButton *pbAddMemoryBreakpoint;
    QPushButton *pbDelMemoryBreakpoint;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout_5;
    QVBoxLayout *verticalLayout_7;
    QCheckBox *cbRead;
    QSpacerItem *verticalSpacer_2;
    QCheckBox *cbWrite;
    QSpacerItem *verticalSpacer_3;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *cbReadByte;
    QCheckBox *cbReadWord;
    QCheckBox *cbReadLong;
    QCheckBox *cbWriteByte;
    QCheckBox *cbWriteWord;
    QCheckBox *cbWriteLong;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *pbStepInto;
    QPushButton *pbStepOver;
    QPushButton *pbStepOut;
    QPushButton *pbMemoryTransfer;
    QPushButton *pbMemoryEditor;
    QPushButton *pbTraceLogging;
    QPushButton *pbReserved1;
    QPushButton *pbReserved2;
    QPushButton *pbReserved3;
    QPushButton *pbReserved4;
    QPushButton *pbReserved5;
    QSpacerItem *horizontalSpacer_3;
    QDialogButtonBox *dbbButtons;

    void setupUi(QDialog *UIDebugCPU)
    {
        if (UIDebugCPU->objectName().isEmpty())
            UIDebugCPU->setObjectName(QString::fromUtf8("UIDebugCPU"));
        UIDebugCPU->resize(927, 483);
        verticalLayout_13 = new QVBoxLayout(UIDebugCPU);
        verticalLayout_13->setObjectName(QString::fromUtf8("verticalLayout_13"));
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        gbRegisters = new QGroupBox(UIDebugCPU);
        gbRegisters->setObjectName(QString::fromUtf8("gbRegisters"));
        verticalLayout_10 = new QVBoxLayout(gbRegisters);
        verticalLayout_10->setObjectName(QString::fromUtf8("verticalLayout_10"));
        lwRegisters = new QListWidget(gbRegisters);
        lwRegisters->setObjectName(QString::fromUtf8("lwRegisters"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lwRegisters->sizePolicy().hasHeightForWidth());
        lwRegisters->setSizePolicy(sizePolicy);
        lwRegisters->setMinimumSize(QSize(120, 0));
        QFont font;
        font.setFamily(QString::fromUtf8("Courier New"));
        lwRegisters->setFont(font);

        verticalLayout_10->addWidget(lwRegisters);


        horizontalLayout_6->addWidget(gbRegisters);

        verticalLayout_12 = new QVBoxLayout();
        verticalLayout_12->setObjectName(QString::fromUtf8("verticalLayout_12"));
        gbDisassembledCode = new QGroupBox(UIDebugCPU);
        gbDisassembledCode->setObjectName(QString::fromUtf8("gbDisassembledCode"));
        verticalLayout_11 = new QVBoxLayout(gbDisassembledCode);
        verticalLayout_11->setObjectName(QString::fromUtf8("verticalLayout_11"));
        lwDisassembledCode = new UIDisasm(gbDisassembledCode);
        lwDisassembledCode->setObjectName(QString::fromUtf8("lwDisassembledCode"));
        lwDisassembledCode->setMinimumSize(QSize(416, 0));
        QFont font1;
        font1.setFamily(QString::fromUtf8("Courier New"));
        font1.setPointSize(9);
        lwDisassembledCode->setFont(font1);

        verticalLayout_11->addWidget(lwDisassembledCode);


        verticalLayout_12->addWidget(gbDisassembledCode);

        gbBackTrace = new QGroupBox(UIDebugCPU);
        gbBackTrace->setObjectName(QString::fromUtf8("gbBackTrace"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(gbBackTrace->sizePolicy().hasHeightForWidth());
        gbBackTrace->setSizePolicy(sizePolicy1);
        horizontalLayout_8 = new QHBoxLayout(gbBackTrace);
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        tabOtherDebug = new QTabWidget(gbBackTrace);
        tabOtherDebug->setObjectName(QString::fromUtf8("tabOtherDebug"));
        sizePolicy1.setHeightForWidth(tabOtherDebug->sizePolicy().hasHeightForWidth());
        tabOtherDebug->setSizePolicy(sizePolicy1);
        tabOtherDebug->setTabPosition(QTabWidget::South);
        tabOtherDebug->setTabShape(QTabWidget::Rounded);
        tabOtherDebug->setTabsClosable(false);
        tab_3 = new QWidget();
        tab_3->setObjectName(QString::fromUtf8("tab_3"));
        verticalLayout_14 = new QVBoxLayout(tab_3);
        verticalLayout_14->setObjectName(QString::fromUtf8("verticalLayout_14"));
        lwBackTrace = new QListWidget(tab_3);
        lwBackTrace->setObjectName(QString::fromUtf8("lwBackTrace"));
        sizePolicy1.setHeightForWidth(lwBackTrace->sizePolicy().hasHeightForWidth());
        lwBackTrace->setSizePolicy(sizePolicy1);
        lwBackTrace->setFont(font);

        verticalLayout_14->addWidget(lwBackTrace);

        tabOtherDebug->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName(QString::fromUtf8("tab_4"));
        verticalLayout_15 = new QVBoxLayout(tab_4);
        verticalLayout_15->setObjectName(QString::fromUtf8("verticalLayout_15"));
        twTrackInfLoop = new QTableWidget(tab_4);
        if (twTrackInfLoop->columnCount() < 2)
            twTrackInfLoop->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        twTrackInfLoop->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        twTrackInfLoop->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        twTrackInfLoop->setObjectName(QString::fromUtf8("twTrackInfLoop"));
        sizePolicy1.setHeightForWidth(twTrackInfLoop->sizePolicy().hasHeightForWidth());
        twTrackInfLoop->setSizePolicy(sizePolicy1);
        twTrackInfLoop->setEditTriggers(QAbstractItemView::NoEditTriggers);
        twTrackInfLoop->setProperty("showDropIndicator", QVariant(false));
        twTrackInfLoop->setDragDropOverwriteMode(false);

        verticalLayout_15->addWidget(twTrackInfLoop);

        tabOtherDebug->addTab(tab_4, QString());

        horizontalLayout_8->addWidget(tabOtherDebug);


        verticalLayout_12->addWidget(gbBackTrace);


        horizontalLayout_6->addLayout(verticalLayout_12);

        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        gbCodeBreakpoints = new QGroupBox(UIDebugCPU);
        gbCodeBreakpoints->setObjectName(QString::fromUtf8("gbCodeBreakpoints"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(gbCodeBreakpoints->sizePolicy().hasHeightForWidth());
        gbCodeBreakpoints->setSizePolicy(sizePolicy2);
        verticalLayout_5 = new QVBoxLayout(gbCodeBreakpoints);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        leCodeBreakpoint = new QLineEdit(gbCodeBreakpoints);
        leCodeBreakpoint->setObjectName(QString::fromUtf8("leCodeBreakpoint"));
        QSizePolicy sizePolicy3(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(leCodeBreakpoint->sizePolicy().hasHeightForWidth());
        leCodeBreakpoint->setSizePolicy(sizePolicy3);

        horizontalLayout->addWidget(leCodeBreakpoint);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout);

        lwCodeBreakpoints = new QListWidget(gbCodeBreakpoints);
        lwCodeBreakpoints->setObjectName(QString::fromUtf8("lwCodeBreakpoints"));
        sizePolicy.setHeightForWidth(lwCodeBreakpoints->sizePolicy().hasHeightForWidth());
        lwCodeBreakpoints->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(lwCodeBreakpoints);


        horizontalLayout_2->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        pbAddCodeBreakpoint = new QPushButton(gbCodeBreakpoints);
        pbAddCodeBreakpoint->setObjectName(QString::fromUtf8("pbAddCodeBreakpoint"));
        QSizePolicy sizePolicy4(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(pbAddCodeBreakpoint->sizePolicy().hasHeightForWidth());
        pbAddCodeBreakpoint->setSizePolicy(sizePolicy4);

        verticalLayout->addWidget(pbAddCodeBreakpoint);

        pbDelCodeBreakpoint = new QPushButton(gbCodeBreakpoints);
        pbDelCodeBreakpoint->setObjectName(QString::fromUtf8("pbDelCodeBreakpoint"));

        verticalLayout->addWidget(pbDelCodeBreakpoint);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer_4);


        horizontalLayout_2->addLayout(verticalLayout);


        verticalLayout_5->addLayout(horizontalLayout_2);


        verticalLayout_9->addWidget(gbCodeBreakpoints);

        gbMemoryBreakpoints = new QGroupBox(UIDebugCPU);
        gbMemoryBreakpoints->setObjectName(QString::fromUtf8("gbMemoryBreakpoints"));
        sizePolicy2.setHeightForWidth(gbMemoryBreakpoints->sizePolicy().hasHeightForWidth());
        gbMemoryBreakpoints->setSizePolicy(sizePolicy2);
        verticalLayout_8 = new QVBoxLayout(gbMemoryBreakpoints);
        verticalLayout_8->setObjectName(QString::fromUtf8("verticalLayout_8"));
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        leMemoryBreakpoint = new QLineEdit(gbMemoryBreakpoints);
        leMemoryBreakpoint->setObjectName(QString::fromUtf8("leMemoryBreakpoint"));

        horizontalLayout_3->addWidget(leMemoryBreakpoint);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);


        verticalLayout_4->addLayout(horizontalLayout_3);

        lwMemoryBreakpoints = new QListWidget(gbMemoryBreakpoints);
        lwMemoryBreakpoints->setObjectName(QString::fromUtf8("lwMemoryBreakpoints"));
        sizePolicy.setHeightForWidth(lwMemoryBreakpoints->sizePolicy().hasHeightForWidth());
        lwMemoryBreakpoints->setSizePolicy(sizePolicy);

        verticalLayout_4->addWidget(lwMemoryBreakpoints);


        horizontalLayout_4->addLayout(verticalLayout_4);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        pbAddMemoryBreakpoint = new QPushButton(gbMemoryBreakpoints);
        pbAddMemoryBreakpoint->setObjectName(QString::fromUtf8("pbAddMemoryBreakpoint"));

        verticalLayout_3->addWidget(pbAddMemoryBreakpoint);

        pbDelMemoryBreakpoint = new QPushButton(gbMemoryBreakpoints);
        pbDelMemoryBreakpoint->setObjectName(QString::fromUtf8("pbDelMemoryBreakpoint"));

        verticalLayout_3->addWidget(pbDelMemoryBreakpoint);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        horizontalLayout_4->addLayout(verticalLayout_3);


        verticalLayout_8->addLayout(horizontalLayout_4);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        cbRead = new QCheckBox(gbMemoryBreakpoints);
        cbRead->setObjectName(QString::fromUtf8("cbRead"));
        cbRead->setChecked(true);

        verticalLayout_7->addWidget(cbRead);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_7->addItem(verticalSpacer_2);

        cbWrite = new QCheckBox(gbMemoryBreakpoints);
        cbWrite->setObjectName(QString::fromUtf8("cbWrite"));

        verticalLayout_7->addWidget(cbWrite);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_7->addItem(verticalSpacer_3);


        horizontalLayout_5->addLayout(verticalLayout_7);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        cbReadByte = new QCheckBox(gbMemoryBreakpoints);
        cbReadByte->setObjectName(QString::fromUtf8("cbReadByte"));
        cbReadByte->setEnabled(true);
        cbReadByte->setCheckable(true);
        cbReadByte->setChecked(true);

        verticalLayout_6->addWidget(cbReadByte);

        cbReadWord = new QCheckBox(gbMemoryBreakpoints);
        cbReadWord->setObjectName(QString::fromUtf8("cbReadWord"));
        cbReadWord->setEnabled(true);
        cbReadWord->setChecked(false);

        verticalLayout_6->addWidget(cbReadWord);

        cbReadLong = new QCheckBox(gbMemoryBreakpoints);
        cbReadLong->setObjectName(QString::fromUtf8("cbReadLong"));
        cbReadLong->setEnabled(true);

        verticalLayout_6->addWidget(cbReadLong);

        cbWriteByte = new QCheckBox(gbMemoryBreakpoints);
        cbWriteByte->setObjectName(QString::fromUtf8("cbWriteByte"));
        cbWriteByte->setEnabled(false);

        verticalLayout_6->addWidget(cbWriteByte);

        cbWriteWord = new QCheckBox(gbMemoryBreakpoints);
        cbWriteWord->setObjectName(QString::fromUtf8("cbWriteWord"));
        cbWriteWord->setEnabled(false);

        verticalLayout_6->addWidget(cbWriteWord);

        cbWriteLong = new QCheckBox(gbMemoryBreakpoints);
        cbWriteLong->setObjectName(QString::fromUtf8("cbWriteLong"));
        cbWriteLong->setEnabled(false);

        verticalLayout_6->addWidget(cbWriteLong);


        horizontalLayout_5->addLayout(verticalLayout_6);


        verticalLayout_8->addLayout(horizontalLayout_5);


        verticalLayout_9->addWidget(gbMemoryBreakpoints);


        horizontalLayout_6->addLayout(verticalLayout_9);


        verticalLayout_13->addLayout(horizontalLayout_6);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        pbStepInto = new QPushButton(UIDebugCPU);
        pbStepInto->setObjectName(QString::fromUtf8("pbStepInto"));

        horizontalLayout_7->addWidget(pbStepInto);

        pbStepOver = new QPushButton(UIDebugCPU);
        pbStepOver->setObjectName(QString::fromUtf8("pbStepOver"));

        horizontalLayout_7->addWidget(pbStepOver);

        pbStepOut = new QPushButton(UIDebugCPU);
        pbStepOut->setObjectName(QString::fromUtf8("pbStepOut"));

        horizontalLayout_7->addWidget(pbStepOut);

        pbMemoryTransfer = new QPushButton(UIDebugCPU);
        pbMemoryTransfer->setObjectName(QString::fromUtf8("pbMemoryTransfer"));

        horizontalLayout_7->addWidget(pbMemoryTransfer);

        pbMemoryEditor = new QPushButton(UIDebugCPU);
        pbMemoryEditor->setObjectName(QString::fromUtf8("pbMemoryEditor"));

        horizontalLayout_7->addWidget(pbMemoryEditor);

        pbTraceLogging = new QPushButton(UIDebugCPU);
        pbTraceLogging->setObjectName(QString::fromUtf8("pbTraceLogging"));

        horizontalLayout_7->addWidget(pbTraceLogging);

        pbReserved1 = new QPushButton(UIDebugCPU);
        pbReserved1->setObjectName(QString::fromUtf8("pbReserved1"));

        horizontalLayout_7->addWidget(pbReserved1);

        pbReserved2 = new QPushButton(UIDebugCPU);
        pbReserved2->setObjectName(QString::fromUtf8("pbReserved2"));

        horizontalLayout_7->addWidget(pbReserved2);

        pbReserved3 = new QPushButton(UIDebugCPU);
        pbReserved3->setObjectName(QString::fromUtf8("pbReserved3"));

        horizontalLayout_7->addWidget(pbReserved3);

        pbReserved4 = new QPushButton(UIDebugCPU);
        pbReserved4->setObjectName(QString::fromUtf8("pbReserved4"));

        horizontalLayout_7->addWidget(pbReserved4);

        pbReserved5 = new QPushButton(UIDebugCPU);
        pbReserved5->setObjectName(QString::fromUtf8("pbReserved5"));

        horizontalLayout_7->addWidget(pbReserved5);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_3);

        dbbButtons = new QDialogButtonBox(UIDebugCPU);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout_7->addWidget(dbbButtons);


        verticalLayout_13->addLayout(horizontalLayout_7);


        retranslateUi(UIDebugCPU);
        QObject::connect(dbbButtons, SIGNAL(rejected()), UIDebugCPU, SLOT(reject()));

        tabOtherDebug->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(UIDebugCPU);
    } // setupUi

    void retranslateUi(QDialog *UIDebugCPU)
    {
        UIDebugCPU->setWindowTitle(QCoreApplication::translate("UIDebugCPU", "Debug CPU", nullptr));
        gbRegisters->setTitle(QCoreApplication::translate("UIDebugCPU", "Registers", nullptr));
        gbDisassembledCode->setTitle(QCoreApplication::translate("UIDebugCPU", "Disassembled Code", nullptr));
        gbBackTrace->setTitle(QCoreApplication::translate("UIDebugCPU", "Other Debug", nullptr));
        tabOtherDebug->setTabText(tabOtherDebug->indexOf(tab_3), QCoreApplication::translate("UIDebugCPU", "Backtrace", nullptr));
        QTableWidgetItem *___qtablewidgetitem = twTrackInfLoop->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("UIDebugCPU", "Address", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = twTrackInfLoop->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("UIDebugCPU", "Count", nullptr));
        tabOtherDebug->setTabText(tabOtherDebug->indexOf(tab_4), QCoreApplication::translate("UIDebugCPU", "Track Inf Loop Results", nullptr));
        gbCodeBreakpoints->setTitle(QCoreApplication::translate("UIDebugCPU", "Code Breakpoints", nullptr));
        leCodeBreakpoint->setInputMask(QString());
        pbAddCodeBreakpoint->setText(QCoreApplication::translate("UIDebugCPU", "Add", nullptr));
        pbDelCodeBreakpoint->setText(QCoreApplication::translate("UIDebugCPU", "Del", nullptr));
        gbMemoryBreakpoints->setTitle(QCoreApplication::translate("UIDebugCPU", "Memory Breakpoints", nullptr));
        leMemoryBreakpoint->setInputMask(QString());
        pbAddMemoryBreakpoint->setText(QCoreApplication::translate("UIDebugCPU", "Add", nullptr));
        pbDelMemoryBreakpoint->setText(QCoreApplication::translate("UIDebugCPU", "Del", nullptr));
        cbRead->setText(QCoreApplication::translate("UIDebugCPU", "Read", nullptr));
        cbWrite->setText(QCoreApplication::translate("UIDebugCPU", "Write", nullptr));
        cbReadByte->setText(QCoreApplication::translate("UIDebugCPU", "Byte", nullptr));
        cbReadWord->setText(QCoreApplication::translate("UIDebugCPU", "Word", nullptr));
        cbReadLong->setText(QCoreApplication::translate("UIDebugCPU", "Long", nullptr));
        cbWriteByte->setText(QCoreApplication::translate("UIDebugCPU", "Byte", nullptr));
        cbWriteWord->setText(QCoreApplication::translate("UIDebugCPU", "Word", nullptr));
        cbWriteLong->setText(QCoreApplication::translate("UIDebugCPU", "Long", nullptr));
        pbStepInto->setText(QCoreApplication::translate("UIDebugCPU", "Step Into", nullptr));
        pbStepOver->setText(QCoreApplication::translate("UIDebugCPU", "Step Over", nullptr));
        pbStepOut->setText(QCoreApplication::translate("UIDebugCPU", "Step Out", nullptr));
        pbMemoryTransfer->setText(QCoreApplication::translate("UIDebugCPU", "Memory Transfer", nullptr));
        pbMemoryEditor->setText(QCoreApplication::translate("UIDebugCPU", "Memory Editor", nullptr));
        pbTraceLogging->setText(QCoreApplication::translate("UIDebugCPU", "Trace Logging", nullptr));
        pbReserved1->setText(QCoreApplication::translate("UIDebugCPU", "Reserved1", nullptr));
        pbReserved2->setText(QCoreApplication::translate("UIDebugCPU", "Reserved2", nullptr));
        pbReserved3->setText(QCoreApplication::translate("UIDebugCPU", "Reserved3", nullptr));
        pbReserved4->setText(QCoreApplication::translate("UIDebugCPU", "Reserved4", nullptr));
        pbReserved5->setText(QCoreApplication::translate("UIDebugCPU", "Reserved5", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UIDebugCPU: public Ui_UIDebugCPU {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UIDEBUGCPU_H
