/********************************************************************************
** Form generated from reading UI file 'UISettings.ui'
**
** Created by: Qt User Interface Compiler version 5.15.19
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UISETTINGS_H
#define UI_UISETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "UIShortcutManager.h"
#include "ui/UIPortManager.h"

QT_BEGIN_NAMESPACE

class Ui_UISettings
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *twPages;
    QWidget *wGeneral;
    QGridLayout *gridLayout1;
    QCheckBox *cbEnableMultiThreading;
    QCheckBox *cbClockSync;
    QSpacerItem *spacerItem;
    QLineEdit *leBios;
    QToolButton *tbBios;
    QLabel *lCdRom;
    QComboBox *cbCdRom;
    QLineEdit *leCdRom;
    QToolButton *tbCdRom;
    QLabel *lSaveStates;
    QLineEdit *leSaveStates;
    QToolButton *tbSaveStates;
    QLabel *lTranslation;
    QFrame *line;
    QCheckBox *cbEnableFrameSkipLimiter;
    QCheckBox *cbShowFPS;
    QCheckBox *cbAutostart;
    QLabel *lBios;
    QComboBox *cbCdDrive;
    QComboBox *cbTranslation;
    QDateTimeEdit *dteBaseTime;
    QLabel *label_2;
    QSpinBox *sbNumberOfThreads;
    QCheckBox *cbEnableBiosEmulation;
    QWidget *wVideo;
    QGridLayout *gridLayout2;
    QLabel *lVideoFormat;
    QLabel *lOSDCore;
    QComboBox *cbAspectRatio;
    QLabel *lWinHeight;
    QComboBox *cbVideoCore;
    QLineEdit *leWinWidth;
    QComboBox *cbVideoFormat;
    QComboBox *cbFullscreenResolution;
    QCheckBox *cbBilinear;
    QLabel *lFullScreenResolution;
    QLabel *lWinResolution;
    QComboBox *cbOSDCore;
    QLabel *lWinWidth;
    QLabel *lVideoCore;
    QSpacerItem *spacerItem1;
    QCheckBox *cbFullscreen;
    QLineEdit *leWinHeight;
    QCheckBox *cbEnableIntegerPixelScaling;
    QSpinBox *sbIntegerPixelScalingMultiplier;
    QWidget *wSound;
    QVBoxLayout *vboxLayout1;
    QLabel *lSoundCore;
    QComboBox *cbSoundCore;
    QCheckBox *cbNewScsp;
    QSpacerItem *spacerItem2;
    QWidget *wCartMemory;
    QGridLayout *gridLayout3;
    QLabel *lSH1ROM;
    QToolButton *tbCartridge;
    QToolButton *tbMemory;
    QLineEdit *leMemory;
    QToolButton *tbMpegROM;
    QSpacerItem *spacerItem3;
    QLineEdit *leCartridge;
    QLabel *lMemory;
    QLabel *lMpegROM;
    QLabel *lCartridge;
    QLineEdit *leMpegROM;
    QLabel *lCartridgeModemIP;
    QLineEdit *leCartridgeModemIP;
    QComboBox *cbCartridge;
    QLabel *lCartridgeModemPort;
    QLineEdit *leCartridgeModemPort;
    QLineEdit *leSH1ROM;
    QToolButton *tbSH1ROM;
    QWidget *wInput;
    QGridLayout *gridLayout4;
    QLabel *lInput;
    QComboBox *cbInput;
    UIPortManager *pmPort1;
    UIPortManager *pmPort2;
    QLabel *lGunMouseSensitivity;
    QSlider *sGunMouseSensitivity;
    QWidget *wAdvanced;
    QVBoxLayout *vboxLayout2;
    QLabel *lRegion;
    QComboBox *cbRegion;
    QLabel *lSH2Interpreter;
    QComboBox *cbSH2Interpreter;
    QLabel *l68kCore;
    QComboBox *cb68kCore;
    QLabel *lSH1Interpreter;
    QComboBox *cbSH1Interpreter;
    QCheckBox *cbEnableCDBlockLLE;
    QCheckBox *cbEnableSh2DmaTiming;
    QCheckBox *cbEnableScuDmaTiming;
    QCheckBox *cbEnableSh2Cache;
    QLabel *label_3;
    QSpacerItem *spacerItem4;
    QWidget *wView;
    QGridLayout *gridLayout;
    QSpacerItem *spacerItem5;
    QLabel *lHideMenubar;
    QRadioButton *rbMenubarNever;
    QRadioButton *rbMenubarFullscreen;
    QRadioButton *rbMenubarAlways;
    QLabel *lHideToolbar;
    QRadioButton *rbToolbarNever;
    QRadioButton *rbToolbarFullscreen;
    QRadioButton *rbToolbarAlways;
    QLabel *label;
    QRadioButton *rbLogWindowMessage;
    QRadioButton *rbMenubarFullscreenHover;
    QRadioButton *rbLogWindowNever;
    QWidget *tab;
    QVBoxLayout *verticalLayout;
    UIShortcutManager *twShortcuts;
    QDialogButtonBox *dbbButtons;
    QButtonGroup *bgShowLogWindow;
    QButtonGroup *bgShowMenubar;
    QButtonGroup *bgShowToolbar;

    void setupUi(QDialog *UISettings)
    {
        if (UISettings->objectName().isEmpty())
            UISettings->setObjectName(QString::fromUtf8("UISettings"));
        UISettings->resize(513, 523);
        vboxLayout = new QVBoxLayout(UISettings);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        twPages = new QTabWidget(UISettings);
        twPages->setObjectName(QString::fromUtf8("twPages"));
        QFont font;
        font.setBold(false);
        font.setWeight(50);
        twPages->setFont(font);
        wGeneral = new QWidget();
        wGeneral->setObjectName(QString::fromUtf8("wGeneral"));
        gridLayout1 = new QGridLayout(wGeneral);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        cbEnableMultiThreading = new QCheckBox(wGeneral);
        cbEnableMultiThreading->setObjectName(QString::fromUtf8("cbEnableMultiThreading"));

        gridLayout1->addWidget(cbEnableMultiThreading, 21, 0, 1, 1);

        cbClockSync = new QCheckBox(wGeneral);
        cbClockSync->setObjectName(QString::fromUtf8("cbClockSync"));

        gridLayout1->addWidget(cbClockSync, 19, 0, 1, 1);

        spacerItem = new QSpacerItem(20, 2, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout1->addItem(spacerItem, 23, 0, 1, 1);

        leBios = new QLineEdit(wGeneral);
        leBios->setObjectName(QString::fromUtf8("leBios"));

        gridLayout1->addWidget(leBios, 1, 0, 1, 1);

        tbBios = new QToolButton(wGeneral);
        tbBios->setObjectName(QString::fromUtf8("tbBios"));

        gridLayout1->addWidget(tbBios, 1, 1, 1, 1);

        lCdRom = new QLabel(wGeneral);
        lCdRom->setObjectName(QString::fromUtf8("lCdRom"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        lCdRom->setFont(font1);
        lCdRom->setTextFormat(Qt::PlainText);

        gridLayout1->addWidget(lCdRom, 4, 0, 1, 2);

        cbCdRom = new QComboBox(wGeneral);
        cbCdRom->setObjectName(QString::fromUtf8("cbCdRom"));

        gridLayout1->addWidget(cbCdRom, 5, 0, 1, 2);

        leCdRom = new QLineEdit(wGeneral);
        leCdRom->setObjectName(QString::fromUtf8("leCdRom"));

        gridLayout1->addWidget(leCdRom, 7, 0, 1, 1);

        tbCdRom = new QToolButton(wGeneral);
        tbCdRom->setObjectName(QString::fromUtf8("tbCdRom"));

        gridLayout1->addWidget(tbCdRom, 7, 1, 1, 1);

        lSaveStates = new QLabel(wGeneral);
        lSaveStates->setObjectName(QString::fromUtf8("lSaveStates"));
        lSaveStates->setFont(font1);
        lSaveStates->setTextFormat(Qt::PlainText);

        gridLayout1->addWidget(lSaveStates, 8, 0, 1, 2);

        leSaveStates = new QLineEdit(wGeneral);
        leSaveStates->setObjectName(QString::fromUtf8("leSaveStates"));

        gridLayout1->addWidget(leSaveStates, 9, 0, 1, 1);

        tbSaveStates = new QToolButton(wGeneral);
        tbSaveStates->setObjectName(QString::fromUtf8("tbSaveStates"));

        gridLayout1->addWidget(tbSaveStates, 9, 1, 1, 1);

        lTranslation = new QLabel(wGeneral);
        lTranslation->setObjectName(QString::fromUtf8("lTranslation"));
        lTranslation->setFont(font1);
        lTranslation->setTextFormat(Qt::PlainText);

        gridLayout1->addWidget(lTranslation, 10, 0, 1, 2);

        line = new QFrame(wGeneral);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        gridLayout1->addWidget(line, 14, 0, 1, 2);

        cbEnableFrameSkipLimiter = new QCheckBox(wGeneral);
        cbEnableFrameSkipLimiter->setObjectName(QString::fromUtf8("cbEnableFrameSkipLimiter"));

        gridLayout1->addWidget(cbEnableFrameSkipLimiter, 15, 0, 1, 2);

        cbShowFPS = new QCheckBox(wGeneral);
        cbShowFPS->setObjectName(QString::fromUtf8("cbShowFPS"));

        gridLayout1->addWidget(cbShowFPS, 16, 0, 1, 2);

        cbAutostart = new QCheckBox(wGeneral);
        cbAutostart->setObjectName(QString::fromUtf8("cbAutostart"));

        gridLayout1->addWidget(cbAutostart, 17, 0, 1, 2);

        lBios = new QLabel(wGeneral);
        lBios->setObjectName(QString::fromUtf8("lBios"));
        lBios->setFont(font1);
        lBios->setTextFormat(Qt::PlainText);

        gridLayout1->addWidget(lBios, 0, 0, 1, 2);

        cbCdDrive = new QComboBox(wGeneral);
        cbCdDrive->setObjectName(QString::fromUtf8("cbCdDrive"));
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(cbCdDrive->sizePolicy().hasHeightForWidth());
        cbCdDrive->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(cbCdDrive, 6, 0, 1, 2);

        cbTranslation = new QComboBox(wGeneral);
        cbTranslation->setObjectName(QString::fromUtf8("cbTranslation"));

        gridLayout1->addWidget(cbTranslation, 11, 0, 1, 2);

        dteBaseTime = new QDateTimeEdit(wGeneral);
        dteBaseTime->setObjectName(QString::fromUtf8("dteBaseTime"));
        dteBaseTime->setCalendarPopup(true);

        gridLayout1->addWidget(dteBaseTime, 20, 0, 1, 1);

        label_2 = new QLabel(wGeneral);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout1->addWidget(label_2, 22, 0, 1, 1);

        sbNumberOfThreads = new QSpinBox(wGeneral);
        sbNumberOfThreads->setObjectName(QString::fromUtf8("sbNumberOfThreads"));

        gridLayout1->addWidget(sbNumberOfThreads, 22, 1, 1, 1);

        cbEnableBiosEmulation = new QCheckBox(wGeneral);
        cbEnableBiosEmulation->setObjectName(QString::fromUtf8("cbEnableBiosEmulation"));

        gridLayout1->addWidget(cbEnableBiosEmulation, 2, 0, 1, 1);

        twPages->addTab(wGeneral, QString());
        wVideo = new QWidget();
        wVideo->setObjectName(QString::fromUtf8("wVideo"));
        gridLayout2 = new QGridLayout(wVideo);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        lVideoFormat = new QLabel(wVideo);
        lVideoFormat->setObjectName(QString::fromUtf8("lVideoFormat"));
        lVideoFormat->setFont(font1);
        lVideoFormat->setTextFormat(Qt::PlainText);

        gridLayout2->addWidget(lVideoFormat, 16, 0, 1, 3);

        lOSDCore = new QLabel(wVideo);
        lOSDCore->setObjectName(QString::fromUtf8("lOSDCore"));
        lOSDCore->setFont(font1);
        lOSDCore->setTextFormat(Qt::PlainText);

        gridLayout2->addWidget(lOSDCore, 3, 0, 1, 3);

        cbAspectRatio = new QComboBox(wVideo);
        cbAspectRatio->setObjectName(QString::fromUtf8("cbAspectRatio"));

        gridLayout2->addWidget(cbAspectRatio, 6, 0, 1, 3);

        lWinHeight = new QLabel(wVideo);
        lWinHeight->setObjectName(QString::fromUtf8("lWinHeight"));

        gridLayout2->addWidget(lWinHeight, 8, 0, 1, 1);

        cbVideoCore = new QComboBox(wVideo);
        cbVideoCore->setObjectName(QString::fromUtf8("cbVideoCore"));

        gridLayout2->addWidget(cbVideoCore, 1, 0, 1, 3);

        leWinWidth = new QLineEdit(wVideo);
        leWinWidth->setObjectName(QString::fromUtf8("leWinWidth"));

        gridLayout2->addWidget(leWinWidth, 7, 1, 1, 1);

        cbVideoFormat = new QComboBox(wVideo);
        cbVideoFormat->setObjectName(QString::fromUtf8("cbVideoFormat"));

        gridLayout2->addWidget(cbVideoFormat, 17, 0, 1, 3);

        cbFullscreenResolution = new QComboBox(wVideo);
        cbFullscreenResolution->setObjectName(QString::fromUtf8("cbFullscreenResolution"));

        gridLayout2->addWidget(cbFullscreenResolution, 10, 0, 1, 3);

        cbBilinear = new QCheckBox(wVideo);
        cbBilinear->setObjectName(QString::fromUtf8("cbBilinear"));

        gridLayout2->addWidget(cbBilinear, 2, 0, 1, 1);

        lFullScreenResolution = new QLabel(wVideo);
        lFullScreenResolution->setObjectName(QString::fromUtf8("lFullScreenResolution"));
        lFullScreenResolution->setFont(font1);
        lFullScreenResolution->setTextFormat(Qt::PlainText);

        gridLayout2->addWidget(lFullScreenResolution, 9, 0, 1, 1);

        lWinResolution = new QLabel(wVideo);
        lWinResolution->setObjectName(QString::fromUtf8("lWinResolution"));
        lWinResolution->setFont(font1);
        lWinResolution->setTextFormat(Qt::PlainText);

        gridLayout2->addWidget(lWinResolution, 5, 0, 1, 3);

        cbOSDCore = new QComboBox(wVideo);
        cbOSDCore->setObjectName(QString::fromUtf8("cbOSDCore"));

        gridLayout2->addWidget(cbOSDCore, 4, 0, 1, 3);

        lWinWidth = new QLabel(wVideo);
        lWinWidth->setObjectName(QString::fromUtf8("lWinWidth"));

        gridLayout2->addWidget(lWinWidth, 7, 0, 1, 1);

        lVideoCore = new QLabel(wVideo);
        lVideoCore->setObjectName(QString::fromUtf8("lVideoCore"));
        lVideoCore->setFont(font1);
        lVideoCore->setTextFormat(Qt::PlainText);

        gridLayout2->addWidget(lVideoCore, 0, 0, 1, 3);

        spacerItem1 = new QSpacerItem(20, 2, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout2->addItem(spacerItem1, 21, 0, 1, 3);

        cbFullscreen = new QCheckBox(wVideo);
        cbFullscreen->setObjectName(QString::fromUtf8("cbFullscreen"));

        gridLayout2->addWidget(cbFullscreen, 12, 0, 1, 1);

        leWinHeight = new QLineEdit(wVideo);
        leWinHeight->setObjectName(QString::fromUtf8("leWinHeight"));

        gridLayout2->addWidget(leWinHeight, 8, 1, 1, 1);

        cbEnableIntegerPixelScaling = new QCheckBox(wVideo);
        cbEnableIntegerPixelScaling->setObjectName(QString::fromUtf8("cbEnableIntegerPixelScaling"));

        gridLayout2->addWidget(cbEnableIntegerPixelScaling, 18, 0, 1, 1);

        sbIntegerPixelScalingMultiplier = new QSpinBox(wVideo);
        sbIntegerPixelScalingMultiplier->setObjectName(QString::fromUtf8("sbIntegerPixelScalingMultiplier"));
        sbIntegerPixelScalingMultiplier->setMaximumSize(QSize(40, 16777215));
        sbIntegerPixelScalingMultiplier->setMinimum(2);
        sbIntegerPixelScalingMultiplier->setMaximum(4);
        sbIntegerPixelScalingMultiplier->setSingleStep(1);

        gridLayout2->addWidget(sbIntegerPixelScalingMultiplier, 18, 1, 1, 1);

        twPages->addTab(wVideo, QString());
        wSound = new QWidget();
        wSound->setObjectName(QString::fromUtf8("wSound"));
        vboxLayout1 = new QVBoxLayout(wSound);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        lSoundCore = new QLabel(wSound);
        lSoundCore->setObjectName(QString::fromUtf8("lSoundCore"));
        lSoundCore->setFont(font1);
        lSoundCore->setTextFormat(Qt::PlainText);

        vboxLayout1->addWidget(lSoundCore);

        cbSoundCore = new QComboBox(wSound);
        cbSoundCore->setObjectName(QString::fromUtf8("cbSoundCore"));

        vboxLayout1->addWidget(cbSoundCore);

        cbNewScsp = new QCheckBox(wSound);
        cbNewScsp->setObjectName(QString::fromUtf8("cbNewScsp"));

        vboxLayout1->addWidget(cbNewScsp);

        spacerItem2 = new QSpacerItem(20, 2, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout1->addItem(spacerItem2);

        twPages->addTab(wSound, QString());
        wCartMemory = new QWidget();
        wCartMemory->setObjectName(QString::fromUtf8("wCartMemory"));
        gridLayout3 = new QGridLayout(wCartMemory);
        gridLayout3->setObjectName(QString::fromUtf8("gridLayout3"));
        lSH1ROM = new QLabel(wCartMemory);
        lSH1ROM->setObjectName(QString::fromUtf8("lSH1ROM"));

        gridLayout3->addWidget(lSH1ROM, 8, 0, 1, 1);

        tbCartridge = new QToolButton(wCartMemory);
        tbCartridge->setObjectName(QString::fromUtf8("tbCartridge"));

        gridLayout3->addWidget(tbCartridge, 5, 3, 1, 1);

        tbMemory = new QToolButton(wCartMemory);
        tbMemory->setObjectName(QString::fromUtf8("tbMemory"));

        gridLayout3->addWidget(tbMemory, 7, 3, 1, 1);

        leMemory = new QLineEdit(wCartMemory);
        leMemory->setObjectName(QString::fromUtf8("leMemory"));

        gridLayout3->addWidget(leMemory, 7, 0, 1, 3);

        tbMpegROM = new QToolButton(wCartMemory);
        tbMpegROM->setObjectName(QString::fromUtf8("tbMpegROM"));

        gridLayout3->addWidget(tbMpegROM, 12, 3, 1, 1);

        spacerItem3 = new QSpacerItem(20, 2, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout3->addItem(spacerItem3, 13, 1, 1, 1);

        leCartridge = new QLineEdit(wCartMemory);
        leCartridge->setObjectName(QString::fromUtf8("leCartridge"));

        gridLayout3->addWidget(leCartridge, 5, 0, 1, 3);

        lMemory = new QLabel(wCartMemory);
        lMemory->setObjectName(QString::fromUtf8("lMemory"));
        lMemory->setFont(font1);
        lMemory->setTextFormat(Qt::PlainText);

        gridLayout3->addWidget(lMemory, 6, 0, 1, 4);

        lMpegROM = new QLabel(wCartMemory);
        lMpegROM->setObjectName(QString::fromUtf8("lMpegROM"));
        lMpegROM->setFont(font1);
        lMpegROM->setTextFormat(Qt::PlainText);

        gridLayout3->addWidget(lMpegROM, 11, 0, 1, 4);

        lCartridge = new QLabel(wCartMemory);
        lCartridge->setObjectName(QString::fromUtf8("lCartridge"));
        lCartridge->setFont(font1);
        lCartridge->setTextFormat(Qt::PlainText);

        gridLayout3->addWidget(lCartridge, 0, 0, 1, 4);

        leMpegROM = new QLineEdit(wCartMemory);
        leMpegROM->setObjectName(QString::fromUtf8("leMpegROM"));

        gridLayout3->addWidget(leMpegROM, 12, 0, 1, 3);

        lCartridgeModemIP = new QLabel(wCartMemory);
        lCartridgeModemIP->setObjectName(QString::fromUtf8("lCartridgeModemIP"));

        gridLayout3->addWidget(lCartridgeModemIP, 2, 0, 1, 1);

        leCartridgeModemIP = new QLineEdit(wCartMemory);
        leCartridgeModemIP->setObjectName(QString::fromUtf8("leCartridgeModemIP"));
        leCartridgeModemIP->setMaxLength(15);

        gridLayout3->addWidget(leCartridgeModemIP, 2, 2, 1, 2);

        cbCartridge = new QComboBox(wCartMemory);
        cbCartridge->setObjectName(QString::fromUtf8("cbCartridge"));

        gridLayout3->addWidget(cbCartridge, 1, 0, 1, 4);

        lCartridgeModemPort = new QLabel(wCartMemory);
        lCartridgeModemPort->setObjectName(QString::fromUtf8("lCartridgeModemPort"));
        lCartridgeModemPort->setFont(font1);

        gridLayout3->addWidget(lCartridgeModemPort, 4, 0, 1, 1);

        leCartridgeModemPort = new QLineEdit(wCartMemory);
        leCartridgeModemPort->setObjectName(QString::fromUtf8("leCartridgeModemPort"));
        leCartridgeModemPort->setText(QString::fromUtf8(""));
        leCartridgeModemPort->setMaxLength(5);

        gridLayout3->addWidget(leCartridgeModemPort, 4, 2, 1, 2);

        leSH1ROM = new QLineEdit(wCartMemory);
        leSH1ROM->setObjectName(QString::fromUtf8("leSH1ROM"));

        gridLayout3->addWidget(leSH1ROM, 9, 0, 1, 3);

        tbSH1ROM = new QToolButton(wCartMemory);
        tbSH1ROM->setObjectName(QString::fromUtf8("tbSH1ROM"));

        gridLayout3->addWidget(tbSH1ROM, 9, 3, 1, 1);

        twPages->addTab(wCartMemory, QString());
        wInput = new QWidget();
        wInput->setObjectName(QString::fromUtf8("wInput"));
        gridLayout4 = new QGridLayout(wInput);
        gridLayout4->setObjectName(QString::fromUtf8("gridLayout4"));
        lInput = new QLabel(wInput);
        lInput->setObjectName(QString::fromUtf8("lInput"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lInput->sizePolicy().hasHeightForWidth());
        lInput->setSizePolicy(sizePolicy1);
        lInput->setFont(font1);
        lInput->setTextFormat(Qt::PlainText);

        gridLayout4->addWidget(lInput, 0, 0, 1, 2);

        cbInput = new QComboBox(wInput);
        cbInput->setObjectName(QString::fromUtf8("cbInput"));

        gridLayout4->addWidget(cbInput, 1, 0, 1, 2);

        pmPort1 = new UIPortManager(wInput);
        pmPort1->setObjectName(QString::fromUtf8("pmPort1"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(pmPort1->sizePolicy().hasHeightForWidth());
        pmPort1->setSizePolicy(sizePolicy2);

        gridLayout4->addWidget(pmPort1, 2, 0, 1, 1);

        pmPort2 = new UIPortManager(wInput);
        pmPort2->setObjectName(QString::fromUtf8("pmPort2"));
        sizePolicy2.setHeightForWidth(pmPort2->sizePolicy().hasHeightForWidth());
        pmPort2->setSizePolicy(sizePolicy2);

        gridLayout4->addWidget(pmPort2, 2, 1, 1, 1);

        lGunMouseSensitivity = new QLabel(wInput);
        lGunMouseSensitivity->setObjectName(QString::fromUtf8("lGunMouseSensitivity"));
        lGunMouseSensitivity->setFont(font1);

        gridLayout4->addWidget(lGunMouseSensitivity, 3, 0, 1, 1);

        sGunMouseSensitivity = new QSlider(wInput);
        sGunMouseSensitivity->setObjectName(QString::fromUtf8("sGunMouseSensitivity"));
        sGunMouseSensitivity->setMinimum(1);
        sGunMouseSensitivity->setMaximum(400);
        sGunMouseSensitivity->setOrientation(Qt::Horizontal);

        gridLayout4->addWidget(sGunMouseSensitivity, 4, 0, 1, 2);

        twPages->addTab(wInput, QString());
        wAdvanced = new QWidget();
        wAdvanced->setObjectName(QString::fromUtf8("wAdvanced"));
        vboxLayout2 = new QVBoxLayout(wAdvanced);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        lRegion = new QLabel(wAdvanced);
        lRegion->setObjectName(QString::fromUtf8("lRegion"));
        lRegion->setFont(font1);
        lRegion->setTextFormat(Qt::PlainText);

        vboxLayout2->addWidget(lRegion);

        cbRegion = new QComboBox(wAdvanced);
        cbRegion->setObjectName(QString::fromUtf8("cbRegion"));

        vboxLayout2->addWidget(cbRegion);

        lSH2Interpreter = new QLabel(wAdvanced);
        lSH2Interpreter->setObjectName(QString::fromUtf8("lSH2Interpreter"));
        lSH2Interpreter->setFont(font1);
        lSH2Interpreter->setTextFormat(Qt::PlainText);

        vboxLayout2->addWidget(lSH2Interpreter);

        cbSH2Interpreter = new QComboBox(wAdvanced);
        cbSH2Interpreter->setObjectName(QString::fromUtf8("cbSH2Interpreter"));

        vboxLayout2->addWidget(cbSH2Interpreter);

        l68kCore = new QLabel(wAdvanced);
        l68kCore->setObjectName(QString::fromUtf8("l68kCore"));
        l68kCore->setFont(font1);

        vboxLayout2->addWidget(l68kCore);

        cb68kCore = new QComboBox(wAdvanced);
        cb68kCore->setObjectName(QString::fromUtf8("cb68kCore"));

        vboxLayout2->addWidget(cb68kCore);

        lSH1Interpreter = new QLabel(wAdvanced);
        lSH1Interpreter->setObjectName(QString::fromUtf8("lSH1Interpreter"));
        lSH1Interpreter->setFont(font1);

        vboxLayout2->addWidget(lSH1Interpreter);

        cbSH1Interpreter = new QComboBox(wAdvanced);
        cbSH1Interpreter->setObjectName(QString::fromUtf8("cbSH1Interpreter"));

        vboxLayout2->addWidget(cbSH1Interpreter);

        cbEnableCDBlockLLE = new QCheckBox(wAdvanced);
        cbEnableCDBlockLLE->setObjectName(QString::fromUtf8("cbEnableCDBlockLLE"));

        vboxLayout2->addWidget(cbEnableCDBlockLLE);

        cbEnableSh2DmaTiming = new QCheckBox(wAdvanced);
        cbEnableSh2DmaTiming->setObjectName(QString::fromUtf8("cbEnableSh2DmaTiming"));

        vboxLayout2->addWidget(cbEnableSh2DmaTiming);

        cbEnableScuDmaTiming = new QCheckBox(wAdvanced);
        cbEnableScuDmaTiming->setObjectName(QString::fromUtf8("cbEnableScuDmaTiming"));

        vboxLayout2->addWidget(cbEnableScuDmaTiming);

        cbEnableSh2Cache = new QCheckBox(wAdvanced);
        cbEnableSh2Cache->setObjectName(QString::fromUtf8("cbEnableSh2Cache"));

        vboxLayout2->addWidget(cbEnableSh2Cache);

        label_3 = new QLabel(wAdvanced);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setWordWrap(true);

        vboxLayout2->addWidget(label_3);

        spacerItem4 = new QSpacerItem(20, 2, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout2->addItem(spacerItem4);

        twPages->addTab(wAdvanced, QString());
        wView = new QWidget();
        wView->setObjectName(QString::fromUtf8("wView"));
        gridLayout = new QGridLayout(wView);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        spacerItem5 = new QSpacerItem(20, 2, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(spacerItem5, 6, 0, 1, 1);

        lHideMenubar = new QLabel(wView);
        lHideMenubar->setObjectName(QString::fromUtf8("lHideMenubar"));
        lHideMenubar->setFont(font1);

        gridLayout->addWidget(lHideMenubar, 0, 0, 1, 3);

        rbMenubarNever = new QRadioButton(wView);
        bgShowMenubar = new QButtonGroup(UISettings);
        bgShowMenubar->setObjectName(QString::fromUtf8("bgShowMenubar"));
        bgShowMenubar->addButton(rbMenubarNever);
        rbMenubarNever->setObjectName(QString::fromUtf8("rbMenubarNever"));

        gridLayout->addWidget(rbMenubarNever, 1, 0, 1, 1);

        rbMenubarFullscreen = new QRadioButton(wView);
        bgShowMenubar->addButton(rbMenubarFullscreen);
        rbMenubarFullscreen->setObjectName(QString::fromUtf8("rbMenubarFullscreen"));

        gridLayout->addWidget(rbMenubarFullscreen, 1, 1, 1, 1);

        rbMenubarAlways = new QRadioButton(wView);
        bgShowMenubar->addButton(rbMenubarAlways);
        rbMenubarAlways->setObjectName(QString::fromUtf8("rbMenubarAlways"));

        gridLayout->addWidget(rbMenubarAlways, 1, 2, 1, 1);

        lHideToolbar = new QLabel(wView);
        lHideToolbar->setObjectName(QString::fromUtf8("lHideToolbar"));
        lHideToolbar->setFont(font1);

        gridLayout->addWidget(lHideToolbar, 2, 0, 1, 3);

        rbToolbarNever = new QRadioButton(wView);
        bgShowToolbar = new QButtonGroup(UISettings);
        bgShowToolbar->setObjectName(QString::fromUtf8("bgShowToolbar"));
        bgShowToolbar->addButton(rbToolbarNever);
        rbToolbarNever->setObjectName(QString::fromUtf8("rbToolbarNever"));

        gridLayout->addWidget(rbToolbarNever, 3, 0, 1, 1);

        rbToolbarFullscreen = new QRadioButton(wView);
        bgShowToolbar->addButton(rbToolbarFullscreen);
        rbToolbarFullscreen->setObjectName(QString::fromUtf8("rbToolbarFullscreen"));

        gridLayout->addWidget(rbToolbarFullscreen, 3, 1, 1, 1);

        rbToolbarAlways = new QRadioButton(wView);
        bgShowToolbar->addButton(rbToolbarAlways);
        rbToolbarAlways->setObjectName(QString::fromUtf8("rbToolbarAlways"));

        gridLayout->addWidget(rbToolbarAlways, 3, 2, 1, 1);

        label = new QLabel(wView);
        label->setObjectName(QString::fromUtf8("label"));
        label->setFont(font1);

        gridLayout->addWidget(label, 4, 0, 1, 3);

        rbLogWindowMessage = new QRadioButton(wView);
        bgShowLogWindow = new QButtonGroup(UISettings);
        bgShowLogWindow->setObjectName(QString::fromUtf8("bgShowLogWindow"));
        bgShowLogWindow->addButton(rbLogWindowMessage);
        rbLogWindowMessage->setObjectName(QString::fromUtf8("rbLogWindowMessage"));

        gridLayout->addWidget(rbLogWindowMessage, 5, 1, 1, 1);

        rbMenubarFullscreenHover = new QRadioButton(wView);
        bgShowMenubar->addButton(rbMenubarFullscreenHover);
        rbMenubarFullscreenHover->setObjectName(QString::fromUtf8("rbMenubarFullscreenHover"));

        gridLayout->addWidget(rbMenubarFullscreenHover, 1, 3, 1, 1);

        rbLogWindowNever = new QRadioButton(wView);
        bgShowLogWindow->addButton(rbLogWindowNever);
        rbLogWindowNever->setObjectName(QString::fromUtf8("rbLogWindowNever"));

        gridLayout->addWidget(rbLogWindowNever, 5, 0, 1, 1);

        twPages->addTab(wView, QString());
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        verticalLayout = new QVBoxLayout(tab);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        twShortcuts = new UIShortcutManager(tab);
        if (twShortcuts->columnCount() < 2)
            twShortcuts->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        twShortcuts->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        twShortcuts->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        twShortcuts->setObjectName(QString::fromUtf8("twShortcuts"));
        twShortcuts->setTabKeyNavigation(false);
        twShortcuts->setDragDropOverwriteMode(false);
        twShortcuts->setSelectionBehavior(QAbstractItemView::SelectRows);
        twShortcuts->horizontalHeader()->setCascadingSectionResizes(false);
        twShortcuts->horizontalHeader()->setStretchLastSection(false);
        twShortcuts->verticalHeader()->setVisible(false);
        twShortcuts->verticalHeader()->setStretchLastSection(false);

        verticalLayout->addWidget(twShortcuts);

        twPages->addTab(tab, QString());

        vboxLayout->addWidget(twPages);

        dbbButtons = new QDialogButtonBox(UISettings);
        dbbButtons->setObjectName(QString::fromUtf8("dbbButtons"));
        dbbButtons->setOrientation(Qt::Horizontal);
        dbbButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(dbbButtons);


        retranslateUi(UISettings);
        QObject::connect(dbbButtons, SIGNAL(accepted()), UISettings, SLOT(accept()));
        QObject::connect(dbbButtons, SIGNAL(rejected()), UISettings, SLOT(reject()));

        twPages->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(UISettings);
    } // setupUi

    void retranslateUi(QDialog *UISettings)
    {
        UISettings->setWindowTitle(QCoreApplication::translate("UISettings", "Settings", nullptr));
        cbEnableMultiThreading->setText(QCoreApplication::translate("UISettings", "Enable Multithreading", nullptr));
        cbClockSync->setText(QCoreApplication::translate("UISettings", "Synchronize Saturn internal clock with set time", nullptr));
        tbBios->setText(QCoreApplication::translate("UISettings", "...", nullptr));
        lCdRom->setText(QCoreApplication::translate("UISettings", "CD-ROM", nullptr));
        tbCdRom->setText(QCoreApplication::translate("UISettings", "...", nullptr));
        lSaveStates->setText(QCoreApplication::translate("UISettings", "Save States", nullptr));
        tbSaveStates->setText(QCoreApplication::translate("UISettings", "...", nullptr));
        lTranslation->setText(QCoreApplication::translate("UISettings", "Translation", nullptr));
        cbEnableFrameSkipLimiter->setText(QCoreApplication::translate("UISettings", "Enable Frame Skip/Limiter", nullptr));
        cbShowFPS->setText(QCoreApplication::translate("UISettings", "Show FPS", nullptr));
        cbAutostart->setText(QCoreApplication::translate("UISettings", "Autostart", nullptr));
        lBios->setText(QCoreApplication::translate("UISettings", "Bios", nullptr));
        label_2->setText(QCoreApplication::translate("UISettings", "Number of Threads", nullptr));
        cbEnableBiosEmulation->setText(QCoreApplication::translate("UISettings", "Enable Built-in HLE Bios", nullptr));
        twPages->setTabText(twPages->indexOf(wGeneral), QCoreApplication::translate("UISettings", "General", nullptr));
        lVideoFormat->setText(QCoreApplication::translate("UISettings", "Video Format", nullptr));
        lOSDCore->setText(QCoreApplication::translate("UISettings", "OSD Core", nullptr));
        lWinHeight->setText(QCoreApplication::translate("UISettings", "Height", nullptr));
        cbBilinear->setText(QCoreApplication::translate("UISettings", "Bilinear Filtering (Restart required)", nullptr));
        lFullScreenResolution->setText(QCoreApplication::translate("UISettings", "Fullscreen Resolution", nullptr));
        lWinResolution->setText(QCoreApplication::translate("UISettings", "Window Resolution", nullptr));
        lWinWidth->setText(QCoreApplication::translate("UISettings", "Width", nullptr));
        lVideoCore->setText(QCoreApplication::translate("UISettings", "Video Core", nullptr));
        cbFullscreen->setText(QCoreApplication::translate("UISettings", "Start in Fullscreen", nullptr));
        cbEnableIntegerPixelScaling->setText(QCoreApplication::translate("UISettings", "Use integer pixel scaling", nullptr));
        twPages->setTabText(twPages->indexOf(wVideo), QCoreApplication::translate("UISettings", "Video", nullptr));
        lSoundCore->setText(QCoreApplication::translate("UISettings", "Sound Core", nullptr));
        cbNewScsp->setText(QCoreApplication::translate("UISettings", "Enable higher quality sound (may be slower)", nullptr));
        twPages->setTabText(twPages->indexOf(wSound), QCoreApplication::translate("UISettings", "Sound", nullptr));
        lSH1ROM->setText(QCoreApplication::translate("UISettings", "SH1 ROM", nullptr));
        tbCartridge->setText(QCoreApplication::translate("UISettings", "...", nullptr));
        tbMemory->setText(QCoreApplication::translate("UISettings", "...", nullptr));
        tbMpegROM->setText(QCoreApplication::translate("UISettings", "...", nullptr));
        leCartridge->setInputMask(QString());
        lMemory->setText(QCoreApplication::translate("UISettings", "Memory", nullptr));
        lMpegROM->setText(QCoreApplication::translate("UISettings", "Mpeg ROM", nullptr));
        lCartridge->setText(QCoreApplication::translate("UISettings", "Cartridge", nullptr));
        lCartridgeModemIP->setText(QCoreApplication::translate("UISettings", "Default Dial IP", nullptr));
#if QT_CONFIG(tooltip)
        leCartridgeModemIP->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        leCartridgeModemIP->setInputMask(QString());
        lCartridgeModemPort->setText(QCoreApplication::translate("UISettings", "Port", nullptr));
#if QT_CONFIG(tooltip)
        leCartridgeModemPort->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        tbSH1ROM->setText(QCoreApplication::translate("UISettings", "...", nullptr));
        twPages->setTabText(twPages->indexOf(wCartMemory), QCoreApplication::translate("UISettings", "Cart/Memory", nullptr));
        lInput->setText(QCoreApplication::translate("UISettings", "Input", nullptr));
        pmPort1->setTitle(QCoreApplication::translate("UISettings", "Port 1", nullptr));
        pmPort2->setTitle(QCoreApplication::translate("UISettings", "Port 2", nullptr));
        lGunMouseSensitivity->setText(QCoreApplication::translate("UISettings", "Gun/Mouse Sensitivity", nullptr));
        twPages->setTabText(twPages->indexOf(wInput), QCoreApplication::translate("UISettings", "Input", nullptr));
        lRegion->setText(QCoreApplication::translate("UISettings", "Region", nullptr));
        lSH2Interpreter->setText(QCoreApplication::translate("UISettings", "SH2 Interpreter", nullptr));
        l68kCore->setText(QCoreApplication::translate("UISettings", "68000 Core", nullptr));
        lSH1Interpreter->setText(QCoreApplication::translate("UISettings", "SH1 Interpreter", nullptr));
        cbEnableCDBlockLLE->setText(QCoreApplication::translate("UISettings", "Enable CD Block Low-level Emulation", nullptr));
        cbEnableSh2DmaTiming->setText(QCoreApplication::translate("UISettings", "Enable Sh2 Dma Timing", nullptr));
        cbEnableScuDmaTiming->setText(QCoreApplication::translate("UISettings", "Enable Scu Dma Timing", nullptr));
        cbEnableSh2Cache->setText(QCoreApplication::translate("UISettings", "Enable Sh2 Cache", nullptr));
        label_3->setText(QCoreApplication::translate("UISettings", "(CD Block LLE requires cdb105.bin, a real BIOS, and no dynamic recompilers. Sh2 Dma Timing, Scu Dma Timing and Sh2 Cache must also be enabled)", nullptr));
        twPages->setTabText(twPages->indexOf(wAdvanced), QCoreApplication::translate("UISettings", "Advanced", nullptr));
        lHideMenubar->setText(QCoreApplication::translate("UISettings", "Hide Menubar", nullptr));
        rbMenubarNever->setText(QCoreApplication::translate("UISettings", "Never", nullptr));
        rbMenubarFullscreen->setText(QCoreApplication::translate("UISettings", "On fullscreen", nullptr));
        rbMenubarAlways->setText(QCoreApplication::translate("UISettings", "Always", nullptr));
        lHideToolbar->setText(QCoreApplication::translate("UISettings", "Hide Toolbar", nullptr));
        rbToolbarNever->setText(QCoreApplication::translate("UISettings", "Never", nullptr));
        rbToolbarFullscreen->setText(QCoreApplication::translate("UISettings", "On fullscreen", nullptr));
        rbToolbarAlways->setText(QCoreApplication::translate("UISettings", "Always", nullptr));
        label->setText(QCoreApplication::translate("UISettings", "Show Log Window", nullptr));
        rbLogWindowMessage->setText(QCoreApplication::translate("UISettings", "On message", nullptr));
        rbMenubarFullscreenHover->setText(QCoreApplication::translate("UISettings", "Display on Hover", nullptr));
        rbLogWindowNever->setText(QCoreApplication::translate("UISettings", "Never", nullptr));
        twPages->setTabText(twPages->indexOf(wView), QCoreApplication::translate("UISettings", "View", nullptr));
        QTableWidgetItem *___qtablewidgetitem = twShortcuts->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("UISettings", "Description", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = twShortcuts->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("UISettings", "Shortcut", nullptr));
        twPages->setTabText(twPages->indexOf(tab), QCoreApplication::translate("UISettings", "Shortcuts", nullptr));
    } // retranslateUi

};

namespace Ui {
    class UISettings: public Ui_UISettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UISETTINGS_H
